Package: agricolaeplotr
Type: Package
Title: Visualization of Design of Experiments from the 'agricolae'
        Package
Version: 1.0.0
Authors@R: 
    person(family="Harbers", given="Jens", email="jensharbers@gmail.com", role=c("aut","cre"),
           comment = c(ORCID = "0000-0001-6634-623X"))
Maintainer: Jens Harbers <jensharbers@gmail.com>
Description: Visualization of Design of Experiments from the 'agricolae' package with 'ggplot2' framework
    The user provides an experiment design from the 'agricolae' package, calls the corresponding function and will receive a 
    visualization with 'ggplot2' based functions that are specific for each design. As there are many different designs, each design is tested on its type.
    The output can be modified with standard 'ggplot2' commands or with other packages with 'ggplot2' function extensions.
License: GPL (>= 3)
Encoding: UTF-8
Imports: ggplot2, agricolae, raster, sp (>= 2.0.0), methods, FielDHub,
        utils, tibble, sf, dplyr, tidyr, stringr, stplanr, ggspatial
Language: en-US
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, leaflet
Config/testthat/edition: 3
BugReports: https://github.com/jensharbers/agricolaeplotr/issues
URL: https://github.com/jensharbers/agricolaeplotr
Depends: R (>= 4.0)
VignetteBuilder: knitr
Note: 'None'
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-10 20:15:48 UTC; JensH
Author: Jens Harbers [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6634-623X>)
Repository: CRAN
Date/Publication: 2026-05-11 05:00:02 UTC
Built: R 4.5.2; ; 2026-05-11 08:04:22 UTC; unix

Package: agricolaeplotr
Type: Package
Title: Visualization of Design of Experiments from the 'agricolae'
        Package
Version: 1.0.0
Authors@R: 
    person(family="Harbers", given="Jens", email="jensharbers@gmail.com", role=c("aut","cre"),
           comment = c(ORCID = "0000-0001-6634-623X"))
Maintainer: Jens Harbers <jensharbers@gmail.com>
Description: Visualization of Design of Experiments from the 'agricolae' package with 'ggplot2' framework
    The user provides an experiment design from the 'agricolae' package, calls the corresponding function and will receive a 
    visualization with 'ggplot2' based functions that are specific for each design. As there are many different designs, each design is tested on its type.
    The output can be modified with standard 'ggplot2' commands or with other packages with 'ggplot2' function extensions.
License: GPL (>= 3)
Encoding: UTF-8
Imports: ggplot2, agricolae, raster, sp (>= 2.0.0), methods, FielDHub,
        utils, tibble, sf, dplyr, tidyr, stringr, stplanr, ggspatial
Language: en-US
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, leaflet
Config/testthat/edition: 3
BugReports: https://github.com/jensharbers/agricolaeplotr/issues
URL: https://github.com/jensharbers/agricolaeplotr
Depends: R (>= 4.0)
VignetteBuilder: knitr
Note: 'None'
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-10 20:15:48 UTC; JensH
Author: Jens Harbers [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6634-623X>)
Repository: CRAN
Date/Publication: 2026-05-11 05:00:02 UTC
Built: R 4.6.0; ; 2026-05-11 05:45:50 UTC; unix

# Generated by roxygen2: do not edit by hand

export(DOE_obj)
export(citations)
export(full_control_positions)
export(make_polygons)
export(plot_alpha)
export(plot_bib)
export(plot_cyclic)
export(plot_dau)
export(plot_design.factorial_crd)
export(plot_design.factorial_lsd)
export(plot_design.factorial_rcbd)
export(plot_design_crd)
export(plot_fieldhub)
export(plot_graeco)
export(plot_latin_square)
export(plot_lattice_simple)
export(plot_lattice_triple)
export(plot_longest_diagonal)
export(plot_rcbd)
export(plot_split_crd)
export(plot_split_lsd)
export(plot_split_rcbd)
export(plot_strip)
export(plot_youden)
export(protective_layers)
export(sample_locations)
export(serpentine)
export(summary)
export(test_input_extend)
export(test_input_ncols)
export(test_input_nrows)
export(test_input_reverse)
export(test_input_shift)
export(test_name_in_column)
export(test_names_design)
export(test_string)
export(theme_gi)
export(theme_poster)
export(theme_pres)
export(to_table)
import(FielDHub)
import(agricolae)
import(ggplot2)
import(sf)
importFrom(dplyr,`%>%`)
importFrom(dplyr,filter)
importFrom(ggplot2,geom_sf)
importFrom(ggplot2,ggplot)
importFrom(ggspatial,annotation_north_arrow)
importFrom(ggspatial,annotation_scale)
importFrom(raster,crs)
importFrom(raster,extent)
importFrom(sf,st_sample)
importFrom(stats,dist)
importFrom(stplanr,line_segment)
importFrom(stringr,str_wrap)
importFrom(tibble,as_tibble)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,gather)

---
output: github_document
---
## Installation of the package
Use the following command to install the package from CRAN:

``` R
install.packages("agricolaeplotr")
```

# Usage of the Package

This section demonstrates the usage of the package and its underlying functions. Factorial experiments are ubiquitous in all science and technology fields, and as an example, a factorial AB-design will be used. While some parameters are specifically relevant to agriculture, most others are beneficial for every user.

## Load the Package

Use the following command to load the package after installation. The two packages below 'agricolaeplotr' are only needed for the examples.


``` R

library("agricolaeplotr")

library("ggplot2")

library("agricolae")

```

## Example: Factorial AB Design with Complete Randomization

To create a design, we first utilize the `agricolae` package. All examples provided are directly sourced from `agricolae`.

After creating the object, everything is set to plot a basic graph. It is assumed that the height and width of each plot are both set to 1. In agricultural designs, it is recommended to input the measures from a plot to estimate the dimensions needed for implementing such an experiment in the field. Knowing the required dimensions in meters or other units is crucial for machinery and experiment management.

Complete randomized designs lack a factor like blocks, requiring the user to input suitable numbers for columns and rows. The product of these numbers must be greater than the size of the experiment, allowing the program to place all plots.

The following figure illustrates the output of a factorial design with two factors. The first factor has three levels, and the second one has two. The output is a standard ggplot2 design. This implies that users can apply all operations that ggplot2 and other packages using ggplot2 functions can offer. There are no layer restrictions or overly specialized layers preventing other transformations. Additionally, users may leverage 'plotly' to create interactive visualizations of the designs. This is particularly useful for field demonstrations involving various project stakeholders such as scientists, farmers, and funding agencies.

``` R
library(agricolae) # origin of the needed design object
trt <- c(3, 2) # factorial 3x2
outdesign <- design.ab(trt, r = 3, serie = 2, design = 'crd')

head(outdesign$book, 10)

plot_design.factorial_crd(outdesign, ncols = 6, nrows = 3, width = 1, height = 1)


```
![factorial design](C:\Users\Jens Harbers\Documents\RPlot.jpeg)


## Planned Features for Future Versions

- Introduce a Shiny interface for interactive experiment layout.
- Incorporate additional custom field experiment tools.
- Enable the export of experiments to the ISOBUS standard.
- Implement the export of designs to PostgreSQL.

library(shiny)
library(agricolae)
library(ggplot2)
library(DBI)
library(sf)
library(leaflet)
library(RPostgres)
library(agricolaeplotr)
library(jsonlite)
library(sp)
library(DT)
library(qrcode)
library(jsonify)
library(shinycssloaders)
#cit.list <- c("base", names(utils::sessionInfo()$otherPkgs))
#for(i in 1:length(cit.list)){
# print(c(cit.list[i],packageDescription(cit.list[i], fields=c("License","Version"))))
#}

# generate_qr_labels <- function(input_df, output_file="labelscorrected",experiment_name="My Experiment", img_width = 65, img_height = 32, page_width = 210, page_height = 297) {
#   # Define layout constants
#   cols <- floor(page_width / img_width)  # Number of columns
#   rows <- floor(page_height / img_height)  # Number of rows
#
#
#   total_images <- nrow(input_df)
#   images_per_page <- cols*rows
#
#   num_pages <- ceiling(total_images / images_per_page)
#
#   input_df$trial <- experiment_name
#
#   input_df <- data.frame(lapply(input_df,as.factor))
#   # Convert mm to inches
#   mm_to_inches <- function(mm) {
#     return(mm / 25.4)
#   }
#  print("Here")
#   for (page in seq_len(num_pages)) {
#     start_idx <- (page - 1) * images_per_page + 1
#     end_idx <- min(page * images_per_page, total_images)
#
#     # Set up plotting device (e.g., PDF or PNG)
#     file_name <- sprintf(paste0(output_file,"_%02d.pdf"), page)  # Auto-numerated filename
#
#     # Open a PDF to save output
#     pdf(file_name, width = mm_to_inches(page_width), height = mm_to_inches(page_height))
#
#     # Set up a blank plot for the whole A4 page
#     par(mar = c(0, 0, 0, 0))  # Remove margins
#     plot(1, type = "n", axes = FALSE, xlab = "", ylab = "", xlim = c(0, page_width), ylim = c(0, page_height))
#
#     df_chunk <- input_df[c(start_idx:end_idx),]
#     # Loop through data and draw labels in a grid
#     for (i in 1:nrow(df_chunk)) {
#       # Calculate row and column position
#       row <- ((i - 1) %% rows) + 1
#       col <- ((i - 1) %/% rows) + 1
#
#       # Convert to plot coordinates (in mm)
#       x_left <- (col - 1) * img_width
#       x_right <- x_left + img_width
#       y_bottom <- page_height - (row * img_height)
#       y_top <- y_bottom + img_height
#
#       # Draw a rectangle for the label boundary
#       rect(x_left, y_bottom, x_right, y_top, border = "white", lwd = 1)
#       # Generate QR code
#       qr_text <- jsonify::to_json(df_chunk[i,])
#       qr <- 1- qrcode::qr_code(qr_text, ecl = "Q")
#
#       # Draw QR code on the left side
#       qr_x_left <- x_left + 0
#       qr_x_right <- qr_x_left + 26
#       qr_y_bottom <- y_bottom + 0
#       qr_y_top <- qr_y_bottom + 26
#       rasterImage(qr, qr_x_left, qr_y_bottom, qr_x_right, qr_y_top)
#
#       # Add text on the right side
#       text_x <- x_right - 7  # Align text to the right
#       text_y_top <- y_top - 8
#
#       line_spacing <- 5
#
#       for (j in seq_along(df_chunk)) {
#         text_y_position <- text_y_top - (j - 1) * line_spacing  # Adjust vertical spacing
#         text(text_x, text_y_position, paste(colnames(df_chunk)[j], ":", df_chunk[i, j]),
#              cex = 1.2, font = 2, adj = 1)
#       }
#     }
#
#     # Close PDF output
#     dev.off()
#   }
#
#   files <- Sys.glob(paste0(output_file,"*.pdf"))
#   qpdf::pdf_combine(input = files, output = paste0(output_file,".pdf"))
# }



ui <- fluidPage(
  tags$head(
    tags$link(rel = "icon", type = "image/png", sizes = "48x48", href = "Logo.png"),
    tags$script(
      '


@media (max-width: 1999px) {
  .sidepanel {
    float: none;
    width: 100%;
  }

  .maincontent {
    float: none;
    width: 100%;
  }
};
      '
    ),

    tags$style(HTML("
      .nav-tabs {
        border-bottom: 2px solid #ddd;
      }
      .nav-tabs > li > a {
        background-color: #FFFFFF;
        color: black;
        font-size: 18px;
        padding: 15px 30px;
        border-color: #000000;
        border-radius: 1px;
        transition: background-color 0.3s ease;
        margin-right: 1px;
        box-shadow: 0 1px 2px rgba(0, 0, 0, 0.1);
      }
      .nav-tabs > li > a:hover {
        background-color: #E57738;
      }
      .nav-tabs > li.active > a,
      .nav-tabs > li.active > a:hover,
      .nav-tabs > li.active > a:focus {
        background-color: #FFFFFF;
        color: black;
        border: none;
        outline: none;
        box-shadow: 0 0 0 3px rgba(0, 123, 255, 0.5);
      }
      .tab-content {
        padding: 20px;
        border: 1px solid #ddd;
        border-radius: 8px;
        box-shadow: 0 4px 6px rgba(0, 0, 0, 0.1);
      }
      .btn-custom {
        background-color: #FFFFFF;
        color: black;
        font-size: 16px;
        padding: 10px 20px;
        border-color: black;
        border-radius: 5px;
        transition: background-color 0.3s ease;
        margin: 2px;
      }
      .btn-custom:hover {
        background-color: #000000;
        color: white;
      }
      .btn-custom:focus {
      background.color: #000000;
      color: white;
      text-decoration: underline;
      outline: none;
      }

      .input-label {
        font-weight: bold;
      }

      .logout-button {
          top: 5px;
          right: 5px;
          padding: 8px 16px;
          font-size: 12px;
      }

      .nav-tabs > li > a {
        background-color: #FFFFFF;
        color: black;
        font-size: 16px;
        padding: 10px 20px;
        border-color: black;
        border-radius: 5px;
        transition: background-color 0.3s ease;
        margin: 2px;
      }
      .nav-tabs > li.active > a {
        background-color: #000000;
        color: white;
      }

      .nav-tabs > li.active > a:hover {
        background-color: #000000;
        color: white;
      }

      .nav-tabs > li.active > a:visited {
        background-color: #d3d3d3;
        color: black;
      }

    "))
  ),

  HTML('<meta name="viewport" content="width=device-width, initial-scale=1">'),
  div(
    class = "top-header",
    style = "position: relative; height: 60px;",
    #title=div(img(src='Logo.png', style="width: 120px; height: 50px;"))
    titlePanel("Field Experiment Designer", windowTitle = "Field Experiment Designer" ),

  ),
  # auth0::logoutButton("Log Out",style = "position: absolute; top: 5px; right: 5px; z-index:10000;", class="btn-custom", icon("right-from-bracket")),


  sidebarLayout(
    sidebarPanel(tags$style(".well {background-color: #FFFFFF; border-color: black}"),
                 conditionalPanel(
                   condition = "input.tabs_a=='1'",
                   tags$label("Upload a JSON file containing previous experimental designs", class="input-label"),
                   fileInput("file", "Upload JSON File", accept = ".json"),
                   tags$label("Choose your planned experimental design", class="input-label"),
                   selectInput(
                     "design",
                     NULL,
                     choices = c(
                       "Complete Factorial Design" = "design.ab" ,
                       "Split Plot Design" = "design.split",
                       "Alpha design type (0,1)" = "design.alpha",
                       "Randomized Balanced Incomplete Block Designs (BIB)" = "design.bib",
                       "Completely Randomized Design" = "design.crd",
                       "Cyclic designs" = "design.cyclic",
                       "Augmented block design" = "design.dau",
                       "Graeco - latin square design" = "design.graeco",
                       "Lattice designs" = "design.lattice",
                       "Latin Square Design" = "design.lsd",
                       "Randomized Complete Block Design" = "design.rcbd",
                       "Strip Plot Design" = "design.strip",
                       "Youden Plot Design" = "design.youden",
                       "Special Design" = "custom_spec"
                     ),
                     selected = "design.ab"
                   ),
                   conditionalPanel("input.design !='custom_spec'",
                   tags$label("Choose a seed number (automatic generated)", class="input-label"),
                   numericInput(
                     "seed",
                     NULL,
                     value = sample(0:2147483647, 1),
                     min = 0,
                     max = 2147483647
                   )),
                   conditionalPanel(
                     condition = "input.design == 'design.ab'",
                     tags$label("Choose the number of factors", class="input-label"),
                     numericInput(
                       "numIndividuals",
                       NULL,
                       value = 3,
                       min = 2,
                       max = 5
                     ),
                     uiOutput("fields")
                   ),
                   conditionalPanel(
                     condition = "input.design == 'design.ab' || input.design == 'design.split'",
                     tags$label("Choose your factorial design", class="input-label"),
                     selectInput(
                       "factorial_design",
                       NULL,
                       choices = c("rcbd", "crd", "lsd"),
                       selected = "crd"
                     )
                   ),
                   conditionalPanel("input.design !='custom_spec'",
                   tags$label("Choose a Method to randomize", class="input-label"),
                   selectInput(
                     "kind",
                     NULL,
                     choices = c(
                       "Wichmann-Hill",
                       "Marsaglia-Multicarry",
                       "Super-Duper",
                       "Mersenne-Twister",
                       "Knuth-TAOCP",
                       "Knuth-TAOCP-2002",
                       "default"
                     ))
                   ),
                   conditionalPanel(
                     condition = "input.design != 'design.graeco' && input.design != 'design.lsd' && input.design != 'design.lattice' && input.design != 'custom_spec'" ,
                     tags$label("Number of replications", class="input-label"),
                     numericInput(
                       "r",
                       NULL,
                       value = 3,
                       min = 1,
                       step = 1
                     )
                   ),
                   conditionalPanel(
                     condition = "input.design == 'design.alpha' || input.design == 'design.bib' || input.design == 'design.cyclic'",
                     tags$label("Number of Blocks", class="input-label"),
                     numericInput("k", NULL, value = 2, min = 1)
                   ),
                   conditionalPanel(
                     condition = "input.design == 'design.bib'",
                     tags$label("Maximum replications", class="input-label"),
                     numericInput(
                       "maxrep",
                       NULL,
                       value = 20,
                       min = 1,
                       step = 1
                     )
                   ),
                   conditionalPanel(
                     condition = "input.design == 'design.lattice'",
                     checkboxInput(
                       "simple",
                       "Lattice kind (unchecked = triple, checked = simple)",
                       value = FALSE
                     )
                   ),
                    conditionalPanel("input.design !='custom_spec'",tags$label("Choose Leading Numeration of plots", class="input-label"),
                                                        selectInput("serie", NULL, choices = c(1, 2, 3)),
                                                        checkboxInput("randomization", "Randomization", value = TRUE))
,
                   conditionalPanel(
                     condition = "input.design == 'design.dau' || input.design == 'design.graeco' || input.design == 'design.split' || input.design == 'design.strip'",
                     uiOutput("trt_duo")
                   ),
                   conditionalPanel(
                     condition = "input.design == 'design.split' || input.design == 'design.youden' || input.design == 'design.lsd' || input.design == 'design.rcbd'",
                     checkboxInput("first", "Randomize first replication", value = TRUE)
                   ),
                   conditionalPanel(
                     condition = "input.design == 'design.alpha' || input.design == 'design.bib' || input.design == 'design.crd' || input.design == 'design.lattice' || input.design == 'design.lsd' || input.design == 'design.cyclic' || input.design == 'design.rcbd' || input.design == 'design.youden'",
                     uiOutput("trt_single")
                   ),
                   conditionalPanel(
                     condition = "input.design == 'design.crd'",
                     checkboxInput("continue", "Continuous numbering of plot", value = FALSE)
                   ),
                   conditionalPanel(
                     condition = "input.design == 'design.cyclic'",
                     checkboxInput("rowcol", "TRUE: row-column design", value = FALSE)
                   ),
                   conditionalPanel(
                     condition = "input.design == 'custom_spec'",
                     numericInput("seed","Seed",213463,0,149324847),
                     numericInput("size","Size",2,1,149324847)
                   )

                 ),
                 conditionalPanel(
                   condition = "input.tabs_a == '2' || input.tabs_a == '3'",
                   tags$label("Width [m]", class="input-label"),
                   numericInput("width", NULL, value = 2, min = 0),
                   tags$label("Height [m]", class="input-label"),
                   numericInput("height", NULL, value = 2, min = 0),
                   tags$label("Relative Space Width [%]", class="input-label"),
                   numericInput("space_width", NULL, value = 0.05, min = 0, max = 1, step = 0.05),
                   tags$label("Relative Space Height [%]", class="input-label"),
                   numericInput("space_height", NULL, value = 0.1, min = 0, max = 1, step = 0.05),
                   conditionalPanel(
                     condition = "(input.design == 'design.split' && input.factorial_design == 'crd') || (input.design == 'design.ab' && input.factorial_design == 'crd') || input.design == 'design.crd'",
                     tags$label("Number of Columns", class="input-label"),
                     numericInput("ncols", NULL, value = 5, min = 1)
                   ),
                   conditionalPanel(
                     condition = "input.design == 'design.ab'",
                     uiOutput("choosen_factor")
                   ),
                   conditionalPanel(condition = "input.tabs_a == '2'",
                   tags$label("Northing (EPSG:31467)", class="input-label"),
                   numericInput("northing", NULL, value = 5939183.21, min = 0),
                   tags$label("Easting (EPSG:31467)", class="input-label"),
                   numericInput("easting", NULL, value = 3454206.89, min = 0),
                   tags$label("Rotation in Degree", class="input-label"),
                   numericInput("rotation", NULL, value = 0, min = -180, max = 180)),
                   checkboxInput("reverse_x", "Revert X Axis", value = FALSE),
                   checkboxInput("reverse_y", "Revert Y Axis", value = FALSE),
                   selectInput("colorOption","Color Option", choices = LETTERS[1:8], selected="D"),
                   selectInput("background","Background Color", choices = c("white","yellow","blue","green"), selected="white"),
                   conditionalPanel(
                     condition = "input.design == 'design.ab'",
                     uiOutput("choose_factor")
                   )
                 ),
                 conditionalPanel(
                   condition = "input.design == 'design.split' && (input.tabs_a == '2' || input.tabs_a == '3')",
                   checkboxInput("swap_factor", "Plot Split Factor", value = TRUE)
                 ),
                 conditionalPanel(
                   condition = "input.tabs_a=='3'",
                   fluidPage(
                     downloadButton(
                       'downloadpdf',
                       'Download PDF of the current design',
                       class = "btn-custom",
                       icon = icon("file-pdf")
                     )
                   )
                 ),

                 conditionalPanel(
                   condition = "input.tabs_a == '2'",
                   checkboxInput("rowcol", "TRUE: row-column design", value = FALSE),
                   tags$label("Which Layer Do You Prefer?", class="input-label"),
                   selectInput("maplayer", NULL, choices = c("OpenStreetMap.DE", "Esri.WorldImagery"))
                 ),
                 conditionalPanel(
                   condition = "input.tabs_a == 'database'",
                   downloadButton('downloadData', 'Download Data', class = "btn-custom", icon = icon("table")),
                   downloadButton('downloadParams', 'Download Model Parameter', class = "btn-custom", icon = icon("cog")),
                   downloadButton("exportgeojson", label = "Download GEOJSON", icon = icon("globe"), class = "btn-custom"),
                   tags$hr(),
                   tags$label("Username", class="input-label"),
                   textInput("user", NULL, value = "username"),
                   tags$label("Password", class="input-label"),
                   passwordInput("password", NULL, value = "password"),
                   tags$label("Host of DB", class="input-label"),
                   textInput("host", NULL, value = "127.0.0.1"),
                   tags$label("Port for Connection", class="input-label"),
                   numericInput("port", NULL, value = 7432, min = 0, max = 100000, step = 1),
                   tags$label("DB Name for Connection", class="input-label"),
                   textInput("dbname", NULL, value = "postgresDB"),
                   tags$label("DB Table Name or Filename", class="input-label"),
                   textInput("tablename", NULL, value = "experiment_data"),
                   actionButton('saveSQL', 'Send Design to Postgres', class = "btn-custom", icon = icon("database")),
                   # downloadButton('ISOBUS', 'Send Design to ISOBUS', class = "btn-custom", icon = icon("tractor"))
                 ),
                 conditionalPanel(
                   condition = "input.tabs_a == 'cloud'",
                   tags$label("Username", class="input-label"),
                   textInput("user", NULL, value = "username"),
                   tags$label("Password", class="input-label"),
                   passwordInput("password", NULL, value = "password"),
                   tags$label("Host of DB", class="input-label"),
                   textInput("host", NULL, value = "127.0.0.1"),
                   tags$label("Port for Connection", class="input-label"),
                   numericInput("port", NULL, value = 7432, min = 0, max = 100000, step = 1),
                   tags$label("DB Name for Connection", class="input-label"),
                   textInput("dbname", NULL, value = "postgresDB"),
                   tags$label("DB Table Name or Filename", class="input-label"),
                   textInput("tablename", NULL, value = "experiment_data"),
                   actionButton('cloudUpload', 'Send Design to Cloud Database', class = "btn-custom", icon = icon("cloud"))
                 ),
                conditionalPanel(condition = "input.tabs_a=='summary'",
                 selectInput("experimentparts","Focus of Summary",choices=c("Net Plot"="net_plot","Gross Plot"="gross_plot","Field"="field","Experiment"="experiment","Full"="all"), selected ="all"),
                 numericInput("digits", "Number of Significant Digits:", value = 3, min = 0)
                 ),
                 # conditionalPanel(
                 #   condition = "input.tabs_a == 'sampleLabels'",
                 #   textInput("experiment_name", "Experiment Name:", value = "My Experiment"),
                 #   textInput("output_file", "Output File Name:", value = "labelscorrected"),
                 #   numericInput("img_width", "Image Width:", value = 65, min = 1),
                 #   numericInput("img_height", "Image Height:", value = 32, min = 1),
                 #   numericInput("page_width", "Page Width:", value = 210, min = 1),
                 #   numericInput("page_height", "Page Height:", value = 297, min = 1),
                 #   downloadButton('makeLabels', 'Create Labels', class = "btn-custom", icon = icon("qrcode"))),

                 width = 3
    ),
    mainPanel(
      tabsetPanel(
        type = "tabs", id = "tabs_a",
        tabPanel("Create Experiment", DTOutput("oid1") %>% withSpinner(color="black"), value = "1",icon = icon("gear")),
        tabPanel("Map Plot", plotOutput("doe_plot_map") %>% withSpinner(color="black") , value = "3",icon = icon("map")),
        tabPanel("Experiment Plot", leafletOutput("doe_plot") %>% withSpinner(color="black") , value = "2",icon = icon("location")),
        tabPanel("Data Export", verbatimTextOutput("sql_db"), value = "database", icon = icon("download")),
        tabPanel("Cloud", value = "cloud", icon = icon("cloud")),
        tabPanel("Flightmap", value = "flightmap", icon = icon("plane"),
                 tags$iframe(src = "https://maptool-dipul.dfs.de/?language=de",allow="geolocation",
                             width = "100%", height = "600px", frameborder = "0")%>% withSpinner(color="black")),
        tabPanel("Summary",DTOutput("oid2") %>% withSpinner(color="black"),value="summary",icon=icon("clipboard")),
        tabPanel("About",value="about",icon=icon("info-circle")),
        #tabPanel("Label",value="sampleLabels",icon=icon("qrcode"))
        #tabPanel("Nature Conservation", value = "flightmap", icon = icon("frog",style = "color: rgb(255,0,0)"),
        #         tags$iframe(src = "https://maptool-dipul.dfs.de/?language=de",allow="geolocation",
        #                     width = "100%", height = "600px", frameborder = "0"))


      ),
      tags$script(HTML(
        "$(document).on('keydown', function(e) {
      if (e.altKey) {
        switch(e.key) {
          case '1': Shiny.setInputValue('nav_key', '1', {priority: 'event'}); break;
          case '3': Shiny.setInputValue('nav_key', '2', {priority: 'event'}); break;
          case '2': Shiny.setInputValue('nav_key', '3', {priority: 'event'}); break;
          case '4': Shiny.setInputValue('nav_key', 'database', {priority: 'event'}); break;
          case '5': Shiny.setInputValue('nav_key', 'cloud', {priority: 'event'}); break;
          case '6': Shiny.setInputValue('nav_key', 'flightmap', {priority: 'event'}); break;
          case '7': Shiny.setInputValue('nav_key', 'summary', {priority: 'event'}); break;
          case '8': Shiny.setInputValue('nav_key', 'about', {priority: 'event'}); break;
        }
      }
    });"
      ))
    )
  )



)

# Define server logic to summarize and view selected data set ----

server <- function(session, input, output) {

  observeEvent(input$nav_key, {
    updateTabsetPanel(session, "tabs_a", selected = input$nav_key)
  })


  observeEvent(input$file, {
    req(input$file)
    json_data <- fromJSON(input$file$datapath)
    updateSelectInput(session, "design", selected = json_data$design)
    updateNumericInput(session, "numIndividuals", value = json_data$numIndividuals)
    updateNumericInput(session, "r", value = unique(json_data$r))
    updateNumericInput(session, "serie", value = json_data$serie)
    updateNumericInput(session, "seed", value = json_data$seed)
    updateSelectInput(session, "kind",  selected = json_data$kind)
    updateNumericInput(session, "northing", value = json_data$northing)
    updateNumericInput(session, "easting", value = json_data$easting)
    updateNumericInput(session, "rotation", value = json_data$rotation)
    updateCheckboxInput(session, "first", value = json_data$first)
    updateCheckboxInput(session, "randomization", value = json_data$randomization)
    updateNumericInput(session, "k", value = json_data$k)
    updateNumericInput(session, "maxrep", value = json_data$maxrep)
    updateSelectizeInput(session, "treatment_single", choices = json_data$trt, selected = json_data$trt)
    updateSelectizeInput(session, "treatment_11", choices = json_data$trt1, selected = json_data$trt1)
    updateSelectizeInput(session, "treatment_12", choices = json_data$trt, selected = json_data$trt)
    updateSelectizeInput(session, "treatment1", choices = json_data$treatment1, selected = json_data$treatment1)
    updateSelectizeInput(session, "treatment2", choices = json_data$treatment2, selected = json_data$treatment2)
    updateSelectizeInput(session, "treatment3", choices = json_data$treatment3, selected = json_data$treatment3)
    updateSelectizeInput(session, "treatment4", choices = json_data$treatment4, selected = json_data$treatment4)
    updateSelectizeInput(session, "treatment5", choices = json_data$treatment5, selected = json_data$treatment5)
    updateNumericInput(session, "width", value = json_data$width)
    updateNumericInput(session, "height", value = json_data$height)
    updateNumericInput(session, "space_width", value = json_data$space_width)
    updateNumericInput(session, "space_height", value = json_data$space_height)
    updateCheckboxInput(session, "reverse_y", value = json_data$reverse_y)
    updateCheckboxInput(session, "reverse_x", value = json_data$reverse_x)
    updateCheckboxInput(session, "rowcol", value = json_data$rowcol)
    updateCheckboxInput(session, "colorOption", value = json_data$colorOption)
    updateCheckboxInput(session, "background", value = json_data$background)

  })

 #### GPS Server stuff goes here

  coords <- reactive({
    input$gps_coords
  })

  transform_coords <- reactive({
    if (!is.null(coords())) {
      wgs84 <- st_sfc(st_point(c(coords()$lon, coords()$lat)), crs = 4326)
      utm <- st_transform(wgs84, crs = 31467) # UTM Zone 33N
      utm_coords <- st_coordinates(utm)
      list(easting = utm_coords[2], northing = utm_coords[1])
    } else {
      list(easting = 0, northing = 0)
    }
  })

  observe({
    if (!is.null(coords())) {
      utm <- transform_coords()
      updateNumericInput(session, "easting", value = utm$easting)
      updateNumericInput(session, "northing", value = utm$northing)
    }
  })

  outdesign <- reactive({
    if((input$design == "design.ab") && (input$factorial_design == "crd")){
      trt <- switch(input$numIndividuals,
                    c(length(input$treatment1)),
                    c(length(input$treatment1),
                      length(input$treatment2)),
                    c(length(input$treatment1),
                      length(input$treatment2),
                      length(input$treatment3)),
                    c(length(input$treatment1),
                      length(input$treatment2),
                      length(input$treatment3),
                      length(input$treatment4)),
                    c(length(input$treatment1),
                      length(input$treatment2),
                      length(input$treatment3),
                      length(input$treatment4),
                      length(input$treatment5))
      )

      rst <- design.ab(
        trt,
        r = input$r,
        serie = as.numeric(input$serie),
        design = input$factorial_design,
        kinds = input$kind,
        seed = input$seed)

      endnumber <- dim(rst$book)[2]

      for( i in 3:endnumber){

        rst$book[,i] <- factor(rst$book[,i], labels=input[[paste0("treatment",i-2)]])
      }
      return(rst)
    }

    if((input$design == "design.ab") && (input$factorial_design == "lsd")){
      trt <- switch(input$numIndividuals,
                    c(length(input$treatment1)),
                    c(length(input$treatment1),
                      length(input$treatment2)),
                    c(length(input$treatment1),
                      length(input$treatment2),
                      length(input$treatment3)),
                    c(length(input$treatment1),
                      length(input$treatment2),
                      length(input$treatment3),
                      length(input$treatment4)),
                    c(length(input$treatment1),
                      length(input$treatment2),
                      length(input$treatment3),
                      length(input$treatment4),
                      length(input$treatment5))
      )

      rst <- design.ab(
        trt,
        r = input$r,
        serie = as.numeric(input$serie),
        design = input$factorial_design,
        kinds = input$kind,
        seed = input$seed)

      endnumber <- dim(rst$book)[2]

      for( i in 4:endnumber){

        rst$book[,i] <- factor(rst$book[,i], labels=input[[paste0("treatment",i-3)]])
      }
      return(rst)
    }

    if((input$design == "design.ab") && (input$factorial_design == "rcbd")){
      trt <- switch(input$numIndividuals,
                    c(length(input$treatment1)),
                    c(length(input$treatment1),
                      length(input$treatment2)),
                    c(length(input$treatment1),
                      length(input$treatment2),
                      length(input$treatment3)),
                    c(length(input$treatment1),
                      length(input$treatment2),
                      length(input$treatment3),
                      length(input$treatment4)),
                    c(length(input$treatment1),
                      length(input$treatment2),
                      length(input$treatment3),
                      length(input$treatment4),
                      length(input$treatment5))
      )
      rst <- design.ab(
        trt=trt,
        r = input$r,
        serie = as.numeric(input$serie),
        design = input$factorial_design,
        kinds = input$kind,
        seed = input$seed,
        first = input$first,
        randomization = input$randomization)

      endnumber <- dim(rst$book)[2]

      for( i in 3:endnumber){

        rst$book[,i] <- factor(rst$book[,i], labels=input[[paste0("treatment",i-2)]])
      }

      return(rst)
    }

    if(input$design == "design.youden"){
      trt <- input$treatment_single
      rst2 <- design.youden(trt = trt,
                            r = input$r,
                            serie = as.numeric(input$serie),
                            kinds = input$kind,
                            seed = input$seed,
                            first = input$first,
                            randomization = input$randomization)
      return(rst2)
    }

    if(input$design == "design.crd"){
      trt <- input$treatment_single
      rst3 <- design.crd(trt = trt,
                         r = input$r,
                         serie = as.numeric(input$serie),
                         kinds = input$kind,
                         seed = input$seed,
                         randomization = input$randomization)
      return(rst3)
    }

    if(input$design == "design.rcbd"){
      trt <- input$treatment_single
      rst3 <- design.rcbd(trt = trt,
                          r = input$r,
                          serie = as.numeric(input$serie),
                          kinds = input$kind,
                          seed = input$seed,
                          first = input$first,
                          continue = input$continue,
                          randomization = input$randomization)
      return(rst3)
    }

    if(input$design == "design.lsd"){
      trt <- input$treatment_single
      rst3 <- design.lsd(trt = trt,
                         serie = as.numeric(input$serie),
                         kinds = input$kind,
                         seed = input$seed,
                         randomization = input$randomization,
                         first = input$first)
      return(rst3)
    }


    if(input$design == "design.dau"){
      trt1 <- input$treatment_12
      trt2 <- input$treatment_11
      rst4 <- design.dau(trt1 = trt1,
                         trt2 = trt2,
                         r = input$r,
                         serie = as.numeric(input$serie),
                         kinds = input$kind,
                         seed = input$seed,
                         randomization = input$randomization)
      return(rst4)
    }

    if(input$design == "design.strip"){
      trt1 <- input$treatment_11
      trt2 <- input$treatment_12
      rst4 <- design.strip(trt1 = trt1,
                           trt2 = trt2,
                           r = input$r,
                           serie = as.numeric(input$serie),
                           kinds = input$kind,
                           seed = input$seed,
                           randomization = input$randomization)
      return(rst4)
    }

    if(input$design == "design.graeco"){
      trt1 <- input$treatment_11
      trt2 <- input$treatment_12
      rst5 <- design.graeco(trt1 = trt1,
                            trt2 = trt2,
                            serie = as.numeric(input$serie),
                            kinds = input$kind,
                            seed = input$seed,
                            randomization = input$randomization)
      return(rst5)
    }


    if(input$design == "design.alpha"){
      trt <- input$treatment_single
      rst6 <- design.alpha(trt = trt,
                           r = input$r,
                           k = input$k,
                           serie = as.numeric(input$serie),
                           kinds = input$kind,
                           seed = input$seed,
                           randomization = input$randomization)
      return(rst6)
    }

    if(input$design == "design.bib"){
      trt <- input$treatment_single
      rst6 <- design.bib(trt = trt,
                         r = input$r,
                         k = input$k,
                         serie = as.numeric(input$serie),
                         kinds = input$kind,
                         seed = input$seed,
                         maxRep = input$maxrep,
                         randomization = input$randomization)
      return(rst6)
    }

    if(input$design == "design.cyclic"){
      trt <- input$treatment_single
      rst6 <- design.cyclic(trt = trt,
                            r = input$r,
                            k = input$k,
                            rowcol = input$rowcol,
                            serie = as.numeric(input$serie),
                            kinds = input$kind,
                            seed = input$seed,
                            randomization = input$randomization)
      return(rst6)
    }

    if((input$design == "design.lattice") && ( input$simple == TRUE)){
      trt <- input$treatment_single
      rst6 <- design.lattice(trt = trt,
                             r = 2,
                             serie = as.numeric(input$serie),
                             kinds = input$kind,
                             seed = input$seed,
                             randomization = input$randomization)
      return(rst6)
    }

    if((input$design == "design.lattice") && (input$simple == FALSE)){
      trt <- input$treatment_single
      rst6 <- design.lattice(trt = trt,
                             r = 3,
                             serie = as.numeric(input$serie),
                             kinds = input$kind,
                             seed = input$seed,
                             randomization = input$randomization)
      return(rst6)
    }

    if((input$design == "design.split") && (input$factorial_design == "crd")){
      trt1 <- input$treatment_11
      trt2 <- input$treatment_12
      rst4 <- design.split(trt1 = trt1,
                           trt2 = trt2,
                           r = input$r,
                           design = input$factorial_design,
                           serie = as.numeric(input$serie),
                           kinds = input$kind,
                           seed = input$seed,
                           first = input$first,
                           randomization = input$randomization)
      return(rst4)
    }

    if((input$design == "design.split") && (input$factorial_design == "rcbd")){
      trt1 <- input$treatment_11
      trt2 <- input$treatment_12
      rst4 <- design.split(trt1 = trt1,
                           trt2 = trt2,
                           r = input$r,
                           design = input$factorial_design,
                           serie = as.numeric(input$serie),
                           kinds = input$kind,
                           seed = input$seed,
                           first = input$first,
                           randomization = input$randomization)
      return(rst4)
    }
    if((input$design == "design.split") && (input$factorial_design == "lsd")){
      trt1 <- input$treatment_11
      trt2 <- input$treatment_12
      rst4 <- design.split(trt1 = trt1,
                           trt2 = trt2,
                           r = input$r,
                           design = input$factorial_design,
                           serie = as.numeric(input$serie),
                           kinds = input$kind,
                           seed = input$seed,
                           first = input$first,
                           randomization = input$randomization)
      return(rst4)
    }



  })


  # print a summary

  plt <- reactive({
    if((input$design == "design.ab") && (input$factorial_design == "crd")){

      plt <- plot_design.factorial_crd(outdesign(),
                                       ncols = input$ncols ,
                                       nrows = ceiling(length(outdesign()$book[,1])/input$ncols),
                                       factor_name = input$choosen_factor,
                                       width = input$width,
                                       height = input$height,
                                       space_width = (1-input$space_width),
                                       space_height = (1-input$space_height),
                                       reverse_x = input$reverse_x,
                                       reverse_y = input$reverse_y)
    }

    if((input$design == "design.ab") && (input$factorial_design == "lsd")){
      plt <- (plot_design.factorial_lsd(design = outdesign(),
                                        factor_name = input$choosen_factor,
                                        width = input$width,
                                        height = input$height,
                                        space_width = (1-input$space_width),
                                        space_height = (1-input$space_height),
                                        reverse_y = input$reverse_y,
                                        reverse_x = input$reverse_x))
    }

    if((input$design == "design.ab") && (input$factorial_design == "rcbd")){
      plt <- (plot_design.factorial_rcbd(design = outdesign(),
                                         factor_name = input$choosen_factor,
                                         width = input$width,
                                         height = input$height,
                                         space_width = (1-input$space_width),
                                         space_height = (1-input$space_height),
                                         reverse_y = input$reverse_y,
                                         reverse_x = input$reverse_x))
    }

    if(input$design== "design.alpha"){

      plt <- (plot_alpha(design = outdesign(),
                         factor_name = "trt",
                         width = input$width,
                         height = input$height,
                         space_width = (1-input$space_width),
                         space_height = (1-input$space_height),
                         reverse_y = input$reverse_y,
                         reverse_x = input$reverse_x))
    }

    if(input$design == "design.bib"){

      plt <- (plot_bib(design = outdesign(),
                       factor_name = "trt",
                       width = input$width,
                       height = input$height,
                       space_width = (1-input$space_width),
                       space_height = (1-input$space_height),
                       reverse_y = input$reverse_y,
                       reverse_x = input$reverse_x))
    }

    if(input$design == "design.crd"){

      plt <- (plot_design_crd(design = outdesign(),
                              ncols = input$ncols ,
                              nrows = ceiling(length(outdesign()$book[,1])/input$ncols),
                              factor_name = "trt",
                              width = input$width,
                              height = input$height,
                              space_width = (1-input$space_width),
                              space_height = (1-input$space_height),
                              reverse_y = input$reverse_y,
                              reverse_x = input$reverse_x))
    }
    if(input$design == "design.cyclic"){

      plt <- (plot_cyclic(design = outdesign(),
                          factor_name = "trt",
                          width = input$width,
                          height = input$height,
                          space_width = (1-input$space_width),
                          space_height = (1-input$space_height),
                          reverse_y = input$reverse_y,
                          reverse_x = input$reverse_x))
    }
    if(input$design == "design.dau"){

      plt <- (plot_dau(design = outdesign(),
                       factor_name = "trt",
                       width = input$width,
                       height = input$height,
                       space_width = (1-input$space_width),
                       space_height = (1-input$space_height),
                       reverse_y = input$reverse_y,
                       reverse_x = input$reverse_x))
    }

    if(input$design == "design.lsd"){

      plt <- (plot_latin_square(design = outdesign(),
                                factor_name = "trt",
                                width = input$width,
                                height = input$height,
                                space_width = (1-input$space_width),
                                space_height = (1-input$space_height),
                                reverse_y = input$reverse_y,
                                reverse_x = input$reverse_x))
    }

    if(input$design == "design.rcbd"){

      plt <- (plot_rcdb(design = outdesign(),
                        factor_name = "trt",
                        width = input$width,
                        height = input$height,
                        space_width = (1-input$space_width),
                        space_height = (1-input$space_height),
                        reverse_y = input$reverse_y,
                        reverse_x = input$reverse_x))
    }

    if(input$design == "design.strip"){

      plt <- (plot_strip(design = outdesign(),
                         width = input$width,
                         factor_name_1 = "trt1",
                         factor_name_2 = "trt2",
                         height = input$height,
                         space_width = (1-input$space_width),
                         space_height = (1-input$space_height),
                         reverse_y = input$reverse_y,
                         reverse_x = input$reverse_x))
    }

    if(input$design == "design.youden"){

      plt <- (plot_youden(design = outdesign(),
                          factor_name = "trt",
                          width = input$width,
                          height = input$height,
                          space_width = (1-input$space_width),
                          space_height = (1-input$space_height),
                          reverse_y = input$reverse_y,
                          reverse_x = input$reverse_x))
    }

    if(input$design == "design.graeco"){

      plt <- (plot_graeco(design = outdesign(),
                          factor_name = "trt1",
                          width = input$width,
                          height = input$height,
                          space_width = (1-input$space_width),
                          space_height = (1-input$space_height),
                          reverse_y = input$reverse_y,
                          reverse_x = input$reverse_x))
    }

    if(input$design == "design.lattice" &&
       input$simple == TRUE){

      plt <- (plot_lattice_simple(design = outdesign(),
                                  factor_name = "trt",
                                  width = input$width,
                                  height = input$height,
                                  space_width = (1-input$space_width),
                                  space_height = (1-input$space_height),
                                  reverse_y = input$reverse_y,
                                  reverse_x = input$reverse_x))
    }

    if(input$design == "design.lattice" &&
       input$simple == FALSE){

      plt <- (plot_lattice_triple(design = outdesign(),
                                  factor_name = "trt",
                                  width = input$width,
                                  height = input$height,
                                  space_width = (1-input$space_width),
                                  space_height = (1-input$space_height),
                                  reverse_y = input$reverse_y,
                                  reverse_x = input$reverse_x))
    }

    if(input$design=="design.split" &&
       input$factorial_design=="crd"){
      plt <- (plot_split_crd(design = outdesign(),
                             nrows = ceiling(length(outdesign()$book[,1])/input$ncols),
                             ncols = input$ncols,
                             width = input$width,
                             height = input$height,
                             factor_name_1 = "trt1",
                             factor_name_2 = "trt2",
                             subplots = TRUE,
                             space_width = (1-input$space_width),
                             space_height = (1-input$space_height),
                             reverse_y = input$reverse_y,
                             reverse_x = input$reverse_x))

      if(input$swap_factor == FALSE){

        plt <- (plot_split_crd(design = outdesign(),
                               nrows = ceiling(length(outdesign()$book[,1])/input$ncols),
                               ncols = input$ncols,
                               width = input$width,
                               height = input$height,
                               factor_name_1 = "trt2",
                               factor_name_2 = "trt1",
                               subplots = TRUE,
                               space_width = (1-input$space_width),
                               space_height = (1-input$space_height),
                               reverse_y = input$reverse_y,
                               reverse_x = input$reverse_x))

      }
    }

    if(input$design=="design.split" &&
       input$factorial_design=="lsd"){
      plt <- (plot_split_lsd(design = outdesign(),
                             width = input$width,
                             height = input$height,
                             factor_name_1 = "trt1",
                             factor_name_2 = "trt2",
                             subplots = TRUE,
                             space_width = (1-input$space_width),
                             space_height = (1-input$space_height),
                             reverse_y = input$reverse_y,
                             reverse_x = input$reverse_x
      ))

      if(input$swap_factor == FALSE){
        plt <- (plot_split_lsd(design = outdesign(),
                               width = input$width,
                               height = input$height,
                               factor_name_1 = "trt2",
                               factor_name_2 = "trt1",
                               subplots = TRUE,
                               space_width = (1-input$space_width),
                               space_height = (1-input$space_height),
                               reverse_y = input$reverse_y,
                               reverse_x = input$reverse_x))
      }
    }

    if(input$design=="design.split" &&
       input$factorial_design=="rcbd"){
      plt <- (plot_split_rcbd(design = outdesign(),
                              width = input$width,
                              height = input$height,
                              factor_name_1 = "trt1",
                              factor_name_2 = "trt2",
                              subplots = TRUE,
                              space_width = (1-input$space_width),
                              space_height = (1-input$space_height),
                              reverse_y = input$reverse_y,
                              reverse_x = input$reverse_x
      ))

      if(input$swap_factor == FALSE){
        plt <- (plot_split_rcbd(design = outdesign(),
                                width = input$width,
                                height = input$height,
                                factor_name_1 = "trt2",
                                factor_name_2 = "trt1",
                                subplots = TRUE,
                                space_width = (1-input$space_width),
                                space_height = (1-input$space_height),
                                reverse_y = input$reverse_y,
                                reverse_x = input$reverse_x
        ))
      }
    }





    return(plt)
  })
  output$choosen_factor <- renderUI({
    numIndividuals <- as.integer(input$numIndividuals)
    selectInput(
      "choosen_factor",
      "Choose factor to plot the experiment",
      choices = LETTERS[1:numIndividuals]
    )
  })

  output$fields <- renderUI({
    numIndividuals <- as.integer(input$numIndividuals)
    lapply(1:numIndividuals, function(i) {
      selectizeInput(
        inputId = paste0("treatment", i),
        label = paste(
          "Type your intended levels of your Treatment,
          do not forget to include a control",
          i
        )
        ,
        choices = c("control")
        ,
        multiple = TRUE
        ,
        options = list(create = TRUE)
      )

    })
  })

  output$trt_duo <- renderUI({
    numIndividuals <- as.integer(2)
    lapply(1:numIndividuals, function(i) {
      selectizeInput(
        inputId = paste0("treatment_1", i),
        label = paste(
          "Type your intended levels of your Treatment",i,
          "do not forget to include a control if necessary"
        )
        ,
        choices = c("control")
        ,
        multiple = TRUE
        ,
        options = list(create = TRUE)
      )

    })
  })

  output$trt_single <- renderUI({
    selectizeInput(
      inputId = "treatment_single",
      label = "Type your intended levels of your Treatment,
               do not forget to include a control if necessary."
      ,
      choices = c("control")
      ,
      multiple = TRUE
      ,
      options = list(create = TRUE)
    )

  })

  output$oid1 <- renderDT({
    datatable(outdesign()$book, editable=FALSE)

  })

  output$oid2 <- renderDT({

    stats <- DOE_obj(plt())
    r <- to_table(stats,part = input$experimentparts, digits = input$digits)
    datatable(r, editable=FALSE)
  })

  output$doe_plot <- renderLeaflet({

    plt2 <-
      leaflet() %>% addTiles() %>% addPolygons(
        data = polygons_list(),
        weight = 1,
        fillColor = polygons_list()@data$fill,
        opacity = 1,
        fillOpacity = 1,
        label = c(polygons_list()@data$plot_nr)
      ) %>% addProviderTiles(input$maplayer)

    plt2


  })

  output$doe_plot_map <- renderPlot({
    plt3 <- plt() + theme_pres() + scale_fill_viridis_d(option = input$colorOption)+
    theme(panel.background = element_rect(fill = input$background))
    plt3
  })

  output$downloadData <- downloadHandler(
    filename = function() {
      paste0("dataset-", Sys.Date(), input$design, ".csv", sep = "")
    },
    content = function(filename) {
      write.csv(outdesign()$book, file = filename)
    }
  )

  output$downloadParams <- downloadHandler(
    filename = function() {
      paste0("model_Parameter", Sys.Date(), input$design, ".json", sep = "")
    },
    content = function(filename) {
      param_list <- outdesign()$parameter
      param_list$northing <- input$northing
      param_list$easting <- input$easting
      param_list$rotation <- input$rotation
      param_list$design <- input$design
      param_list$first <- input$first
      param_list$randomization <- input$randomization
      param_list$numIndividuals <- input$numIndividuals
      param_list$r <- input$r
      param_list$k <- input$k
      param_list$maxrep <- input$maxrep
      param_list$kind <- input$kind
      param_list$treatment_11 <- input$treatment_11
      param_list$treatment_12 <- input$treatment_12
      param_list$treatment_13 <- input$treatment_13
      param_list$treatment_14 <- input$treatment_14
      param_list$treatment_15 <- input$treatment_15
      param_list$treatment1 <- input$treatment1
      param_list$treatment2 <- input$treatment2
      param_list$treatment3 <- input$treatment3
      param_list$treatment4 <- input$treatment4
      param_list$treatment5 <- input$treatment5
      param_list$trt <- input$treatment_single
      param_list$width <- input$width
      param_list$height <- input$height
      param_list$space_width <- input$space_width
      param_list$space_height <- input$space_height
      param_list$reverse_y <- input$reverse_y
      param_list$reverse_x <- input$reverse_x
      param_list$rowcol <- input$rowcol
      param_list$colorOption <- input$colorOption
      param_list$background <- input$background

      json_obj <- jsonlite::toJSON(param_list)
      write(json_obj, filename)
    }
  )


  observeEvent(
    input$cloudUpload,
    {dat <- layer_data(plt())[, -(6:9)]
    drv <- DBI::dbDriver(RPostgres::Postgres())
    con <-
      dbConnect(
        drv,
        dbname = input$dbname,
        user = input$user,
        password = input$password,
        host = input$host,
        port = input$port
      )
    dbWriteTable(
      conn = con,
      value = dat,
      name = input$tablename,
      overwrite = TRUE
    )
    #dbWriteTable(conn=con,value=df_hex,name=input$tablename,overwrite =TRUE)
    dbDisconnect(con)

    showModal(
      modalDialog(
        title = "Data are written to Cloud Database",
        paste0(
          "The experiment design is successfully
                       written to cloud."
        ),
        easyClose = TRUE,
        footer = NULL
      )
    )
    }
  )
  # observeEvent(input$ISOBUS, {
  #   sf_df <- st_as_sf(polygons_list())
  #   print(polygons_list())
  #   st_crs(sf_df) <- 4326
  #   sf_df <- st_transform(sf_df, crs = 4326)
  #
  #   xml_doc <- xml_new_root("ISO11783_WorkingSet",
  #                           .namespace = "http://www.w3.org/2001/XMLSchema-instance",
  #                           `xsi:noNamespaceSchemaLocation` = "ISO11783-10.xsd")
  #
  #   working_set <- xml_add_child(xml_doc, "WorkingSet")
  #   device <- xml_add_child(working_set, "Device")
  #   sf_df$id <- 1:nrow(sf_df)
  #   # Iterate over each row in the sf dataframe
  #   for (i in 1:nrow(sf_df)) {
  #     id <- sf_df$id[i]
  #     name <- sf_df$name[i]
  #     coords <- st_coordinates(sf_df[i, ])
  #
  #     device_element <- xml_add_child(device, "DeviceElement", id = as.character(id), name = name)
  #     spatial_data <- xml_add_child(device_element, "SpatialData")
  #     polygon <- xml_add_child(spatial_data, "Polygon", id = as.character(id), name = name)
  #     coordinate_list <- xml_add_child(polygon, "CoordinateList")
  #
  #     # Add coordinates to the XML
  #     for (j in 1:nrow(coords)) {
  #       xml_add_child(coordinate_list, "Coordinate",
  #                     latitude = as.character(coords[j, "Y"]),
  #                     longitude = as.character(coords[j, "X"]))
  #     }
  #   }
  #
  #
  #   # Save the XML to a file
  #   write_xml(xml_doc, paste0(input$tablename,".xml"))
  #
  #   showModal(
  #     modalDialog(
  #       title = "ISOBUS Export Success!",
  #       paste0(
  #         "The experiment design is successfully written to ",
  #         input$tablename,
  #         '.xml .'
  #       ),
  #       easyClose = TRUE,
  #       footer = NULL
  #     )
  #   )
  # })

  observeEvent(input$saveSQL, {
    dat <- layer_data(plt())[, -(6:9)]
    drv <- DBI::dbDriver("PostgreSQL")
    con <-
      dbConnect(
        drv,
        dbname = input$dbname,
        user = input$user,
        password = input$password,
        host = input$host,
        port = input$port
      )
    dbWriteTable(
      conn = con,
      value = dat,
      name = input$tablename,
      overwrite = TRUE
    )
    #dbWriteTable(conn=con,value=df_hex,name=input$tablename,overwrite =TRUE)
    dbDisconnect(con)

    showModal(
      modalDialog(
        title = "Data are written to PostgreSQL.",
        paste0(
          "The experiment design is successfully written to ",
          input$dbname,
          " in table ",
          input$tablename,
          '.'
        ),
        easyClose = TRUE,
        footer = NULL
      )
    )
  })


  output$downloadpdf <- downloadHandler(
    filename = function() {
      paste0(
        "TreatmentPlan",
        Sys.Date(),
        input$design,
        "_",
        input$choosen_factor,
        ".pdf",
        sep = ""
      )
    },
    content = function(filename) {
      ggsave(
        filename = filename,
        device = "pdf",
        width = 130 ,
        height = 130,
        units = "mm"
      )
    }
  )

  polygons_list <- reactive({
    plt2 <- plt() + scale_fill_viridis_d(option = input$colorOption)
    polygons_list <-  make_polygons(plt2, north = input$northing,
                                    east = input$easting)

    polygons_list <- sf::as_Spatial(polygons_list)

    row.names(polygons_list) <- rownames(polygons_list@data)

    polygons_list <-
      sp::elide(polygons_list,
                rotate = input$rotation,
                center = apply(bbox(polygons_list), 1, mean))

  })

  output$exportgeojson <- downloadHandler(
    filename = function() {
      paste0("TreatmentPlan_", Sys.Date(), "_", input$design, "_", input$tablename, ".geojson")
    },
    content = function(file) {
      df_hex <- st_as_sf(polygons_list())  # Convert to sf object
      st_write(df_hex, file, driver = "GeoJSON")  # Save as GeoJSON
    }
  )

  output$package_list <- renderDT({
    pckg <- data.frame(installed.packages())
    pck_df <- pckg[pckg$Package %in% .packages(),c(3,10:12)]
    })


  observeEvent(input$tabs_a, {
    if (input$tabs_a == "about") {
      showModal(modalDialog(
        title = "Über diese Anwendung",

        # Liste der verwendeten R-Pakete
        h4("Verwendete R-Pakete:"),
        DTOutput(session$ns("package_list")),

        # Lizenzinfo
        h4("Lizenz:"),
        p("Diese Anwendung steht unter Lizenz."),

        # Link zur Website
        h4("Mehr Informationen:"),
        tags$a(href = "https://www.deine-webseite.de", "Besuche meine Website", target = "_blank"),

        easyClose = TRUE,
        footer = modalButton("Schließen")
      ))
    }
  })





  # output$makeLabels <- downloadHandler(
  #   filename = function() {
  #     paste0(input$output_file, ".pdf")
  #   },
  #   content = function(file) {
  #     generate_qr_labels(input_df = outdesign$book(),
  #                                     output_file = file,
  #                                     experiment_name = input$experiment_name,
  #                                     img_width = input$img_width,
  #                                     img_height = input$img_height,
  #                                     page_width = input$page_width,
  #                                     page_height = input$page_height)
  #   }
  # )
}



# Create Shiny app ----
shinyApp(ui, server)

## ----setup, eval = FALSE------------------------------------------------------
# install.packages("agricolaeplotr")

## ----message=FALSE, warning=FALSE---------------------------------------------
library("ggplot2")
library("agricolae")
library("agricolaeplotr")
library("raster")

## -----------------------------------------------------------------------------
library(agricolae) # origin of the needed design object
trt<-c(3,2) # factorial 3x2
outdesign <- design.ab(trt, r=3, serie=2,design = 'crd')

head(outdesign$book,10)

plot_design.factorial_crd(outdesign,ncols=7,nrows=3, width = 1, height = 1)


## ----echo=TRUE, results='asis'------------------------------------------------
plot_design.factorial_crd(outdesign,ncols=6,nrows=3, width = 1, height = 1, reverse_y = TRUE)

## ----echo=TRUE, results='asis'------------------------------------------------
plot_design.factorial_crd(outdesign,ncols=6,nrows=3, width = 1, height = 1, reverse_x = TRUE)

## ----results='asis'-----------------------------------------------------------
plot_design.factorial_crd(outdesign,ncols=6,nrows=3, width = 1, height = 1, reverse_x = TRUE,reverse_y = TRUE)

## ----echo = TRUE, results='asis'----------------------------------------------
plot_design.factorial_crd(outdesign,ncols=6,nrows=3, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE)

## ----echo = TRUE, results='asis'----------------------------------------------
plot_design.factorial_crd(outdesign,ncols=6,nrows=3, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE,space_width = 1,space_height = 1)

## ----echo = TRUE, results='asis'----------------------------------------------
plot_design.factorial_crd(outdesign,ncols=6,nrows=3, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE,space_width = 0.7,space_height = 0.8)

## ----echo = TRUE, results='asis'----------------------------------------------
plot_design.factorial_crd(outdesign,ncols=6,nrows=3, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE, factor_name = "B")

## ----echo = TRUE, results='asis'----------------------------------------------
set.seed(129984)
trt<-c(3,2) # factorial 3x2
outdesign <- design.ab(trt, r=3, serie=2,design = 'crd')
plot_design.factorial_crd(outdesign,ncols=3,nrows=6, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE, factor_name = "B")

## ----echo = TRUE, results='asis'----------------------------------------------
set.seed(129866478)
trt<-c(3,2) # factorial 3x2
outdesign <- design.ab(trt, r=3, serie=2,design = 'crd')
plot_design.factorial_crd(outdesign,ncols=3,nrows=6, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE, factor_name = "B") + theme_poster()

## ----echo = TRUE, results='asis'----------------------------------------------
set.seed(12986)
trt<-c(3,2) # factorial 3x2
outdesign <- design.ab(trt, r=3, serie=2,design = 'crd')

plot_design.factorial_crd(outdesign,ncols=3,nrows=6, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE, factor_name = "B") + theme_pres()

## ----echo = TRUE, results='asis'----------------------------------------------
trt<-c(3,2) # factorial 3x2
outdesign <- design.ab(trt, r=3, serie=2,design = 'crd')
plot_design.factorial_crd(outdesign,ncols=3,nrows=6, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE, factor_name = "B") + theme_pres() + scale_fill_viridis_d()

## -----------------------------------------------------------------------------
set.seed(23488833)
trt <-c(3,2)
outdesign<-design.ab(trt, serie=2, design="lsd",seed = 454555)
length_table <- dim(outdesign$book)[1] # length of the table

outdesign$book$yield <- sample(c(5:12,c(NA,NA,NA)), size = length_table, replace = TRUE)
plot_design.factorial_lsd(outdesign,factor_name = "yield") + scale_fill_viridis_c()


## -----------------------------------------------------------------------------
set.seed(23488833)
trt <-c(3,2)
outdesign<-design.ab(trt, serie=2, design="lsd",seed = 454555)
length_table <- dim(outdesign$book)[1] # length of the table

yield <- sample(c(5:20,c(NA,NA,NA)), size = length_table, replace = TRUE)
df <- cbind(plots=outdesign$book$plots,yield)
head(df,10)
outdesign$book <- merge(outdesign$book,df, by.x = "plots", by.y = "plots")
plot_design.factorial_lsd(outdesign,factor_name = "yield") + scale_fill_viridis_c()


## -----------------------------------------------------------------------------

set.seed(1298664)
plots <- as.factor(1:(8*6))
block <- as.factor((rep(1:6,each=8)))
A <- as.vector(replicate(8,sample(rep(1:2,times=3),6,replace=FALSE)))
outcome <- runif(48,20,100)
experiment <- cbind(plots,block,A,outcome)
experiment <- as.data.frame(experiment)
head(experiment)

experiment_design <- list()
experiment_design$parameters$design<- "factorial"
experiment_design$parameters$applied <-  "rcbd"

experiment_design$book <- experiment
head(experiment_design)
plot_design.factorial_rcbd(experiment_design,factor_name = "A")
plot_design.factorial_rcbd(experiment_design,factor_name = "outcome")


## -----------------------------------------------------------------------------
set.seed(1298664)
t1<-c('a','b','c','d','e',"f","g","h")
t2<-c("u",'v','w','x','y',"z")
outdesign2 <- design.split(trt1=t1, trt2=t2, r=r,serie = 2,
                           seed = 0, kinds = 'Super-Duper',
                           randomization=TRUE,first=TRUE,design = 'lsd')

plot_split_lsd(outdesign2,factor_name_1 = "t1",factor_name_2 = "t2",width = 2,height = 2, subplots = FALSE,labels = "plots")


plot_split_lsd(outdesign2,width = 2,height = 2, subplots = TRUE, labels = "splots", factor_name_1 = "t1", factor_name_2 = "t2")


## -----------------------------------------------------------------------------
set.seed(1298664)
t1<-c('a','b','c','d','e','f','g')
t2<-c('v','w','x','y','z')
r <- 4
outdesign2 <- design.split(trt1=t1, trt2=t2, r=r,
serie = 2, seed = 0, kinds = 'Super-Duper',
randomization=TRUE,first=TRUE,design = 'crd')
plot_split_crd(outdesign2,ncols = 5,nrows=6, subplots = FALSE,
               factor_name_1 = "t1",factor_name_2 = "t2")
plot_split_crd(outdesign2,ncols = 5,nrows=6, subplots = TRUE, labels="splots",factor_name_1 = "t1",factor_name_2 = "t2")


## -----------------------------------------------------------------------------
set.seed(1298664)
T1<-c('a','b','c','d','e',"f","g")
T2<-c("we",'v','w','x','y','z',"d")
r = 3
outdesign2 <- design.split(trt1=T1, trt2=T2, r=r,serie = 2,
 seed = 0, kinds = 'Super-Duper',randomization=TRUE,
 first=TRUE,design = 'rcbd')

plot_split_rcbd(outdesign2,width = 5,height = 5,subplots = FALSE, 
                factor_name_1 = "T1",factor_name_2 = "T2")

plot_split_rcbd(outdesign2,width = 5,height = 5,labels = "splots",
                factor_name_1 = "T1",factor_name_2 = "T2")



## ----message=FALSE, warning=FALSE---------------------------------------------

trt<-c(3,2) # factorial 3x2
outdesign <- design.ab(trt, r=3, serie=2,design = 'crd')
plt <- plot_design.factorial_crd(outdesign,ncols=3,nrows=6, width = 5, height = 7.5 , reverse_x = FALSE,reverse_y = TRUE, factor_name = "B") + theme_pres() + scale_fill_viridis_d()

spat_df <- make_polygons(plt,east = 3454206.89, 
                         north = 5939183.21 ,
                         projection_output = '+init=EPSG:4326')

plot(spat_df["fill"],col=spat_df$fill)

# this part does not work well in a vignette
library(leaflet)

spat_df <- sf:::as_Spatial(spat_df)

spat_df <- sp::elide(spat_df,rotate = -90)
                   
 leaflet(spat_df) %>% addPolygons(
   fillColor = spat_df$fill,
   opacity=1,
   color="black",
   fillOpacity = 1) %>% addProviderTiles(provider = "OpenStreetMap.DE")

## -----------------------------------------------------------------------------
varieties<-c('perricholi','yungay','maria bonita','tomasa')
outdesign <-design.youden(varieties,r=2,serie=2,seed=23)
p <- plot_youden(outdesign, labels = 'varieties', width=4, height=3)
stats <- DOE_obj(p)
r <- to_table(stats,part = "net_plot", digits = 2)
r
r <- to_table(stats,part = "gross_plot", digits = 2)
r
r <- to_table(stats,part = "field", digits = 2)
r
r <- to_table(stats,part = "experiment", digits = 2)
r
r <- to_table(stats,part = "all", digits = 2)
r

## -----------------------------------------------------------------------------

varieties<-c('perricholi','yungay','maria bonita','tomasa')
outdesign <-design.youden(varieties,r=2,serie=2,seed=23)
design <- outdesign$book
design

p <- full_control_positions(design,"col","row","varieties","plots",
                       width=3,height=4.5,
                       space_width=1,space_height=1,
                       shift_x=-0.5*3,shift_y=-0.5*4.5)
p

p <- full_control_positions(design,"col","row","varieties","plots",
                       width=3,height=4.5,
                       space_width=0.93,space_height=0.945,
                       start_origin = TRUE)
                       p

## -----------------------------------------------------------------------------
citation("agricolaeplotr")

## ----message=FALSE, warning=FALSE---------------------------------------------
library("agricolaeplotr")
library("FielDHub")
library("ggplot2")

## ----message=FALSE, warning=FALSE---------------------------------------------
example("RCBD")
plt <- plot(rcbd2)
p <- full_control_positions(plt$field_book, "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT")
p

## ----message=FALSE, warning=FALSE---------------------------------------------
latinSq1 <- latin_square(t = 4, reps = 2, plotNumber = 101, planter = "cartesian", seed = 1980)
plt <- plot(latinSq1)
p <- full_control_positions(plt$field_book, "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT")
p

## ----message=FALSE, warning=FALSE---------------------------------------------
example("full_factorial")
plt <- plot(fullFact2, l = 1)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "Loc1",], "COLUMN", "ROW", factor_name = "FACTOR_A", label = "FACTOR_A")
p

## ----message=FALSE, warning=FALSE---------------------------------------------
example("incomplete_blocks")
plt <- plot(ibd2)
p <- full_control_positions(plt$field_book, "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT")
p

## ----message=FALSE, warning=FALSE---------------------------------------------
example("diagonal_arrangement")
plt <- plot(spatAB)
p <- full_control_positions(plt$field_book, "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT")
p

## ----message=FALSE, warning=FALSE---------------------------------------------
example("RCBD_augmented")
plt <- plot(ARCBD2)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "A",], "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT") + guides(fill = "none")
p

## ----message=FALSE, warning=FALSE---------------------------------------------
example("rectangular_lattice")
plt <- plot(rectangularLattice2)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "LOC1",], "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT")
p

## ----message=FALSE, warning=FALSE---------------------------------------------
example("strip_plot")
plt <- plot(strip2)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "A",], "COLUMN", "ROW", factor_name = "TRT_COMB", label = "TRT_COMB")
p

## ----message=FALSE, warning=FALSE---------------------------------------------
example("square_lattice")
plt <- plot(squareLattice2)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "CASSELTON",], "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT")
p

## ----message=FALSE, warning=FALSE---------------------------------------------
example("split_plot")
plt <- plot(SPDExample2)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "A",], "COLUMN", "ROW", factor_name = "TRT_COMB", label = "TRT_COMB")
p

## ----message=FALSE, warning=FALSE---------------------------------------------
example("split_split_plot")
plt <- plot(SSPD2)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "A",], "COLUMN", "ROW", factor_name = "TRT_COMB", label = "TRT_COMB") + guides(fill = "none")
p

## ----message=FALSE, warning=FALSE---------------------------------------------
example("CRD")
plt <- plot(crd3)
p <- full_control_positions(plt$field_book, "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT") + guides(fill = "none")
p

## ----message=FALSE, warning=FALSE---------------------------------------------
example("sparse_allocation")
plt <- plot(sparse)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "LOC1",], "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT") + guides(fill = "none")
p

## ----message=FALSE, warning=FALSE---------------------------------------------
example("row_column")
plt <- plot(rowcold2, l = 1)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "1",], "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT") + guides(fill = "none")
p

.onAttach <- function(libname, pkgname) {
 packageStartupMessage( "\nType 'citation(\"agricolaeplotr\")' for citing this R package in publications.")
}


#' checks matrix column input
#'
#' checks if input is suitable for matrix column indication
#' @param x input to be tested
#'
#' @return error
#' @export
#' @examples
#' library(agricolaeplotr)
#' test_input_ncols(9)
test_input_ncols <- function(x) {
  if ((length(x)) > 1) {
    stop("The length of one integer value needs to be exact 1.")
  }
  if (is.numeric(x) == FALSE) {
    stop(paste0("Value is not an integer. It is from class ",
                class(x), "."))
  } else {
    if (round(x) != x) {
      stop("It is numeric value, but not an integer")
    }
    if (x <= 0) {
      stop("Integer value is smaller or equal than zero.
           The value needs to be greater than zero.")
    }
  }
}

#' checks matrix rows input
#'
#' checks if input is suitable for matrix row indication
#' @param x input to be tested
#'
#' @return error
#' @export
#' @examples
#' library(agricolaeplotr)
#' test_input_nrows(10)
test_input_nrows <- function(x) {
  if ((length(x)) > 1) {
    stop("The length of one integer value needs to be exact 1.")
  }
  if (is.numeric(x) == FALSE) {
    stop(paste0("Value is not an integer. It is from class ",
                class(x), "."))
  } else {
    if (round(x) != x) {
      stop("It is numeric value, but not an integer")
    }
    if (x <= 0) {
      stop("Integer value is smaller or equal than zero.
           The value needs to be greater than zero.")
    }
  }
}


#' Test if input for width and height is numeric
#'
#' Test if input is numeric for field width and height
#' @param x input to be tested
#'
#' @return error
#' @export
#' @examples
#' library(agricolaeplotr)
#' test_input_extend(3)
test_input_extend <- function(x) {

  if (is.numeric(x) == FALSE) {
    stop(paste0("Value is not numeric. It is from class ",
                class(x), "."))
  } else {
    if (x <= 0) {
      stop("Integer value is smaller or equal than zero.
           The value needs to be greater than zero.")
    }
  }
}


#' Test if input for shift parameter is numeric
#'
#' Test if input is numeric for shift parameter
#' @param x input to be tested
#' @export
#' @return error
#' @examples
#' library(agricolaeplotr)
#' test_input_shift(0.5)
test_input_shift <- function(x) {

  if (is.numeric(x) == FALSE) {
    stop(paste0("Value is not numeric. It is from class ",
                class(x), "."))
  }
}


#' Test if input is a logical
#'
#' Test if input is a logical
#' @param x input to be tested
#'
#' @return error
#' @export
#' @examples
#' library(agricolaeplotr)
#' test_input_reverse(TRUE)
test_input_reverse <- function(x) {
  if ((length(x)) > 1) {
    stop("The length of one boolean value needs to be exact 1.")
  }
  if (is.logical(x) == FALSE) {
    stop(paste0("Value is not logical. Only TRUE and FALSE are allowed.
                Your value is from class ",
                class(x), "."))
  }
}

#' Test if input is a string
#'
#' Test if input is a string
#' @param x input to be tested
#'
#' @return error
#' @export
#' @examples
#' library(agricolaeplotr)
#' test_string('smallstring')
test_string <- function(x) {
  if (is.character(x) == FALSE) {
    stop(paste0("Input parameter needs to be a string. ",
                "Your value is from class ", class(x),
                "."))
  }
}


#' Test if input column names
#'
#' Test if input is in column names of a table
#' @param design design from \code{agricolae} package
#' @param x string input
#'
#' @return error
#' @export
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)

#' trt<-c(2,4)
#' k=6
#' outdesign<-design.ab(trt, r=k, serie=3,design='rcbd')
#' test_name_in_column('B',outdesign)
test_name_in_column <- function(x, design) {
  test_string(x)
  if (x %in% colnames(design$book) == FALSE) {
    stop(paste0("Your string is not one of the column
                names the table of the design."))
  }
}


#' Test of experimental design
#'
#' Test if the outdesign file contains book and parameter list
#' @param design design from \code{agricolae} package
#'
#' @return error
#' @export
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)

#' trt<-c(2,4)
#' k=6
#' outdesign<-design.ab(trt, r=k, serie=3,design='rcbd')
#' test_names_design(outdesign)
test_names_design <- function(design) {
  if (sum(c("parameters", "book") %in% names(design)) <
      2) {
    stop("The function expects a list of at least two following names:
         parameters , book ")

  }
}

#### plot factorial_rcbd ####
#' Plot Factorial Designs with rcbd Design
#'
#' Plot a design of a factorial  experiment with randomized complete block design (rcbd) from design.ab
#' @param design outdesign from \code{agricolae} package
#' @param y Describes the y coordinates of a experiment design
#' @param factor_name Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#' @import ggplot2
#' @import agricolae
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)

#' trt<-c(2,4)
#' k=6
#' outdesign<-design.ab(trt, r=k, serie=3,design='rcbd')
#' plot_design.factorial_rcbd(design=outdesign,factor_name = 'B')
#' plot_design.factorial_rcbd(outdesign,reverse_y = TRUE,reverse_x = TRUE)
#'
plot_design.factorial_rcbd <- function(design,
                                       y = "row",
                                       factor_name = "A",
                                       width = 1,
                                       height = 1,
                                       space_width = 0.95,
                                       space_height = 0.85,
                                       reverse_x = FALSE,
                                       reverse_y = FALSE){
  if (design$parameters$design == "factorial" &&
      design$parameters$applied == "rcbd") {

    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name, design)
    plots <- as.numeric(design$book[, 1])
    mat <- matrix(plots, byrow = TRUE,
                  ncol = max(as.numeric(design$book$block)))
    dims <- dim(mat)
    table <- as.table(mat)
    rownames(table) <- 1:dims[1]
    colnames(table) <- 1:dims[2]
    table <- as.data.frame(table)
    colnames(table) <- c("row", "col", "plots")
    table[, y] <- as.numeric(table[, y])
    table$col <- as.numeric(table$col)

    table <- merge(table, design$book, by.x = "plots",
                   by.y = "plots")

    table$col <- rep(1:(length(table$plots)/max(as.numeric(table[, y]))),
                        times = max(as.numeric(table[, y])))

    # no space, 0 == only space no space, 0
    # == only space

    table$col <- table$col * width
    table$row <- as.numeric(table[, y]) * height
    if (reverse_y == TRUE) {
      table$row <- abs(table$row - max(table$row)) +
        min(table$row)
    }

    if (reverse_x == TRUE) {
      table$col <- abs(table$col - max(table$col)) +
        min(table$col)
    }

    plt <- ggplot(table, aes(x = .data[["col"]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[["plots"]]),
        colour = "black"
      )


    return(plt)

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from a factorial design with a
                random complete block design (rcbd) is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }
}

#### plot design crd ####
#' Plot Complete Randomized Design
#'
#' Plot a design of a factorial experiment with randomized complete block design from \code{agricolae} design.ab
#' @param design outdesign from \code{agricolae} package
#' @param ncols integer value, choose the number of columns to which the experiment should be plotted
#' @param nrows integer value, choose the number of rows to which the experiment should be plotted
#' @param y Describes the y coordinates of a experiment design, default is row
#' @param factor_name Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels Describes the column from that the plots are taken to display them
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)

#' trt = c(2,3,4,5,6)
#' outdesign1 <-design.crd(trt,r=5,serie=2,2543,'Mersenne-Twister')
#' plot_design_crd(outdesign1,ncols = 13,nrows = 3)

plot_design_crd <- function(design,
                            ncols,
                            nrows,
                            y = "row",
                            factor_name = "trt",
                            labels = "plots",
                            width = 1,
                            height = 1,
                            space_width = 0.95,
                            space_height = 0.85,
                            reverse_y = FALSE,
                            reverse_x = FALSE) {
  if (design$parameters$design == "crd") {

    plots <- as.numeric(design$book[, 1])
    if (ncols * nrows >= length(plots)) {
    nas <- rep(NaN, ncols * nrows - length(plots))
    plots <- c(plots, nas)

    mat <- matrix(plots, byrow = TRUE, ncol = ncols)
    dims <- dim(mat)
    table <- as.table(mat)
    rownames(table) <- 1:dims[1]
    colnames(table) <- 1:dims[2]
    table <- as.data.frame(table)
    colnames(table) <- c("row", "col", "plots")
    table$row <- as.numeric(table$row)
    table$col <- as.numeric(table$col)

    table <- merge(table, design$book, by.x = "plots",
                   by.y = "plots")

    table$row <- as.numeric(table$row) * height
    table$col <- as.numeric(table$col) *
      width

    if (reverse_y == TRUE) {
      table$row <- abs(table$row - max(table$row)) +
        min(table$row)
    }
    if (reverse_x == TRUE) {
      table$col <- abs(table$col - max(table$col)) +
        min(table$col)
    }
    plt <- ggplot(table, aes(x = .data[["col"]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )

    plt

    return(plt)
    } else {
      stop(paste("You have in multiplication of ncols:",
                 ncols, "and nrows:", nrows, "Elements.",
                 "You need at minimum a product of both higher than",
                 length(plots)))
    }
  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from a factorial design with a
                complete random design (crd) is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }
}


#### plot alpha design ####
#' Plot Alpha design Experiments
#'
#' Plot a design of an experiment with an alpha design from \code{agricolae} design.alpha
#'
#' @param design outdesign from \code{agricolae} package
#' @param x Describes the x coordinates of a experiment design
#' @param y Describes the y coordinates of a experiment design
#' @param factor_name Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels Describes the column from that the plots are taken to display them
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' trt<-1:30
#' t <- length(trt)
#' # size block k
#' k<-3
#' # Blocks s
#' s<-t/k
#' # replications r
#' r <- 2
#' outdesign<- design.alpha(trt,k,r,serie=2)
#' plot_alpha(outdesign)
plot_alpha <- function(design, x = "cols", y = "block",
                       factor_name = "trt", labels = "plots", width = 1,
                       height = 1, space_width = 0.95,
                       space_height = 0.85,
                       reverse_y = FALSE, reverse_x = FALSE) {
  if (design$parameters$design == "alpha") {
    test_string(x)
    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(labels, design)
    test_name_in_column(factor_name, design)

    table <- as.data.frame(design$book)
    table[, y] <- as.numeric(table[, y]) * height
    table[, x]  <- as.numeric(table[, x] ) * width


    if (reverse_x == TRUE) {
      table[, x]  <- abs(table[, x]  - max(table[, x] )) +
        min(table[, x] )
    }

    if (reverse_y == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }
    plt <- ggplot(table, aes(x = .data[[x]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )

    plt

    return(plt)

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from an alpha design is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }

}



####### plot lattice design triple######

#' Plot Triple Lattice Design
#'
#' Plot a design of a factorial  experiment with a latin square design from \code{agricolae} design.lattice with r=3
#' @param design outdesign from \code{agricolae} package
#' @param y Describes the y coordinates of a experiment design
#' @param factor_name Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels Describes the column from that the plots are taken to display them
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)

#' trt<-LETTERS[1:9]
#' outdesign<-design.lattice(trt,r=3,serie=2)
#' plot_lattice_triple(design=outdesign,reverse_x=TRUE)
#'
plot_lattice_triple <- function(design,
                                y = "block", factor_name = "trt",
                                labels = "plots",
                                width = 1,
                                height = 1,
                                space_width = 0.95,
                                space_height = 0.85,
                                reverse_y = FALSE,
                                reverse_x = FALSE) {
  if (design$parameters$design == "lattice" && design$parameters$type ==
      "triple") {


    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name, design)
    test_name_in_column(labels, design)

    table <- design$book
    max_b <- max(table(table[, y]))

    table$part <- rep(1:max_b,
                      times = length(table[, y])/max_b)
    table[, y] <- as.numeric(table[, y]) * height
    table$part <- table$part * width

    if (reverse_y == TRUE) {
      table$part <- abs(table$part - max(table$part)) +
        min(table$part)
    }

    if (reverse_x == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }

    plt <- ggplot(table, aes(x = .data[["part"]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )

    plt

    return(plt)

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from an lattice design is needed here.",
                "you have a design of type ", design$parameters$design,
                " and a type of ", design$parameters$type,
                "."))
  }

}



####### plot lattice design simple #####
#' Plot Simple Lattice Design
#'
#' Plot a design of a factorial  experiment with a lattice design from \code{agricolae} design.lattice with r=2
#' @param design outdesign from \code{agricolae} package
#' @param y Describes the y coordinates of a experiment design
#' @param factor_name Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels Describes the column from that the plots are taken to display them
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)

#' trt<-1:100
#' outdesign<-design.lattice(trt,r=2,serie=3) # simple lattice design, 10x10
#' plot_lattice_simple(outdesign,width = 2, height = 1)

plot_lattice_simple <- function(design,
                                y = "block",
                                factor_name = "trt",
                                labels = "plots",
                                width = 1,
                                height = 1,
                                space_width = 0.95,
                                space_height = 0.85,
                                reverse_y = FALSE,
                                reverse_x = FALSE) {
  if (design$parameters$design == "lattice" && design$parameters$type ==
      "simple") {


    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name, design)
    test_name_in_column(labels, design)

    table <- design$book
    table[, y] <- as.numeric(table[, y])
    table$part <- rep(1:sqrt(length(unique(table[, factor_name]))),
                      times = max(table[, y]))
    table[, y] <- table[, y] * height
    table$part <- table$part * width
    if (reverse_y == TRUE) {
      table$part <- abs(table$part - max(table$part)) +
        min(table$part)
    }

    if (reverse_x == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }

    plt <- ggplot(table, aes(x = .data[["part"]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )

    plt

    return(plt)

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from an lattice design is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }

}



######## latin square design ######

#' Plot Latin Square Design
#'
#' Plot a design of a factorial  experiment with a latin square design from \code{agricolae} design.lsd
#' @param design outdesign from \code{agricolae} package
#' @param x Describes the x coordinates of a experiment design
#' @param y Describes the y coordinates of a experiment design
#' @param factor_name Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels Describes the column from that the plots are taken to display them
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)

#' trt<-LETTERS[1:9]
#' outdesign<- design.lsd(trt,serie=2)
#' plot_latin_square(outdesign, reverse_y = TRUE)

plot_latin_square <- function(design,
                              x = "col",
                              y = "row",
                              factor_name = "trt",
                              labels = "plots",
                              width = 1,
                              height = 1,
                              space_width = 0.95,
                              space_height = 0.85,
                              reverse_y = FALSE,
                              reverse_x = FALSE) {
  if (design$parameters$design == "lsd") {

    test_string(x)
    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name, design)
    test_name_in_column(labels, design)
    table <- as.data.frame(design$book)
    table[, x]  <- as.numeric(table[, x] ) * width
    table[, y] <- as.numeric(table[, y]) * height
    if (reverse_y == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }
    if (reverse_x == TRUE) {
      table[, x]  <- abs(table[, x]  - max(table[, x] )) +
        min(table[, x] )
    }
    plt <- ggplot(table, aes(x = .data[[x]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )

    plt

    return(plt)

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from an latin_square design is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }

}
#### graeco latin square #######

#' Plot Graeco Latin Square Design
#'
#' Plot a design of an experiment with an Graeco - latin square design from \code{agricolae} design.graeco
#' @param design outdesign from \code{agricolae} package
#' @param x Describes the x coordinates of a experiment design
#' @param y Describes the y coordinates of a experiment design
#' @param factor_name Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels Describes the column from that the plots are taken to display them
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' T1<-c('a','b','c','d')
#' T2<-c('v','w','x','y','z','zz')
#' outdesign <- design.graeco(trt1=T1, trt2=T2, serie = 2,
#'  seed = 0, kinds = 'Super-Duper',randomization=TRUE)
#' plot_graeco(outdesign, factor_name = 'T2',reverse_y = TRUE)
#' plot_graeco(outdesign, factor_name = 'T2',reverse_x = TRUE)

plot_graeco <- function(design,
                        x = "col",
                        y = "row",
                        factor_name = "T1",
                        labels = "plots",
                        width = 1,
                        height = 1,
                        space_width = 0.95,
                        space_height = 0.85,
                        reverse_y = FALSE,
                        reverse_x = FALSE) {
  if (design$parameters$design == "graeco") {


    test_string(x)
    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name, design)
    test_name_in_column(labels, design)

    table <- as.data.frame(design$book)
    table[, x]  <- as.numeric(table[, x] ) * width
    table[, y] <- as.numeric(table[, y]) * height


    if (reverse_y == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }
    if (reverse_x == TRUE) {
      table[, x]  <- abs(table[, x]  - max(table[, x] )) +
        min(table[, x] )
    }
    plt <- ggplot(table, aes(x = .data[[x]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )

    return(plt)

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from an graeco design is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }

}



#### plot strip design ####
#' Plot Strip Design
#'
#' Plot a design of an experiment with an Strip Plot design from \code{agricolae} design.strip
#' @param design outdesign from \code{agricolae} package
#' @param x Describes the x coordinates of a experiment design
#' @param y Describes the y coordinates of a experiment design
#' @param factor_name_1 Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param factor_name_2 Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels Describes the column from that the plots are taken to display them
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' T1<-c('a','b','c','d')
#' T2<-c('v','w','x','y','z')
#' r = 3
#' outdesign <- design.strip(trt1=T1, trt2=T2, r=r,serie = 2,
#'  seed = 0, kinds = 'Super-Duper',randomization=TRUE)
#' plot_strip(outdesign,factor_name_1 = "T1",factor_name_2="T2")
#' plot_strip(outdesign,factor_name_1 = "T1",factor_name_2="T2",reverse_x = TRUE)
plot_strip <- function(design,
                       x = "col",
                       y = "row",
                       factor_name_1 = "T1",
                       factor_name_2 = "T2",
                       labels = "plots",
                       width = 1,
                       height = 1,
                       space_width = 0.95,
                       space_height = 0.85,
                       reverse_y = FALSE,
                       reverse_x = FALSE) {
  if (design$parameters$design == "strip") {


    test_string(x)
    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name_1, design)
    test_name_in_column(factor_name_2, design)
    test_name_in_column(labels, design)

    table <- as.data.frame(design$book)
     #########
    # if blocks are horizontal:
    table[, x] <- as.numeric(table[, factor_name_1]) +
      (max(as.numeric(table[, factor_name_1]))*as.numeric(table$block)-1) -
      (max(as.numeric(table[, factor_name_1])))+1
    table[, x] <- sort(table[, x])
    table[, x]  <- table[, x] * width
    table[, y] <- as.numeric(table[, factor_name_2]) * height
        #########

  # if block are not horizontal
  # table[, y]  <- as.numeric(table[, factor_name_2]) +
  #    (max(as.numeric(table[, factor_name_2]))*as.numeric(table$block)-1) -
  #    (max(as.numeric(table[, factor_name_2])))+1
  #   table[, y] <- sort(table[, y])
  # table[, y] <- table[,y] * height
  # table[, x] <- as.numeric(table[, factor_name_1]) * width




    if (reverse_y == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }
    if (reverse_x == TRUE) {
      table[, x]  <- abs(table[, x]  - max(table[, x] )) +
        min(table[, x] )
    }
    plt <- ggplot(table, aes(x = .data[[x]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name_1]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )


    return(plt)

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from an strip design is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }

}




#### plot bib design #######

#' Plot Randomized Balanced Incomplete Block Designs
#'
#' Plot a design of an experiment with an Randomized Balanced Incomplete Block Designs (BIB) from design.bib
#' @param design outdesign from \code{agricolae} package
#' @param y Describes the y coordinates of a experiment design
#' @param factor_name Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels Describes the column from that the plots are taken to display them
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' trt<-c('A','B','C','D')
#' k<-3
#' outdesign<-design.bib(trt,k,serie=2,seed =41,kinds ='Super-Duper') # seed = 41
#' plot_bib(outdesign)
#' #now let us change position of the columns
#' plot_bib(outdesign,reverse_x = TRUE)
plot_bib <- function(design,
                     y = "block",
                     factor_name = "trt",
                     labels = "plots",
                     width = 1,
                     height = 1,
                     space_width = 0.95,
                     space_height = 0.85,
                     reverse_y = FALSE,
                     reverse_x = FALSE) {
  if (design$parameters$design == "bib") {



    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name, design)
    test_name_in_column(labels, design)

    table <- as.data.frame(design$book)
    table$col <- rep(1:max(table(table[, y])),
                     times = max(as.numeric(table[, y])))
    table$col <- table$col * width
    table[, y] <- as.numeric(table[, y]) * height
    if (reverse_y == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }
    if (reverse_x == TRUE) {
      table$col <- abs(table$col - max(table$col)) +
        min(table$col)
    }
    plt <- ggplot(table, aes(x = .data[["col"]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )


    return(plt)

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from an bib design is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }

}



####### plot cyclic design #######

#' Plot Cyclic Design
#'
#' Plot a design of an experiment with an cyclic design from \code{agricolae} design.cyclic
#' @param design outdesign from \code{agricolae} package
#' @param y Describes the y coordinates of a experiment design
#' @param factor_name Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels Describes the column from that the plots are taken to display them
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)

#' k <- 2
#' r <- 6
#' trt <-c('CIP-101','CIP-201','CIP-301','CIP-401','CIP-501',LETTERS[1:2])
#' outdesign<- design.cyclic(trt,k=k, r=r, serie=3, rowcol=TRUE)
#' plot_cyclic(outdesign, factor_name = 'trt')
plot_cyclic <- function(design,
                        y = "block",
                        factor_name = "trt",
                        labels = "plots",
                        width = 1,
                        height = 1,
                        space_width = 0.95,
                        space_height = 0.85,
                        reverse_y = FALSE,
                        reverse_x = FALSE) {
  if (design$parameters$design == "cyclic") {



    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(labels, design)

    table <- design$book
    table$part <- rep(1:max(table(table[, y])),
                  times = length(table[, factor_name])/max(table(table[, y])))

    table[, y] <- as.numeric(table[, y]) * height
    table$part <- table$part * width
    if (reverse_y == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }

    if (reverse_x == TRUE) {
      table$part <- abs(table$part - max(table$part)) +
        min(table$part)
    }
    plt <- ggplot(table, aes(x = .data[["part"]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )

    return(plt)

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from an cyclic design is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }

}




####### plot dau design #######

#' Plot Design of Augmented Blocks (dau)
#'
#' Plot a design of an experiment with an augmented block design from \code{agricolae} design.dau
#' @param design outdesign from \code{agricolae} package
#' @param y Describes the y coordinates of a experiment design
#' @param factor_name Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels Describes the column from that the plots are taken to display them
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)

#' T1<-c('A','B','C','D','E','F')
#' T2<-letters[19:26]
#' outdesign <-design.dau(T1,T2, r=5,serie=2)
#' plot_dau(outdesign)
#' plot_dau(outdesign,reverse_y = TRUE)
plot_dau <- function(design,
                     y = "block",
                     factor_name = "trt",
                     labels = "plots",
                     width = 1,
                     height = 1,
                     space_width = 0.95,
                     space_height = 0.85,
                     reverse_y = FALSE,
                     reverse_x = FALSE) {
  if (design$parameters$design == "dau") {


    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name, design)
    test_name_in_column(labels, design)
    table <- design$book
    freq <- table(design$book$block)
    freq <- as.vector(freq)
    seqx <- function(x) {
      return(1:x)
    }
    col <- lapply(freq, seqx)
    table$col <- unlist(col)


    table$col <- table$col * width
    table[, y] <- as.numeric(table[, y]) * height

    if (reverse_y == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }
    if (reverse_x == TRUE) {
      table$col <- abs(table$col - max(table$col)) +
        min(table$col)
    }
    plt <- ggplot(table, aes(x = .data[["col"]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )


    return(plt)

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from an cyclic design is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }

}




###### plot rcbd ######

#' Plot randomized complete block designs
#'
#' Plot a design of an experiment with randomized complete block design (rcbd) design from \code{agricolae} design.rcbd
#' @param design outdesign from \code{agricolae} package
#' @param y Describes the y coordinates of a experiment design
#' @param factor_name Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels Describes the column from that the plots are taken to display them
#' @param treatment_label Describes the column(s) from the treatments the plots are taken to display them as a label text
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param label_width numeric value, describes the maximum width of a label
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @importFrom stringr str_wrap
#' @export
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)

#' # 5 treatments and 6 blocks
#' trt<-c('A','B','C','D','E')
#' outdesign <-design.rcbd(trt,6,serie=2,986,'Wichmann-Hill') # seed = 986
#' plot_rcbd(outdesign)
#' plot_rcbd(outdesign,reverse_y = TRUE,reverse_x = TRUE)
#'
plot_rcbd <- function(design,
                      y = "block",
                      factor_name = "trt",
                      labels = "plots",
                      treatment_label = "trt",
                      width = 1,
                      height = 1,
                      space_width = 0.95,
                      space_height = 0.85,
                      label_width = 10,
                      reverse_y = FALSE,
                      reverse_x = FALSE) {
  if (design$parameters$design == "rcbd") {


    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name, design)
    test_name_in_column(labels, design)

    table <- as.data.frame(design$book)

    table$col <- rep(seq_len(length(unique(table[, factor_name]))),
                     length(unique(table[, y])))
    table[, y] <- as.numeric(table[, y])
    if (reverse_y == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }
    if (reverse_x == TRUE) {
      table$col <- abs(table$col - max(table$col)) +
        min(table$col)
    }
    table$col <- table$col * width
    table[, y] <- as.numeric(table[, y]) * height
    table[, labels] <- str_wrap(table[,treatment_label], width = label_width)

    plt <- ggplot(table, aes(x = .data[["col"]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_label(
        aes(label = .data[[labels]]),
        colour = "black",
        fill = "white"
      )


    return(plt)

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from a Randomized Complete Block Design is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }

}





##### plot factorial crd ########

#' Plot Factorial Complete Randomized Designs (crd)
#'
#' Plot a design of a factorial experiment with completely randomized design (crd) from design.ab
#'
#' @param design outdesign from \code{agricolae} package
#' @param ncols integer value, choose the number of columns to which the experiment should be plotted
#' @param nrows integer value, choose the number of rows to which the experiment should be plotted
#' @param y Describes the y coordinates of a experiment design, default is row
#' @param factor_name Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param labels string indicates the column of which the labels should be displayed
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#' @import ggplot2
#' @import agricolae
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)

#' trt<-c(3,2) # factorial 3x2
#' outdesign <- design.ab(trt, r=3, serie=2,design = 'crd')
#' plot_design.factorial_crd(outdesign,ncols = 8,nrows = 6)
#' plot_design.factorial_crd(outdesign,reverse_y = TRUE,ncols = 8,nrows = 6)
#' plot_design.factorial_crd(outdesign,reverse_y = TRUE,reverse_x = TRUE,ncols = 8,nrows = 6)

plot_design.factorial_crd <- function(design,
                                      ncols,
                                      nrows,
                                      y = "row",
                                      factor_name = "A",
                                      labels="plots",
                                      width = 1,
                                      height = 1,
                                      space_width = 0.95,
                                      space_height = 0.85,
                                      reverse_y = FALSE,
                                      reverse_x = FALSE) {
  if (design$parameters$design == "factorial" &&
      design$parameters$applied == "crd") {


    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name, design)
    test_name_in_column(labels, design)

    plots <- as.numeric(design$book[, 1])
    if (ncols * nrows >= length(plots)) {
    nas <- rep(NaN, ncols * nrows - length(plots))
    plots <- c(plots, nas)

    mat <- matrix(plots, byrow = TRUE, ncol = ncols)
    dims <- dim(mat)
    table <- as.table(mat)
    rownames(table) <- 1:dims[1]
    colnames(table) <- 1:dims[2]
    table <- as.data.frame(table)
    colnames(table) <- c("row", "col", "plots")
    table[, y] <- as.numeric(table[, y])
    table$col <- as.numeric(table$col)

    table <- merge(table, design$book, by.x = "plots",
                   by.y = "plots")
    table$col <- table$col * width
    table[, y] <- table[, y] * height
    if (reverse_y == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }
    if (reverse_x == TRUE) {
      table$col <- abs(table$col - max(table$col)) +
        min(table$col)
    }
    plt <- ggplot(table, aes(x = .data[["col"]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )

    return(plt)
    } else {
      stop(paste("You have in multiplication of ncols:",
                 ncols, "and nrows:", nrows, "Elements.",
                 "You need at minimum a product of both higher than",
                 length(plots)))
    }
  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from a factorial design with
         a random complete block design (rcbd) is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }
}


##### plot factorial lsd ########

#' Plot Factorial Latin Square Designs (lsd)
#'
#' Plot a design of a factorial  experiment with latin square design (lsd) design from \code{agricolae} design.ab
#'
#' @param design outdesign from \code{agricolae} package
#' @param x Describes the x coordinates of a experiment design
#' @param y Describes the y coordinates of a experiment design
#' @param labels Describes the column from that the plots are taken to display them
#' @param factor_name Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#' @import ggplot2
#' @import agricolae
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)

#' trt<-c(3,2) # factorial 3x2
#' outdesign <-design.ab(trt, r=3, serie=2,design = 'lsd')
#' plot_design.factorial_lsd(outdesign,factor_name = 'B',reverse_x = TRUE)
#'
plot_design.factorial_lsd <- function(design,
                                      x = "col",
                                      y = "row",
                                      factor_name = "A",
                                      labels = "plots",
                                      width = 1,
                                      height = 1,
                                      space_width = 0.95,
                                      space_height = 0.85,
                                      reverse_y = FALSE,
                                      reverse_x = FALSE) {
  if (design$parameters$design == "factorial" &&
      design$parameters$applied == "lsd") {
    test_string(x)
    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name, design)
    test_name_in_column(labels, design)
    table <- as.data.frame(design$book)
    table[, x]  <- as.numeric(table[, x]) * width
    table[, y] <- as.numeric(table[, y]) * height
    if (reverse_y == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }
    if (reverse_x == TRUE) {
      table[, x]  <- abs(table[, x]  - max(table[, x] )) +
        min(table[, x] )
    }
    plt <- ggplot(table, aes(x = .data[[x]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )

    plt

    return(plt)

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from an latin square design is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }

}

########## plot split plot design rcbd
########## #############

#' Plot Split Plot Designs with rcbd
#'
#' Plot a design of a split plot experiment with randomized complete blocks design (rcbd) from design.split
#' @param design outdesign from \code{agricolae} package
#' @param y string defines the block
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#' @param factor_name_1 string Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param factor_name_2 string Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels string Describes the column from that the plots are taken to display them
#' @param subplots should the plot function return the subplots (default) or main plots?
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#' @import ggplot2
#' @import agricolae
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' T1<-c('a','b','c','d','e')
#' T2<-c('v','w','x','y','z','zz')
#' r = 3
#' outdesign2 <- design.split(trt1=T1, trt2=T2, r=r,serie = 2,
#'  seed = 0, kinds = 'Super-Duper',randomization=TRUE,
#'  first=TRUE,design = 'rcbd')
#' plot_split_rcbd(outdesign2,width = 1,height = 1)
#' plot_split_rcbd(outdesign2,width = 1,height = 1,reverse_y = TRUE)
#' plot_split_rcbd(outdesign2,width = 1,height = 1,reverse_x = TRUE,reverse_y = TRUE)

plot_split_rcbd <- function(design,
                            y = "block",
                            factor_name_1 = "T1",
                            factor_name_2 = "T2",
                            subplots=TRUE,
                            labels = "plots",
                            width = 1,
                            height = 1,
                            space_width = 0.95,
                            space_height = 0.85,
                            reverse_y = FALSE,
                            reverse_x = FALSE) {
  if (design$parameters$design == "split" &&
      design$parameters$applied == "rcbd") {

    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name_1, design)
    test_name_in_column(factor_name_2, design)

    test_name_in_column(labels, design)

    table <- design$book

    table$row <- rep(rep(seq_len(length(unique(table[, factor_name_1]))),
                         each=length(unique(table[, factor_name_2]))),
                     times=max(as.numeric(table[, y])))
    table$row <- table$row * width

    table$sequence <- rep(1:(length(unique(table[, factor_name_1])) *
                               length(unique(table[, factor_name_2]))),
                          times = max(as.numeric(table[, y])))

    table$sequence <- table$sequence * width
    table[, y] <- as.numeric(table[, y]) * height

    if (reverse_y == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }

    if (reverse_x == TRUE) {

      table$row <- abs(table$row - max(table$row)) +
        min(table$row)

      table$sequence <- abs(table$sequence - max(table$sequence)) +
        min(table$sequence)
    }


    if(subplots == TRUE){

      divider <- length(unique(table[, factor_name_1]))
      table$sequence <- table$sequence/divider

      plt2 <- ggplot(table, aes(x = .data[["sequence"]], y = .data[[y]])) +
        geom_tile(
          aes(fill = .data[[factor_name_2]]),
          width = (width / divider) * space_width,
          height = height * space_height
        ) +
        theme_bw() +
        theme(line = element_blank()) +
        geom_text(
          aes(label = .data[[labels]]),
          colour = "black"
        )


      return(plt2)

    } else{

      plt <- ggplot(table, aes(x = .data[["row"]], y = .data[[y]])) +
        geom_tile(
          aes(fill = .data[[factor_name_1]]),
          width = width * space_width,
          height = height * space_height
        ) +
        theme_bw() +
        theme(line = element_blank()) +
        geom_text(
          aes(label = .data[[labels]]),
          colour = "black"
        )

      return(plt)
    }

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from a split plot design with a
                random complete block design (rcbd) is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }
}






########## plot split plot design lsd
########## #############

#' Plot Split Plot Design lsd
#'
#' Plot a design of a split plot experiment with latin squared design (lsd) from design.split
#' @param design outdesign from \code{agricolae} package
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#' @param factor_name_1 string Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param factor_name_2 string Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels string Describes the column from that the plots are taken to display them
#' @param subplots should the plot function return the subplots (default) or main plots?
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#' @import ggplot2
#' @import agricolae
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' T1<-c('a','b','c','d','e')
#' T2<-c('v','w','x','y')
#' outdesign2 <- design.split(trt1=T1, trt2=T2, r=r,serie = 2,
#'                            seed = 0, kinds = 'Super-Duper',
#'                            randomization=TRUE,first=TRUE,design = 'lsd')
#' plot_split_lsd(outdesign2,width = 4,height = 4)

plot_split_lsd <- function(design,
                           factor_name_1 = "T1",
                           factor_name_2 = "T2",
                           labels = "plots",
                           subplots=TRUE,
                           width = 1,
                           height = 1,
                           space_width = 0.95,
                           space_height = 0.85,
                           reverse_y = FALSE,
                           reverse_x = FALSE) {
  if (design$parameters$design == "split" && design$parameters$applied ==
      "lsd") {

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name_1, design)
    test_name_in_column(factor_name_2, design)
    test_name_in_column(labels, design)

    table <- design$book

    table$sequence <- rep(1:(length(unique(table[, factor_name_1])) *
                             length(unique(table[, factor_name_2]))),
                        times = max(as.numeric(table$row)))

    table$sequence <- table$sequence * width

    table$col <- as.numeric(table$col) * width
    table$row <- as.numeric(table$row) * height

    # plot subplots

    if (reverse_y == TRUE) {
      table$row <- abs(table$row - max(table$row)) +
        min(table$row)
    }

    if (reverse_x == TRUE) {
      table$col <- abs(table$col - max(table$col)) +
        min(table$col)

      table$sequence <- abs(table$sequence -
                              max(table$sequence)) + min(table$sequence)
    }

    if(subplots == TRUE){

    divider <- length(unique(table[, factor_name_1]))
    table$sequence <- table$sequence/divider

    plt2 <- ggplot(table, aes(x = .data[["sequence"]], y = .data[["row"]])) +
      geom_tile(
        aes(fill = .data[[factor_name_2]]),
        width = (width / divider) * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )


    return(plt2)
    } else {
      plt <- ggplot(table, aes(x = .data[["col"]], y = .data[["row"]])) +
        geom_tile(
          aes(fill = .data[[factor_name_1]]),
          width = width * space_width,
          height = height * space_height
        ) +
        theme_bw() +
        theme(line = element_blank()) +
        geom_text(
          aes(label = .data[[labels]]),
          colour = "black"
        )

      return(plt)

    }

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from a split plot design with a
                random complete block design (lsd) is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }
}

########## plot split plot design crd
########## #############

#' Plot Split Plot Designs (crd)
#'
#' Plot a design of a split plot experiment with a complete randomized design (crd) from design.split
#'
#' @param design outdesign from \code{agricolae} package
#' @param ncols Number of columns for the design
#' @param nrows Number of rows for the design
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#' @param factor_name_1 string Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param factor_name_2 string Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels string Describes the column from that the plots are taken to display them
#' @param subplots should the plot function return the subplots (default) or main plots?
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#' @import ggplot2
#' @import agricolae
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' T1<-c('a','b','c','d','e','f','g')
#' T2<-c('v','w','x','y','zzz')
#' r <- 4
#' outdesign2 <- design.split(trt1=T1, trt2=T2, r=r,
#' serie = 2, seed = 0, kinds = 'Super-Duper',
#' randomization=TRUE,first=TRUE,design = 'crd')
#' plot_split_crd(outdesign2,ncols = 6,nrows=5)
#'
#' outdesign2 <- design.split(trt1=T1, trt2=T2, r=r,
#' serie = 2, seed = 0, kinds = 'Super-Duper',
#' randomization=FALSE,first=TRUE,design = 'crd')
#' plot_split_crd(outdesign2,ncols = 6,nrows=5)

plot_split_crd <- function(design, nrows, ncols,
                           factor_name_1 = "T1",
                           factor_name_2 = "T2",
                           labels = "plots",
                           subplots = TRUE,
                           width = 1,
                           height = 1,
                           space_width = 0.95,
                           space_height = 0.85,
                           reverse_y = FALSE,
                           reverse_x = FALSE) {

  if (design$parameters$design == "split" && design$parameters$applied ==
      "crd") {
    column <- NULL
    test_input_ncols(ncols)
    test_input_nrows(nrows)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)
    test_input_extend(height)
    test_input_extend(width)
    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name_1, design)
    test_name_in_column(factor_name_2, design)
    test_name_in_column(labels, design)

    plots <- as.numeric(unique(design$book[, 1]))
    if (ncols * nrows >= length(plots)) {
      nas <- rep(NaN, (ncols * nrows) - length(plots))
      plots <- c(plots, nas)

      mat <- matrix(plots, byrow = TRUE, ncol = ncols)
      dims <- dim(mat)
      table <- as.table(mat)
      rownames(table) <- 1:dims[1]
      colnames(table) <- 1:dims[2]
      table <- as.data.frame(table)
      colnames(table) <- c("row", "col", "plots")
      table$row <- as.numeric(table$row)
      table$col <- as.numeric(table$col)

      table <- merge(table, design$book, by.x = "plots",
                     by.y = "plots")
      table$plots <- as.factor(table$plots)


      table$col <- table$col * width
      table$row <- table$row * height

      table <- table[with(table, order(plots,
                                       splots)), ]

      freq <- table(table$row)
      freq <- as.vector(freq)
      seqx <- function(x) {
        return(1:x)
      }
      sequence_numbers <- lapply(freq, seqx)

      table$sequence <- unlist(sequence_numbers)

      table$sequence <- table$sequence * width





      if (reverse_y == TRUE) {
        table$row <- abs(table$row - max(table$row)) +
          min(table$row)
      }

      if (reverse_x == TRUE) {
        table$col <- abs(table$col -
                              max(table$col)) + min(table$col)
        table$sequence <- abs(table$sequence -
                                max(table$sequence)) + min(table$sequence)
      }



      if(subplots == TRUE){

      divider <- length(unique(table[, factor_name_1]))
      table$sequence <- table$sequence/divider

      plt2 <- ggplot(table, aes(x = .data[["sequence"]], y = .data[["row"]])) +
        geom_tile(
          aes(fill = .data[[factor_name_2]]),
          width = (width * space_width) / divider,
          height = height * space_height
        ) +
        theme_bw() +
        theme(line = element_blank()) +
        geom_text(
          aes(label = .data[[labels]]),
          colour = "black"
        )

      return(plt2)
      } else {
        plt <- ggplot(table, aes(x = .data[["col"]], y = .data[["row"]])) +
          geom_tile(
            aes(fill = .data[[factor_name_1]]),
            width = width * space_width,
            height = height * space_height
          ) +
          theme_bw() +
          theme(line = element_blank()) +
          geom_text(
            aes(label = .data[[labels]]),
            colour = "black"
          )

        return(plt)

      }

    } else {
      stop(paste("You have in multiplication of ncols:",
                 ncols, "and nrows:", nrows, "Elements.",
                 "You need at minimum a product of both higher than",
                 length(plots)))
    }
  } else {
    stop(paste0("This is not the correct function for your experiment design.,
          A design from a split plot design with a
                random complete block design (crd) is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }
}


#' ggplot2 theme for outdoor presentation
#'
#' This theme is designed to increase font size to ensure readability on outdoor used devices
#' @return ggplot2 theme
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' T1<-c('a','b','c','d','e','f','g')
#' T2<-c('v','w','x','y','z')
#' r <- 4
#' outdesign2 <- design.split(trt1=T1, trt2=T2, r=r,
#' serie = 2, seed = 0, kinds = 'Super-Duper',
#' randomization=FALSE,first=TRUE,design = 'crd')
#' plot_split_crd(outdesign2,ncols = 6,nrows=5)+
#' theme_pres()
theme_pres <- function() {
  theme(
    text = element_text(size = 16, colour = "black"),
    axis.text = element_text(size = 18, colour = "black"),
    axis.title = element_text(size = 18, colour = "black"),
    axis.line = element_line(colour = "black",
                             linewidth = 1, linetype = "solid"),
    axis.ticks = element_line(colour = "black"),
    plot.background = element_rect(fill = "white", color = NA),
    panel.background = element_rect(fill = "white", color = NA)
  )
}


#' ggplot2 theme for poster presentation
#'
#' This theme is designed to increase font size to ensure readability on poster presentations
#' @return ggplot2 theme
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' T1<-c('a','b','c','d','e','f','g')
#' T2<-c('v','w','x','y','z')
#' r <- 4
#' outdesign2 <- design.split(trt1=T1, trt2=T2, r=r,
#' serie = 2, seed = 0, kinds = 'Super-Duper',
#' randomization=FALSE,first=TRUE,design = 'crd')
#' plot_split_crd(outdesign2,ncols = 6,nrows=5)+
#' theme_poster()
theme_poster <- function() {
  theme(
    text = element_text(size = 24, colour = "black"),
    axis.text = element_text(size = 28, colour = "black"),
    axis.title = element_text(size = 28, colour = "black"),
    axis.line = element_line(colour = "black",
                             linewidth = 1, linetype = "solid"),
    axis.ticks = element_line(colour = "black"),
    plot.background = element_rect(fill = "white", color = NA),
    panel.background = element_rect(fill = "white", color = NA)
  )
}


#' Plot Youden Design
#'
#' Plot a Youden experiment design from \code{agricolae} design.youden
#'
#' @param design outdesign from \code{agricolae} package
#' @param x Describes the x coordinates of a experiment design
#' @param y Describes the y coordinates of a experiment design
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? Use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#' @param factor_name string Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels string Describes the column from that the plots are taken to display them.
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#' @import ggplot2
#' @import agricolae
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' varieties<-c('perricholi','yungay','maria bonita','tomasa')
#' outdesign <-design.youden(varieties,r=2,serie=2,seed=23)
#' plot_youden(outdesign, labels = 'varieties')
plot_youden <- function(design, x = "col", y = "row",
                        factor_name = "varieties", labels = "plots", width = 1,
                        height = 1, space_width = 0.95, space_height = 0.85,
                        reverse_y = FALSE, reverse_x = FALSE) {
  if (design$parameters$design == "youden") {

    test_string(x)
    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_name_in_column(factor_name, design)
    test_name_in_column(labels, design)

    table <- as.data.frame(design$book)

    table[, x]  <- as.numeric(table[, x] ) * width
    table[, y] <- as.numeric(table[, y]) * height

    if (reverse_y == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }
    if (reverse_x == TRUE) {
      table[, x]  <- abs(table[, x]  - max(table[, x] )) +
        min(table[, x] )
    }


    plt <- ggplot(table, aes(x = .data[[x]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )


    return(plt)

  } else {
    stop(paste0("This is not the correct function for your experiment design.
          A design from a Youden Design is needed here.",
                "you have a design of type ", design$parameters$design,
                " and an applied type of ", design$parameters$applied,
                "."))
  }

}

#' Measures of a Field Design
#'
#' Returns a list with several useful information about the experiment
#'
#' @param p \code{ggplot} object containing the data of the plot
#'
#' @return a list with several useful information about the experiment and the field
#' @export
#'
#' @examples
#' library(agricolae)
#' library(agricolaeplotr)
#' trt = c(2,3,4,5,6)
#' outdesign1 <-design.crd(trt,r=5,serie=2,2543,'Mersenne-Twister')
#' p <- plot_design_crd(outdesign1,
#'              ncols = 7,
#'              nrows = 4,
#'              width = 10,
#'              height = 10,
#'              reverse_y = TRUE)
#' stats <- DOE_obj(p)
#' stats
DOE_obj <- function(p){

  res <- list()
  dat <- layer_data(p)

  res$eff_width <- diff(range(dat$xmin,dat$xmax)) # nettoweite versuch
  res$eff_height <- diff(range(dat$ymin,dat$ymax)) # nettohoehe versuch
  res$eff_area_total <- res$eff_height * res$eff_width

  res$eff_width_plot <- dat$width[1] # netto weite parzelle
  res$eff_height_plot <- dat$height[1] # netto hoehe parzelle
  res$eff_plot_size <- res$eff_width_plot * res$eff_height_plot # netto groesse je parzelle
  res$eff_total_area <- res$eff_plot_size * dim(dat)[1] # netto Nutzfl?che unter Parzelle

  res$gross_width_plot <- min(dat$x) # bruttobreite Parzelle
  res$gross_height_plot <- min(dat$y) # bruttohoehe parzelle
  res$total_area_plot <- res$gross_width_plot * res$gross_height_plot # bruttoflaeche parzelle
  res$outer_area <- diff(range(c(dat$xmin,dat$xmax))) * diff(range(c(dat$ymin,dat$ymax)))

  res$rel_space_width <- 1 - ((res$gross_width_plot - res$eff_width_plot) / (res$gross_width_plot))
  res$rel_space_height <- 1 - ((res$gross_height_plot - res$eff_height_plot) / (res$gross_height_plot))

  res$n_plots <- dim(dat)[1]
  res$n_cols <- length(unique(dat$x))
  res$n_rows <- length(unique(dat$y))
  res$total_area <- res$total_area_plot * res$n_rows * res$n_cols# bruttofl?che versuch
  res$space_between <- res$total_area - res$eff_total_area

  res$share_eff_plot <- res$eff_plot_size / res$total_area_plot
  res$share_space_plot <- 1 - res$share_eff_plot

  res$abs_space_height <- res$gross_height_plot - res$eff_height_plot
  res$abs_space_width <- res$gross_width_plot - res$eff_width_plot

  res$net_plot_diagonal <- signif(sqrt(res$eff_height_plot^2 + res$eff_width_plot^2),3)
  res$gross_plot_diagonal <- signif(sqrt(res$gross_height_plot^2 + res$gross_width_plot^2),3)
  res$experiment_diagonal <- signif(sqrt(res$eff_width^2 + res$eff_width^2),3)
  res$outer_diagonal <- signif(sqrt(diff(range(c(dat$xmin,dat$xmax)))^2 + diff(range(c(dat$ymin,dat$ymax)))^2),3)

  res$xmin <- min(dat$xmin)
  res$xmax <- max(dat$xmax)
  res$ymin <- min(dat$ymin)
  res$ymax <- max(dat$ymax)
  res$n_fac <- length(unique(dat$fill))
  class(res) <- "FieldLayout"
  res
}

#' summary of a field Layout
#'
#' print a summary of a FieldLayout object
#' @param object an object, created by DOE_obj with a FieldLayout class
#' @param unit a string that corresponds to measure unit (default is m)
#' @param part which part of the summary are you interested?
#' Choose one of the following: "net_plot","gross_plot","field","experiment","all"
#' @param ... further arguments passed to or from other methods
#'
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' varieties<-c('perricholi','yungay','maria bonita','tomasa')
#' outdesign <-design.youden(varieties,r=2,serie=2,seed=23)
#' p <- plot_youden(outdesign, labels = 'varieties')
#' stats <- DOE_obj(p)
#' # print plot summary for net plot (plots without space)
#' summary(stats, part = "net_plot")
#' # print plot summary for gross plot (plots with space)
#' summary(stats, part = "gross_plot")
#' # print plot summary for entire field
#' summary(stats, part = "field")
#' # print plot summary for design summary
#' summary(stats, part = "experiment")
#' # print plot summary for all information shown above in one output
#' summary(stats, part = "all")
###FieldLayout <- function(object,unit,part,...) S3method(summary, FieldLayout)
summary <- function(object, unit="m", part="net_plot",...){
  if(!inherits(object,"FieldLayout")){
    stop("This is not the right class for this kind of summary. You need to use an object from a class \"FieldLayout\".")
  }
  x <- object
  if(!(part  %in% c("net_plot","gross_plot","field","experiment","all"))){
    stop(paste("part parameter needs to be one of the following: net_plot, gross_plot, field, all. You have typed",part))
  }
  if(part %in% c("net_plot","all")){

    print(paste("net plot height:",x$eff_height_plot, unit))
    print(paste("net plot width:",x$eff_width_plot, unit))
    print(paste("net plot diagonal:",x$net_plot_diagonal, unit))
    print(paste("net plot area:",x$eff_plot_size, paste0(unit,"^2")))
    print(paste("share used plot area:",x$share_eff_plot))
    print(paste("share space between plots:",x$share_space_plot))
    print(paste("space_width:",x$abs_space_height, unit))
    print(paste("space_height:",x$abs_space_width, unit))
  }

  if(part %in% c("gross_plot","all")){

    print(paste("gross plot height:",x$gross_height_plot, unit))
    print(paste("gross plot width:",x$gross_width_plot, unit))
    print(paste("gross plot diagonal:",x$gross_plot_diagonal, unit))
    print(paste("gross plot area:",x$total_area_plot,paste0(unit,"^2")))

    print(paste("share used plot area:",x$share_eff_plot))
    print(paste("share space between plots:",x$share_space_plot))
    print(paste("space_width:",x$abs_space_height, unit))
    print(paste("space_height:",x$abs_space_width, unit))
    print(paste("gross space area:",x$space_between, paste0(unit,"^2")))
  }

  if(part %in% c("field","all")){

    print(paste("relative design height:",x$rel_space_height))
    print(paste("relative design width:",x$rel_space_width))
    print(paste("net experiment diagonal:",x$experiment_diagonal, unit))
    print(paste("net experiment width:",x$eff_width, unit))
    print(paste("net experiment height:",x$eff_height, unit))
    print(paste("used plot area:",x$eff_total_area,paste0(unit,"^2")))
    print(paste("used area DOE:",x$outer_area,paste0(unit,"^2")))
    print(paste("used outer area:",x$total_area,paste0(unit,"^2")))
    print(paste("outer field diagonal:",x$outer_diagonal, unit))
  }
  if(part %in% c("experiment","all")){

    print(paste("xmin:", x$xmin))
    print(paste("xmax:", x$xmax))
    print(paste("ymin:", x$ymin))
    print(paste("ymax:", x$ymax))
    print(paste("number columns:",x$n_cols))
    print(paste("number rows:",x$n_rows))
    print(paste("number of plots:",x$n_plots))
    print(paste("number of factors:",x$n_fac))
  }
}

#' to_table
#'
#' Write field experiment information to a dataframe.
#' @param object an object, created by DOE_obj with a FieldLayout class
#' @param unit a string that corresponds to measure unit (default is m)
#' @param part which part of the summary are you interested?
#' Choose one of the following: "net_plot","gross_plot","field","experiment"
#' @param digits integer indicating the number of decimal places (round)
#' or significant digits (signif) to be used. Negative values are allowed
#' @param ... further arguments passed to or from other methods
#'
#' @return dataframe with corresponding information about the experiment
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' varieties<-c('perricholi','yungay','maria bonita','tomasa')
#' outdesign <-design.youden(varieties,r=2,serie=2,seed=23)
#' p <- plot_youden(outdesign, labels = 'varieties', width=4, height=3)
#' stats <- DOE_obj(p)
#' r <- to_table(stats,part = "net_plot", digits = 2)
#' r
#' r <- to_table(stats,part = "gross_plot", digits = 2)
#' r
#' r <- to_table(stats,part = "field", digits = 2)
#' r
#' r <- to_table(stats,part = "experiment", digits = 2)
#' r
#' r <- to_table(stats,part = "all", digits = 2)
#' r
to_table <- function(object,part="net_plot",unit="m",digits=3,...){
  if (!inherits(object,"FieldLayout")){
    stop("The object needs to be from the class 'agricolaeplotr'")
  }
  x <- unclass(object)
  if(!(part  %in% c("net_plot","gross_plot","field","experiment","all"))){
    stop(paste("part parameter needs to be one of the following: net_plot, gross_plot, field, experiment. You have typed",part))
  }
  if(part %in% ("net_plot")){
    df1 <- data.frame(names=rep(0,8))
    df1$names <- c(paste("net plot height:", unit),
                  paste("net plot width:", unit),
                  paste("net plot diagonal:", unit),
                  paste("net plot area:",paste0(unit,"^2")),
                  paste("share used plot area:"),
                  paste("share space between plots:"),
                  paste("space_width:", unit),
                  paste("space_height:",unit))

    df1$vals <- c(x$eff_height_plot,x$eff_width_plot,x$net_plot_diagonal,
                 x$eff_plot_size,x$share_eff_plot,x$share_space_plot,
                 x$abs_space_height,x$abs_space_width)

    df1$vals <- signif(df1$vals,digits=digits)
    return(df1)
  }
  if(part %in% ("gross_plot")){

    df2 <- data.frame(names=rep(0,9))

    df2$names <- c(paste("gross plot height:", unit),
                  paste("gross plot width:", unit),
                  paste("gross plot diagonal:", unit),
                  paste("gross plot area:",paste0(unit,"^2")),
                  paste("gross space area:", paste0(unit,"^2")),

                  paste("share used plot area: %/100"),
                  paste("share space between plots: %/100"),
                  paste("space_width:", unit),
                  paste("space_height:", unit))

    df2$vals <- c(x$gross_height_plot,x$gross_width_plot,x$gross_plot_diagonal,
                 x$total_area_plot,x$space_between,x$share_eff_plot,x$share_space_plot,
                 x$abs_space_height,x$abs_space_width)

    df2$vals <- signif(df2$vals,digits=digits)
    return(df2)
  }


  if(part %in% ("field")){

    df3 <- data.frame(names=rep(0,9))

    df3$names <- c(paste("relative design height:"),
                  paste("relative design width:"),
                  paste("net experiment diagonal:", unit),
                  paste("net experiment width:", unit),
                  paste("net experiment height:", unit),
                  paste("used plot area:",paste0(unit,"^2")),
                  paste("used area DOE:",paste0(unit,"^2")),
                  paste("used outer area:",paste0(unit,"^2")),
                  paste("outer field diagonal:", unit))


    df3$vals <- c(x$rel_space_height,x$rel_space_width,x$experiment_diagonal,x$eff_width,
                 x$eff_height,x$eff_total_area,x$outer_area,x$total_area,x$outer_diagonal)

    df3$vals <- signif(df3$vals,digits=digits)
    return(df3)
  }
  if(part %in% ("experiment")){


    df4 <- data.frame(names=rep(0,8))

    df4$names <-c("xmin:",
                 "xmax:",
                 "ymin:",
                 "ymax:",
                 "number columns:",
                 "number rows:",
                 "number of plots:",
                 "number of factors:")

    df4$vals <- c(x$xmin,
                 x$xmax,
                 x$ymin,
                 x$ymax,
                 x$n_cols,
                 x$n_rows,
                 x$n_plots,
                 x$n_fac)

    df4$vals <- signif(df4$vals,digits=digits)
    return(df4)
  }
  if(part %in% ("all")){
    df5 <- data.frame(names=rep(0,30))
    df5$names <- c(paste("net plot height:", unit),
                   paste("net plot width:", unit),
                   paste("net plot diagonal:", unit),
                   paste("net plot area:",paste0(unit,"^2")),
                   paste("share used plot area:"),
                   paste("share space between plots:"),
                   paste("space_width:", unit),
                   paste("space_height:",unit),
                   paste("gross plot height:", unit),
                   paste("gross plot width:", unit),
                   paste("gross plot diagonal:", unit),
                   paste("gross plot area:",paste0(unit,"^2")),
                   paste("gross space area:", paste0(unit,"^2")),
                   paste("relative design height:"),
                   paste("relative design width:"),
                   paste("net experiment diagonal:", unit),
                   paste("net experiment width:", unit),
                   paste("net experiment height:", unit),
                   paste("used plot area:",paste0(unit,"^2")),
                   paste("used area DOE:",paste0(unit,"^2")),
                   paste("used outer area:",paste0(unit,"^2")),
                   paste("outer field diagonal:", unit),
                   "xmin:",
                   "xmax:",
                   "ymin:",
                   "ymax:",
                   "number columns:",
                   "number rows:",
                   "number of plots:",
                   "number of factors:")


    df5$vals <- c(x$eff_height_plot,x$eff_width_plot,x$net_plot_diagonal,
                  x$eff_plot_size,x$share_eff_plot,x$share_space_plot,
                  x$abs_space_height,x$abs_space_width,
                  x$gross_height_plot,x$gross_width_plot,x$gross_plot_diagonal,
                  x$total_area_plot,x$space_between,

                  x$rel_space_height,x$rel_space_width,x$experiment_diagonal,x$eff_width,
                  x$eff_height,x$eff_total_area,x$outer_area,x$total_area,x$outer_diagonal,
                  x$xmin,
                  x$xmax,
                  x$ymin,
                  x$ymax,
                  x$n_cols,
                  x$n_rows,
                  x$n_plots,
                  x$n_fac)
    df5$vals <- signif(df5$vals,digits=digits)
    return(df5)

  }




   }


#' make_polygons
#'
#' This function coerces all rectangles
#' from a 'ggplot' object to 'SpatialPolygonDataFrame'.
#' @param ggplot_object saved ggplot object, containing the
#' coordinates of the rectangles of a 'ggplot' object of the first two layers
#' @param north float added to the rows
#'  to have a northing ordinate
#' @param east float added to the rows
#'  to have a easting ordinate
#' @param projection_input string defines
#'  in which EPSG projection the ggplot object should be converted to a raster object?
#'  a projection with a metric unit is highly recommended
#' @param projection_output string defines
#'  in which EPSG projection the SpatialPolygonDataFrame should be exported.
#' @importFrom raster extent crs
#' @export
#' @return a SpatialPolygonDataFrame object
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' trt = c(2,3,4)
#' outdesign1 <-design.crd(trt,r=5,serie=2,2543,'Mersenne-Twister')
#' plt <- plot_design_crd(outdesign1,ncols = 13,nrows = 3)
#' spat_df <- make_polygons(plt)
#' spat_df
make_polygons <- function(ggplot_object,
                          north = 3454206.89,
                          east = 5939183.21,
                          projection_input = "+init=epsg:31467",
                          projection_output = "+init=epsg:4326") {
  table_object <- merge(ggplot2::layer_data(ggplot_object,1),
                        ggplot2::layer_data(ggplot_object,2),
                        by.x=c("x","y"), by.y=c("x","y"))
  bounds = table_object[, c(7, 6, 9, 8)]
  bounds$xmax <- bounds$xmax + north
  bounds$xmin <- bounds$xmin + north

  bounds$ymax <- bounds$ymax + east
  bounds$ymin <- bounds$ymin + east


  polygons_list = apply(bounds, 1, function(x) {
    out = raster::extent(x[c(2, 1, 4, 3)])
    out = methods::as(out, 'SpatialPolygons')
    out = methods::as(out, 'SpatialPolygonsDataFrame')
    raster::crs(out) <- sp::CRS(projection_input)
    out
  })


  polygons_list = do.call(rbind, polygons_list)
  polygons_list@data <- table_object

  polygons_list <- sf::st_as_sf(polygons_list)
  # polygons_list <- sp::spTransform(polygons_list, sp::CRS(projection_output))
  polygons_list <- sf::st_transform(polygons_list, crs = sf::st_crs(projection_output))
  return(polygons_list)
}

#' theme_gi
#'
#' Creates a theme for 'ggplot' based graphics to ensure
#' to meet formal requirements for conferences of the
#' Gesellschaft fuer Informatik
#'
#' @return a 'ggplot' graph with a modified theme
#' @export
#'
#' @examples
#' # example borrowed from ggplot2
#' library(ggplot2)
#' df <- data.frame(
#' gp = factor(rep(letters[1:3], each = 10)),
#' y = rnorm(30))
#'
#' p <- ggplot() +
#' geom_point(data = df, aes(gp, y))
#' p <- p + theme_gi();p
theme_gi <- function(){
  theme_bw() +
    theme(text=element_text(size = 10, angle = 0))
    theme(axis.text = element_text(colour = "black"))+
    theme(axis.title = element_text(hjust = 0.5))
}



#' Plot FielDHub Design
#'
#' Plots designs from \code{FielDHub} package
#' @param design outdesign from \code{FielDHub} package with on of the following IDs: c(9,13,14,15,16)
#' @param x Describes the x coordinates of a experiment design
#' @param y Describes the y coordinates of a experiment design
#' @param labels string Describes the column from that the plots are taken to display them
#' @param factor_name string Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#' @param shift_x numeric indicates the shift in units in x-axis.
#' @param shift_y numeric indicates the shift in units for the y-axis.

#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#' @import ggplot2
#' @import FielDHub
#' @examples
#'\dontrun{
#' library(agricolaeplotr)
#' library(FielDHub)
#'H <- paste("H", 1:4, sep = "")
#'V <- paste("V", 1:5, sep = "")
#'
#'strip1 <- FielDHub::strip_plot(Hplots = H,
#'                               Vplots = V,
#'                               b = 1,
#'                               l = 1,
#'                               plotNumber = 101,
#'                               planter = "serpentine",
#'                               locationNames = "A",
#'                               seed = 333)
#'
#'
#'strip1$fieldBook$ROW <- as.numeric(ordered(strip1$fieldBook$VSTRIP,
#'                        levels = unique(strip1$fieldBook$VSTRIP)))
#'strip1$fieldBook$COLUMN <- as.numeric(ordered(strip1$fieldBook$HSTRIP,
#'                        levels = unique(strip1$fieldBook$HSTRIP)))
#'
#' plot_fieldhub(strip1,
#'               x = "ROW",
#'               y = "COLUMN",
#'               labels = "HSTRIP",
#'               factor_name = "HSTRIP",
#'               width = 12,
#'               height = 10,
#'               reverse_y = FALSE,
#'               reverse_x = FALSE)
#'
#'}
#'
plot_fieldhub <- function(design,
                          x = "COLUMN",
                          y = "ROW",
                          labels = "PLOT",
                          factor_name = "TREATMENT",
                          width = 1,
                          height = 1,
                          space_width = 0.95,
                          space_height = 0.85,
                          reverse_y = FALSE,
                          reverse_x = FALSE,
                          shift_x=0,
                          shift_y=0) {
  if (!(design$infoDesign$id_design %in% c(1,2,3,4,7,8,9,10,11,12,13,14,15,16)))
  {
    stop(paste0("This is not the correct function for your experiment design
                or is not supported yet."))

  } else {print("Your Design ID is supported.")}

    design$book <- design$fieldBook
    test_string(x)
    test_string(y)

    test_input_reverse(reverse_x)
    test_input_reverse(reverse_y)

    test_input_extend(height)
    test_input_extend(width)

    test_input_extend(space_height)
    test_input_extend(space_width)

    test_input_shift(shift_x)
    test_input_shift(shift_y)

    test_name_in_column(labels, design)

    table <- as.data.frame(design$book)
    table[, x]  <- as.numeric(table[, x] ) * width + shift_x
    table[, y] <- as.numeric(table[, y]) * height + shift_y
    if (reverse_y == TRUE) {
      table[, y] <- abs(table[, y] - max(table[, y])) +
        min(table[, y])
    }
    if (reverse_x == TRUE) {
      table[, x]  <- abs(table[, x]  - max(table[, x] )) +
        min(table[, x] )
    }
    plt <- ggplot(table, aes(.data[[x]], .data[[y]])) +
      geom_tile(aes(fill = .data[[factor_name]]),
                width = width * space_width, height = height *
                  space_height) + theme_bw() + theme(line = element_blank()) +
      geom_text(aes(label = .data[[labels]]), colour = "black")# +
    # + facet_grid(reformulate(replication_name,location_name))

    plt

    return(plt)

  }



#' Serpentine
#'
#' This function produces a serpentine array of integers beginning by one
#'
#' @param n integer value indicating the upper cap of a numeric sequence
#' @param times integer number of replications
#' @param m integer value indicating the lower
#' cap of a numeric sequence
#'
#' @return vector containing the serpentine sequence
#' @export
#'
#' @examples
#' serpentine(n=20,times = 15)
#' serpentine(n=20,times = 15,m=4)
serpentine <- function(n,times,m=1){
  vec= vector()
  for(i in 1:times){
    if(i %% 2 == 1){
      vec <- cbind(vec,m:n)
    }
    else{
      vec <- cbind(vec,n:m)
    }
  }
  return(as.vector(vec))
}

#' full_control_positions
#'
#' This function provides full control about the plotting. The user also may shift the coordinates as liked.
#'
#' @param design data.frame containing the row and columns of an experiment
#' @param x Describes the x coordinates of a experiment design
#' @param y Describes the y coordinates of a experiment design
#' @param width numeric value, describes the width of a plot in an experiment
#' @param height numeric value, describes the height of a plot in an experiment
#' @param space_width numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width
#' @param space_height numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height
#' @param reverse_y boolean, should the plots of the experiment be changed in reverse order in row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE
#' @param reverse_x boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE
#' @param factor_name string Which factor should be used for plotting, needs to be a column in outdesign$book
#' @param labels string Describes the column from that the plots are taken to display them
#' @param way_x numeric vector indicates the shift of the nth-plot in x-axis.
#' @param way_y numeric vector indicates the shift of the nth-plot in y-axis.
#' @param shift_x numeric indicates the shift in units in x-axis.
#' @param shift_y numeric indicates the shift in units for the y-axis.
#' @param dist_x numeric indicates the shift in plots in x-axis.
#' @param dist_y numeric indicates the shift in plots for the y-axis.
#' @param start_origin boolean. Should the design start at the origin (0|0)?
#' @param shift_columns numeric indicates the shift of the given plots of a specific row by n units in x-axis.
#' @param shift_rows numeric indicates the shift of the given plots of a specific column by n units in  y-axis.
#' @param n_shift_columns numeric vector indicating the number of plots of shift_columns. negative number indicate shift to left, otherwise right
#' @param n_shift_rows numeric vector indicating the number of plots of shift_rows. negative number indicate shift to left, otherwise right
#'
#' @return \code{ggplot} graphic that can be modified, if wished
#' @export
#' @import ggplot2
#' @import agricolae
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' library(ggplot2)
#' varieties<-c('perricholi','yungay','maria bonita','tomasa')
#' outdesign <-design.youden(varieties,r=2,serie=2,seed=23)
#' design <- outdesign$book
#' design
#' p <- full_control_positions(design,"col","row","varieties","plots",
#'                        width=3,height=4.5,
#'                        space_width=0.5,space_height=0.5,
#'                        shift_x=(-0.5*3) + (-0.5*3*0.5),shift_y=-0.5*4.5 + (-0.5*4.5*0.5))
#' p
#' p <- full_control_positions(design,"col","row","varieties","plots",
#'                        width=3,height=4.5,
#'                        space_width=0.13,space_height=0.445,
#'                        shift_x=(-0.5*3) + (-0.5*3*(1-0.13)),shift_y=-0.5*4.5 + (-0.5*4.5*(1-0.445)))
#'                        p
#'
#' varieties<-LETTERS[1:12]
#' outdesign <-design.youden(varieties,r=12,serie=2,seed=23)
#' design <- outdesign$book
#' p <- full_control_positions(design,"col","row","varieties","plots",
#'                        width=3,height=4.5,
#'                        space_width=1,space_height=1,
#'                        shift_x=-0.5*3,shift_y=-0.5*4.5)
#' p
#'
#' p <- full_control_positions(design,"col","row","varieties","plots",
#'                        width=3,height=4.5,
#'                        space_width=0.93,space_height=0.945,
#'                        start_origin = TRUE)
#'                        p
#'
#' p <- full_control_positions(design,"col","row","varieties","plots",
#' width=3,height=4.5,
#'space_width=0.93,space_height=0.945,way_x = c(2,6,8,10,12),way_y=c(3,8),dist_x=2,dist_y=4,
#'start_origin = TRUE, reverse_y = FALSE,  reverse_x = FALSE);p
#'
#'p <- full_control_positions(design,"col","row","varieties","plots",
#'                                     width=3,height=4.5,
#'                                     space_width=0.93,space_height=0.945,
#'                                     way_x = c(2,4,6,8,10,12),way_y=c(3,8),
#'                                     start_origin = FALSE, reverse_y = FALSE,
#'                                     reverse_x = FALSE);p
#'p <- full_control_positions(design,"col","row","varieties","plots",
#'                                     width=3,height=4.5,shift_columns=c(4,8),
#'                                     shift_rows=c(3,5,9),
#'                                     n_shift_columns=c(1,5),
#'                                     n_shift_rows=c(1,-2,6),
#'                                     space_width=0.93,space_height=0.945,
#'                                     way_x = c(2,4,6,8,10,12),way_y=c(3,8),
#'                                     start_origin = TRUE, reverse_y = FALSE,
#'                                     reverse_x = FALSE);p

full_control_positions <- function(design,
                                   x = "col",
                                   y = "row",
                                   factor_name = "trt",
                                   labels = "plots",
                                   width = 1,
                                   height = 1,
                                   space_width = 0.95,
                                   space_height = 0.85,
                                   reverse_y = FALSE,
                                   reverse_x = FALSE,
                                   way_x=0,
                                   way_y=0,
                                   shift_columns=0,
                                   shift_rows=0,
                                   shift_x=0,
                                   shift_y=0,
                                   dist_x=1,
                                   dist_y=1,
                                   n_shift_columns=0,
                                   n_shift_rows=0,
                                   start_origin=FALSE) {

  test_string(x)
  test_string(y)

  test_input_reverse(reverse_x)
  test_input_reverse(reverse_y)

  test_input_extend(height)
  test_input_extend(width)

  test_input_extend(space_height)
  test_input_extend(space_width)

  test_input_shift(shift_x)
  test_input_shift(shift_y)

  test_input_shift(way_x)
  test_input_shift(way_y)

  test_input_shift(dist_x)
  test_input_shift(dist_y)

  test_input_shift(n_shift_columns)
  test_input_shift(n_shift_rows)

  test_input_shift(shift_columns)
  test_input_shift(shift_rows)

  test_input_reverse(start_origin)


  table <- design
  n_vec <- rep(n_shift_columns,length=length(shift_columns))
  m_vec <- rep(n_shift_rows,length=length(shift_rows))

  table[, x]  <- as.numeric(table[, x] )
  table[, y] <- as.numeric(table[, y])

      l <- 1
    for( i in shift_columns){
      table[table[,x] == i,y] = table[table[,x] == i,y] + n_vec[l]
      l <- l + 1
    }


      l <- 1
    for( i in shift_rows){
      table[table[,y] == i,x] = table[table[,y] == i,x] + m_vec[l]
      l <- l + 1
    }

    for (i in way_x ){
        table[, x] <- ifelse(table[, x] > (i + (match(i,way_x) - 1)), table[, x] + dist_x, table[, x])
    }


    for (i in way_y ){
      table[, y] <- ifelse(table[, y] > (i + (match(i,way_y) - 1)), table[, y] + dist_y, table[, y])
    }

      if(start_origin == TRUE){
        shift_x <- width * -0.5 + (width * -0.5 * (1-space_width)) ## makes zero
        shift_y <- height * -0.5 + (height * -0.5 * (1-space_height)) ## makes zero
      }

    table[, x]  <- table[, x] * width + shift_x
    table[, y] <- table[, y] * height + shift_y


  if (reverse_y == TRUE) {
    table[, y] <- abs(table[, y] - max(table[, y])) +
      min(table[, y])
  }
  if (reverse_x == TRUE) {
    table[, x]  <- abs(table[, x]  - max(table[, x] )) +
      min(table[, x] )
  }
    plt <- ggplot(table, aes(x = .data[[x]], y = .data[[y]])) +
      geom_tile(
        aes(fill = .data[[factor_name]]),
        width = width * space_width,
        height = height * space_height
      ) +
      theme_bw() +
      theme(line = element_blank()) +
      geom_text(
        aes(label = .data[[labels]]),
        colour = "black"
      )

  plt

  return(plt)
}

#' Citation
#'
#' Generates citations of all loaded packages
#'
#' @param includeURL boolean, Should the URL be returned?
#' @param bibtex boolean, Should the citations be returned as bibtex?
#'
#' @return printed output to console
#' @export
#'
#' @examples
#' library(ggplot2)
#' library(agricolaeplotr)
#' library(agricolae)
#' library(raster)
#' citations()
citations <- function(includeURL = TRUE, bibtex=TRUE) {


  cit.list <- c('base', names(utils::sessionInfo()$otherPkgs))
  for(i in seq_len(length(cit.list))) {
    ref <- utils::citation(cit.list[i])
    if(includeURL == FALSE) {
      ref$url <- NULL
    }
    if (bibtex==TRUE){
      print(utils::toBibtex(ref))}else
      {
        print(ref, style = 'text')
      }
    cat('\n')
  }
}

############ sampling locations per plot #########

#' Sample Locations
#'
#' Returns locations to sample for each plot.
#'
#' This function takes an experiment design (plot layout) and returns random sample
#' locations within each plot. The function uses the `sf` package to generate
#' spatial polygons for the plots and then samples points within each polygon.
#' Optionally, it can also display the sample locations as a ggplot2-based map.
#'
#' @param design Your experiment design of plot layouts.
#' @param n Number of samples per plot (integer).
#' @param plot Logical, indicating whether to visualize the sample locations as a ggplot2-based map.
#' @param ... further options for `st_sample` and `make_polygons`
#'
#' @return An `sf` object containing the sample locations within each plot.
#' @export
#'
#'
#' @examples
#' library(agricolaeplotr)
#' library(agricolae)
#' library(ggplot2)
#' trt <- c('A', 'B', 'C', 'D')
#' k <- 3
#' outdesign <- design.bib(trt, k, serie = 2, seed = 41, kinds = 'Super-Duper')
#' plot_bib(outdesign)
#' p <- plot_bib(outdesign)
#' sample_locations(p, 3, TRUE, projection_output = 25832)
#'
#' @importFrom sf st_sample
#' @importFrom ggplot2 geom_sf ggplot
sample_locations <- function(design, n, plot = TRUE, ...) {
  # Create spatial polygons for the design
  doe <- make_polygons(design,...)

  # Sample random points within each polygon
  points <- sf::st_sample(doe, size = c(n, n), type = "random", exact = TRUE,...)

  # Optionally, visualize the sample locations as a ggplot2-based map
  if (plot) {
    p <- ggplot2::ggplot(points) +
      ggplot2::geom_sf(data = doe, aes(fill = .data[["fill"]])) +
      ggplot2::geom_sf()
    print(p)
  }

  return(points)
}




#' Plot the longest diagonal of a field
#'
#' This function takes a field and plots the longest diagonal of the field. The field is divided into segments and points are sampled from these segments.
#'
#' @param field An object of class sf representing the field.
#' @param n Integer, the number of sample points along the longest diagonal.
#' @param type Type of sampling. Default is "random".
#' @param n_segments Numeric, the number of segments to divide the longest diagonal (default is 2).
#' @param distance_field_boundary Numeric, the distance to buffer the field for creating the boundary (default is 3.0).
#' @param width_diagonal_path Numeric, the width to buffer the diagonal path (default is 2.0).
#' @export
#'
#' @return
#' \itemize{
#' \item p: A ggplot object showing the field, the buffered field, the buffered line, and the sample points.
#' \item buffered_line: A sf object representing the buffered line.
#' \item my_line: A sf object representing the longest diagonal of the field.
#' \item sample_points: A sf object representing the sampled points.
#' \item length: A numeric value, representing the length of the longest line.
#' }
#'
#' @examples
#' library(sf)
#' my_sf <- st_read(system.file("shape/gfn_schlaege.shp", package="agricolaeplotr"))
#' st_crs(my_sf) <- 25832
#' field <- my_sf[my_sf$SCHLAG_NR == 170,]
#' plot_longest_diagonal(field)
#' @import ggplot2
#' @import sf
#' @importFrom dplyr filter `%>%`
#' @importFrom tidyr gather
#' @importFrom stplanr line_segment
#' @importFrom stats dist
#' @importFrom tibble as_tibble rownames_to_column
#' @importFrom ggspatial annotation_scale annotation_north_arrow
plot_longest_diagonal <- function(field,n=8,type="random",n_segments=2,distance_field_boundary=3.0, width_diagonal_path=2){

  buffered_field <- st_buffer(field,dist = -distance_field_boundary, joinStyle  = "MITRE", mitreLimit = 2,endCapStyle = "ROUND")
  buffered_field_plot <- st_buffer(field,dist = -(distance_field_boundary + distance_field_boundary), joinStyle  = "MITRE", mitreLimit = 2,endCapStyle = "ROUND")
  buffered_field_plot2 <- st_buffer(field,dist = -(width_diagonal_path), joinStyle  = "MITRE", mitreLimit = 2,endCapStyle = "ROUND")

  spat <- as_Spatial(buffered_field_plot$geometry)

  field_boundary <- spat@polygons[[1]]@Polygons[[1]]@coords

  d <- stats::dist(field_boundary) %>% as.matrix() %>%
    as_tibble() %>%
    rownames_to_column(var = "start_node") %>%
    gather(end_node, dist, -start_node) %>%
    filter(dist != 0)

  coords <- d[d$dist == max(d$dist),][1]
  points <- data.frame(field_boundary)[coords$start_node,]
  points <- matrix(unlist(points), ncol = 2, byrow = FALSE)
  points <- unique(points)

  my_line <- st_linestring(points)
  my_line <- st_sfc(my_line,crs = st_crs(field))

  di <- st_as_sf(my_line)
  seg <- stplanr::line_segment(di,n_segments=n_segments)
  buffered_line <- st_buffer(seg,dist = width_diagonal_path, endCapStyle = "FLAT",singleSide=FALSE,mitreLimit = 0.00003)

  sample_points <- st_line_sample(seg,n=n,type=type)

  p <- ggplot() +
    geom_sf(data=field,fill="orange") +
    geom_sf(data=buffered_field_plot2,fill="blue") +
    geom_sf(data=buffered_line) + theme_minimal()+
    geom_sf(data=sample_points,color="red")

  p

  ##### ab hier mehr neues
  len <- st_length(di)
  print(paste("Length of line:",len,units(len)$numerator))

  p <- ggplot() +
    geom_sf(data=field,fill="orange") +
    geom_sf(data=buffered_field_plot,fill="blue") +
    geom_sf(data=buffered_line) + theme_minimal()+
    geom_sf(data=sample_points,color="red")

  p <- p +
    annotation_scale(
      location = "tl",
      bar_cols = c("grey60", "white")
    ) +
    annotation_north_arrow(
      location = "tl", which_north = "true",
      pad_x = unit(0.4, "in"), pad_y = unit(0.4, "in"),
      style = ggspatial::north_arrow_nautical(
        fill = c("grey40", "white"),
        line_col = "grey20"
      )
    )

  return(list(p,buffered_line,my_line,sample_points,len))
}

utils::globalVariables(c("end_node","start_node"))


#' Create Protective Layers for Design of Experiments (DOEs)
#'
#' This function generates protective layers around the polygons of an experiment.
#' These layers can be used to plot boundaries, for example, to protect agricultural
#' on-farm experiments from accidental harvesting.
#'
#' @param design An \code{sf} object containing the polygons of the experiment.
#'               The coordinate reference system (crs) of the data needs to be in
#'               metric distance, not degrees.
#' @param borders A numeric vector specifying the distances (in meters) for which
#'                protective layers should be created. The layers will be created
#'                with decreasing distances, starting from the largest.
#'
#' @return An \code{sf} object representing the protective layers around the
#'         experiment polygons.
#' @export
#'
#' @examples
#' library(agricolaeplotr)
#' library(sf)
#' library(ggplot2)
#' example("make_polygons")
#' polygo <- make_polygons(plt, north = 13454206.89, east = 7939183.21)
#' polygo <- st_transform(polygo, 25832)
#' pl <- protective_layers(polygo)
#' # plot experiment shape
#' ggplot(pl) + geom_sf(fill=c("black","orange","blue","red"))+ theme_minimal()
#' # write them to kml for Google Maps
#' # st_write(pl, "boundaries2.kml", append = FALSE)
#'
protective_layers <- function(design, borders = c(0, 3, 5, 10)) {

  test_input_shift(borders)

  borders <- unique(sort(borders, decreasing = TRUE))
  layer_list <- list()

  for (i in 1:length(borders)) {
    layer_list[[i]] <- st_union(st_buffer(design["x"], borders[i], joinStyle = "MITRE", mitreLimit = 2, endCapStyle = "ROUND"))
  }

  df <- do.call(rbind, layer_list)
  df <- st_as_sfc(df)
  st_crs(df) <- crs(design)

  return(df)
}

test_that("throws error if data type is not an integer but a string", {
  expect_error(test_input_extend("3"))
})
test_that("throws error if data type is not an integer but a float", {
  expect_error(test_input_extend(-2.5))
})
test_that("throws error if data type is not an integer but a boolean", {
  expect_error(test_input_extend(TRUE))
})
sd <- "cat2"
test_that("throws error if data type is not an integer
          but an object containg string", {
  expect_error(test_input_extend(sd))
})

test_that("throws error if data type is not an integer but an float", {
  expect_silent(test_input_extend(2.5))
})
test_that("keeps silent if data type is correct integer ", {
  expect_silent(test_input_extend(2))
})
sd <- 10.5
test_that("throws error if data type is not an integer
          but an object containing float ", {
  expect_silent(test_input_extend(sd))
})

test_that("throws error if data type is not an integer but a string", {
  expect_error(test_input_extend("3"))
})
test_that("throws error if data type is not an integer but float", {
  expect_error(test_input_extend(-2.5))
})
test_that("throws error if data type is not an integer but boolean", {
  expect_error(test_input_extend(TRUE))
})
sd <- "cat2"
test_that("throws error if data type is not an
          integer but an object containg a string", {
  expect_error(test_input_extend(sd))
})

test_that("throws error if data type is not an integer but ", {
  expect_silent(test_input_extend(2.5))
})
test_that("throws error if data type is not an integer but ", {
  expect_silent(test_input_extend(2))
})
sd <- 10.5
test_that("throws error if data type is not an integer but ", {
  expect_silent(test_input_extend(sd))
})

test_that("keeps silent if data type is a boolean", {
  expect_silent(test_input_reverse(TRUE))
})  # no messages if success
test_that("throws error if data type is not an boolean but a string", {
  expect_error(test_input_reverse("TRUE"))
})
test_that("throws error if data type is not an boolean but an integer", {
  expect_error(test_input_reverse(2))
})
test_that("throws error if length is greater than 1", {
  expect_error(test_input_reverse(c(2, 3, 4)))
})

test_that("unkown treatment", {
  expect_silent(test_string("cat"))
})  # no messages if success
test_that("throws error if data type is not a string but a integer", {
  expect_error(test_string(1))
})  # no messages if success
test_that("throws error if data type is not a string but a boolean", {
  expect_error(test_string(TRUE))
})  # no messages if success
test_that("throws error if data type is not a string but a float", {
  expect_error(test_string(2.4))
})  # no messages if success
test_that("throws error if data type is not a string
          but a negative sign float", {
  expect_error(test_string(-2.4))
})  # no messages if success
sd <- 12.55
test_that("throws error if data type is not a string
          but an object containing float", {
  expect_error(test_string(sd))
})  # no messages if success
sd <- "cat2"
test_that("if data type is correct and also works with
          object containing correct data type", {
  expect_silent(test_string(sd))
})  # no messages if success

##### test column and row indicator #######

test_that("throws error if data type is not an integer but a string", {
  expect_error(test_input_ncols("3"))
})
test_that("throws error if data type is not an integer
          but a float with negative sign", {
  expect_error(test_input_ncols(-2.5))
})
test_that("throws error if data type is not an integer but a boolean", {
  expect_error(test_input_ncols(TRUE))
})
sd <- "cat2"
test_that("throws error if data type is not an integer
          but an object containing a string", {
  expect_error(test_input_ncols(sd))
})

test_that("throws error if data type is not an integer
          but a float with positive sign", {
  expect_error(test_input_ncols(2.5))
})
test_that("keeps silent if input is correctly defined", {
  expect_silent(test_input_ncols(2))
})
test_that("throws error if data type is not an
          positive integer", {
  expect_error(test_input_ncols(-2))
})

sd <- 10.5
test_that("throws error if number is not greater than zero", {
            expect_error(test_input_ncols(0))
          })
test_that("throws error if data type is not an integer
          but an object containing a float", {
  expect_error(test_input_ncols(sd))
})
test_that("throws error if length > 1", {
  expect_error(test_input_ncols(c(2, 3, 4)))
})


test_that("throws error if data type is not an integer but a string", {
  expect_error(test_input_nrows("3"))
})
test_that("throws error if data type is not an integer
          but a float with negative sign", {
  expect_error(test_input_nrows(-2.5))
})
test_that("throws error if data type is not an integer
          but a integer with negative sign", {
  expect_error(test_input_nrows(-2))
})
test_that("throws error if number is not greater than zero", {
            expect_error(test_input_nrows(0))
          })

test_that("throws error if data type is not an integer but a boolean", {
  expect_error(test_input_nrows(TRUE))
})
sd <- "cat2"
test_that("throws error if data type is not an integer
          but an object containing a string", {
  expect_error(test_input_nrows(sd))
})

test_that("throws error if data type is not an integer
          but a float with negative sign", {
  expect_error(test_input_nrows(2.5))
})
test_that("keeps silent if input is correcty defined", {
  expect_silent(test_input_nrows(2))
})
sd <- 10.5
test_that("throws error if data type is not an integer
          but an object containing a float with negative sign", {
  expect_error(test_input_nrows(sd))
})
test_that("throws error if length > 1", {
  expect_error(test_input_nrows(c(2, 3, 4)))
})

######## test names #########

test_that("no error, if using default designs in terms of name checking", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_silent(test_names_design(outdesign))
})

test_that("list elements have wrong names", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  names(outdesign) <- c("name1", "sjdfsf")
  expect_error(test_names_design(outdesign))
})

test_that("silent if a table is added to an outdesign", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "rcbd")
  outdesign$book2 <- "something else here?"
  expect_silent(test_names_design(outdesign))
})

test_that("correct name of factor or plot", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "rcbd")
  expect_error(test_name_in_column("plot", outdesign))
})

test_that("incorrect name and type of factor name", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "rcbd")
  expect_error(test_name_in_column(125, outdesign))
})


test_that("incorrect name of factor", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "rcbd")
  expect_error(test_name_in_column(TRUE, outdesign))
})
test_that("correct name of factor", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "rcbd")
  expect_silent(test_name_in_column("plots", outdesign))
})
test_that("correct name of factor", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "rcbd")
  expect_silent(test_name_in_column("A", outdesign))
})
test_that("unkown treatment", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "rcbd")
  expect_error(test_name_in_column("C", outdesign))
})

test_that("unkown treatment", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "rcbd")
  expect_error(test_name_in_column("C", outdesign))
})


test_that("incorrect design youden", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_youden(outdesign))
})

test_that("incorrect design alpha", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_alpha(outdesign))
})

test_that("incorrect design bib", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_bib(outdesign))
})

test_that("incorrect design cyclic", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_cyclic(outdesign))
})

test_that("incorrect design dau", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_dau(outdesign))
})

test_that("incorrect design bib", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "rcbd")
  expect_error(plot_design.factorial_crd(outdesign))
})

test_that("incorrect factorial lsd design", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_design.factorial_lsd(outdesign))
})

test_that("incorrect factorial rcbd design dau", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_design.factorial_rcbd(outdesign))
})

test_that("incorrect design crd", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_design_crd(outdesign))
})

test_that("incorrect graeco design", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_graeco(outdesign))
})

test_that("incorrect design latin square", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_latin_square(outdesign))
})

test_that("incorrect lattice simple design alpha",
          {
            trt <- c(2, 4)
            k <- 2
            outdesign <- design.ab(trt, r = k, serie = 3,
                                   design = "crd")
            expect_error(plot_lattice_simple(outdesign))
          })

test_that("incorrect lattice triple design alpha",
          {
            trt <- c(2, 4)
            k <- 2
            outdesign <- design.ab(trt, r = k, serie = 3,
                                   design = "crd")
            expect_error(plot_lattice_triple(outdesign))
          })

test_that("incorrect rcbd design ", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_rcbd(outdesign))
})

test_that("incorrect split plot design crd", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_split_crd(outdesign))
})

test_that("incorrect split plot design lsd", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_split_lsd(outdesign))
})

test_that("incorrect split plot design crbd", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_split_rcbd(outdesign))
})

test_that("incorrect split plot design lsd", {
  trt <- c(2, 4)
  k <- 2
  outdesign <- design.ab(trt, r = k, serie = 3,
                         design = "crd")
  expect_error(plot_strip(outdesign))
})

test_that("plot error due to low nrow number factorial crd",
          {
            trt <- c(3, 2)  # factorial 3x2
            outdesign <- design.ab(trt, r = 3, serie = 2,
                                   design = "crd")
            expect_error(plot_design.factorial_crd(outdesign,
                         ncols = 5, nrows = 2, reverse_y = TRUE,
                         reverse_x = TRUE))
          })


test_that("plot error due to low nrow number split plot crd",
          {
            T1 <- c("a", "b", "c", "d", "e", "f", "g")
            T2 <- c("v", "w", "x", "y", "zzz")
            r <- 4
            outdesign <- design.split(trt1 = T1, trt2 = T2, r = r,
                        serie = 2, seed = 0, kinds = "Super-Duper",
                        randomization = TRUE,
                      first = TRUE, design = "crd")
            expect_error(plot_split_crd(outdesign, ncols = 1, nrows = 5))
          })

test_that("plot has correct properties", {
  trt<-LETTERS[1:5]
  outdesign<- design.lsd(trt,serie=2)

  p <- plot_latin_square(outdesign, space_width = 0.9, width = 3, height = 5,space_height = 0.9)
  stats <- DOE_obj(p)
  expect_equal(stats$total_area, 375)
  expect_equal(stats$space_between, 71.25)
  expect_equal(stats$eff_total_area, 303.75)
  expect_equal(stats$eff_area_total, 360.15)
  expect_equal(stats$outer_area, 360.15)
  expect_equal(stats$outer_area, stats$eff_area_total)


  expect_equal(stats$rel_space_width, 0.9)
  expect_equal(stats$rel_space_height, 0.9)
  expect_equal(stats$share_eff_plot, 0.81)
  expect_equal(stats$share_space_plot, 0.19)

  expect_equal(stats$gross_width_plot, 3)
  expect_equal(stats$gross_height_plot, 5)
  expect_equal(stats$total_area_plot, 15)
  expect_equal(stats$eff_plot_size, 12.15)

  expect_equal(stats$eff_width, 14.7)
  expect_equal(stats$eff_height, 24.5)
  expect_equal(stats$eff_width_plot, 2.7)
  expect_equal(stats$eff_height_plot, 4.5)
  expect_equal(stats$abs_space_height, 0.5)
  expect_equal(stats$abs_space_width, 0.3)

  expect_equal(stats$net_plot_diagonal, 5.25)
  expect_equal(stats$gross_plot_diagonal, 5.83)
  expect_equal(stats$experiment_diagonal, 20.8)
  expect_equal(stats$outer_diagonal, 28.6)

  expect_equal(stats$xmin, 1.65)
  expect_equal(stats$xmax, 16.35)
  expect_equal(stats$ymin, 2.75)
  expect_equal(stats$ymax, 27.25)
  expect_equal(stats$n_cols, 5)
  expect_equal(stats$n_rows, 5)
  expect_equal(stats$n_plots, 25)
  expect_equal(stats$n_fac, 5)

})

---
title: "Plotting Designs from FielDHub Package"
author: "Jens Harbers"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_width: 6 
    fig_height: 4
vignette: >
  %\VignetteIndexEntry{Plotting Designs from FielDHub Package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}

  
---


# Usage of the package
This chapter shows the usage of the package and the underlying functions.
As factorial experiments are omnipresent in all science and technology fields, a
factorial ab-design will be used as an example. Although some parameters are worth for agriculture only, most other are useful for every user.

Unlike the usual presentation approach, I deliberately inserted the output of the 'FielDHub' functions into the vignette and executed each function individually. This allows interested users to directly adopt the code.

For 'agricolaeplotr', it is necessary to plot each location individually, while 'FielDHub' offers the option to create multiple locations simultaneously.

To illustrate the similarity of both plots, I have juxtaposed both versions.

## Load the package
use the following command to load the package after installation. The three packages below 'agricolaeplotr' are only needed for the examples.

```{r message=FALSE, warning=FALSE}
library("agricolaeplotr")
library("FielDHub")
library("ggplot2")
```

# Example 1: Randomized Complete Block Design (RCBD)
```{r message=FALSE, warning=FALSE}
example("RCBD")
plt <- plot(rcbd2)
p <- full_control_positions(plt$field_book, "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT")
p
```
# Example 2: Latin Square Design
```{r message=FALSE, warning=FALSE}
latinSq1 <- latin_square(t = 4, reps = 2, plotNumber = 101, planter = "cartesian", seed = 1980)
plt <- plot(latinSq1)
p <- full_control_positions(plt$field_book, "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT")
p
```
# Example 3: Full Factorial Design
```{r message=FALSE, warning=FALSE}
example("full_factorial")
plt <- plot(fullFact2, l = 1)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "Loc1",], "COLUMN", "ROW", factor_name = "FACTOR_A", label = "FACTOR_A")
p
```
# Example 4: Incomplete Blocks Design
```{r message=FALSE, warning=FALSE}
example("incomplete_blocks")
plt <- plot(ibd2)
p <- full_control_positions(plt$field_book, "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT")
p
```
# Example 5: Diagonal Arrangement Design
```{r message=FALSE, warning=FALSE}
example("diagonal_arrangement")
plt <- plot(spatAB)
p <- full_control_positions(plt$field_book, "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT")
p
```
# Example 6: Augmented Randomized Complete Block Design (ARCBD)
```{r message=FALSE, warning=FALSE}
example("RCBD_augmented")
plt <- plot(ARCBD2)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "A",], "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT") + guides(fill = "none")
p
```
# Example 7: Rectangular Lattice Design
```{r message=FALSE, warning=FALSE}
example("rectangular_lattice")
plt <- plot(rectangularLattice2)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "LOC1",], "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT")
p
```
# Example 8: Strip Plot Design
```{r message=FALSE, warning=FALSE}
example("strip_plot")
plt <- plot(strip2)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "A",], "COLUMN", "ROW", factor_name = "TRT_COMB", label = "TRT_COMB")
p
```
# Example 9: Square Lattice Design
```{r message=FALSE, warning=FALSE}
example("square_lattice")
plt <- plot(squareLattice2)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "CASSELTON",], "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT")
p
```
# Example 10: Split Plot Design
```{r message=FALSE, warning=FALSE}
example("split_plot")
plt <- plot(SPDExample2)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "A",], "COLUMN", "ROW", factor_name = "TRT_COMB", label = "TRT_COMB")
p
```
# Example 11: Split-Split Plot Design
```{r message=FALSE, warning=FALSE}
example("split_split_plot")
plt <- plot(SSPD2)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "A",], "COLUMN", "ROW", factor_name = "TRT_COMB", label = "TRT_COMB") + guides(fill = "none")
p
```
# Example 12: Completely Randomized Design (CRD)
```{r message=FALSE, warning=FALSE}
example("CRD")
plt <- plot(crd3)
p <- full_control_positions(plt$field_book, "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT") + guides(fill = "none")
p
```

# Example 13: Sparse Allocation Design
```{r message=FALSE, warning=FALSE}
example("sparse_allocation")
plt <- plot(sparse)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "LOC1",], "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT") + guides(fill = "none")
p
```

# Example 14: Row-Column Design
```{r message=FALSE, warning=FALSE}
example("row_column")
plt <- plot(rowcold2, l = 1)
p <- full_control_positions(plt$field_book[plt$field_book$LOCATION == "1",], "COLUMN", "ROW", factor_name = "TREATMENT", label = "TREATMENT") + guides(fill = "none")
p
```

---
title: "agricolaeplotr: A Package for Visualization of Design of Experiments"
author: "Jens Harbers"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_width: 6 
    fig_height: 4
vignette: >
  %\VignetteIndexEntry{agricolaeplotr: A Package for Visualization of Design of Experiments}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}

  
---
## Installation of the package
Use the following command to install the package from CRAN:
```{r setup, eval = FALSE}
install.packages("agricolaeplotr")
```

# Usage of the package
This chapter shows the usage of the package and the underlying functions.
As factorial experiments are omnipresent in all science and technology fields, a
factorial ab-design will be used as an example. Although some parameters are worth for agriculture only, most other are useful for every user.

## Load the package
use the following command to load the package after installation. 

```{r message=FALSE, warning=FALSE}
library("ggplot2")
library("agricolae")
library("agricolaeplotr")
library("raster")
```

## Example: factorial ab design with complete randomization
First we need to create an design using agricolae package.
All examples used in the package are directly taken from agricolae.

After creation of the object, everything is ready to plot a basic plot. It is also assumed, that the height and the width of each plot is set to 1. In agricultural designs, it is recommended to input the measures from a plot to have an idea about the dimensions needed to establish such a experiment in the field.
Knowing the needed amount in meters or other units is important for machinery and management of the experiment.

Complete randomized designs do not have a factor like blocks, so the user is required to input a suitable number for columns and rows. The product of the number of rows and columns must be greater than the size of the experiment, to give the program the chance to place all plots.

Following figure shows the output of a basic design. The output is a standard ggplot2 design. This implies that the user can do all operations, that ggplot2 and other packages that uses ggplot2 functions, can be applied without having layer restrictions or too specialized layers that prevent other transformations. The user also might use plotly to create interactive visualizations of the designs. Useful for field demonstrations for all project stakeholders i.e. (scientists, farmers, funding agencies).

```{r}
library(agricolae) # origin of the needed design object
trt<-c(3,2) # factorial 3x2
outdesign <- design.ab(trt, r=3, serie=2,design = 'crd')

head(outdesign$book,10)

plot_design.factorial_crd(outdesign,ncols=7,nrows=3, width = 1, height = 1)

```

# Change Axis and orientation of the design
The user can change the order of both axis. Although it is a standard to enumerate plots from left to right, increasing from bottom, there is no restriction to enumerate in other ways. The following change reverts the y axis. Reverting y axis causes to have same orientation as 'agricolae' packages shows in some sketches. If the user wants to have same orientation, one must set reverse_y=TRUE.

## revert the y axis

```{r, echo=TRUE, results='asis'}
plot_design.factorial_crd(outdesign,ncols=6,nrows=3, width = 1, height = 1, reverse_y = TRUE)
```

## revert the x axis

```{r, echo=TRUE, results='asis'}
plot_design.factorial_crd(outdesign,ncols=6,nrows=3, width = 1, height = 1, reverse_x = TRUE)
```



## revert both axis

```{r, results='asis'}
plot_design.factorial_crd(outdesign,ncols=6,nrows=3, width = 1, height = 1, reverse_x = TRUE,reverse_y = TRUE)
```

## Changing width and height
As described before, the user can decide the dimensions of each plot. The function assumes that each plot has same dimensions as the others. The dimensions are meant for a **single plot**, not for the entire field dimensions.

```{r, echo = TRUE, results='asis'}
plot_design.factorial_crd(outdesign,ncols=6,nrows=3, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE)
```



## Changing space_width and space_height
These parameters revert to the space of each plot. 
If both set to 1, there is no space between each plot.
Zero indicates a 100 percent space between each plot, so no rectangle will be shown.
Setting values greater one causes in overlaying plots, which are not possible in agricultural experimental design. Therefore the range of this parameter is between >0 and 1.

```{r, echo = TRUE, results='asis'}
plot_design.factorial_crd(outdesign,ncols=6,nrows=3, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE,space_width = 1,space_height = 1)
```

This plot will show more space between each plot.
If calculating the net area, the space parameter needs to be multiplied with the width and height respectively.

```{r, echo = TRUE, results='asis'}
plot_design.factorial_crd(outdesign,ncols=6,nrows=3, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE,space_width = 0.7,space_height = 0.8)
```

## Change of label column

The user can input a column name as a string, that must be in the columns of the design$book table.
The input will be checked if it does so.

```{r, echo = TRUE, results='asis'}
plot_design.factorial_crd(outdesign,ncols=6,nrows=3, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE, factor_name = "B")
```

## Change of factor name column

The user can input a factor_name as a string, that must be in the columns of the design$book table.
The input will be checked if it does so. This example shows how the selection of a different factor than the first one (default) will be performed for factorial experiments in complete randomized designs.

```{r, echo = TRUE, results='asis'}
set.seed(129984)
trt<-c(3,2) # factorial 3x2
outdesign <- design.ab(trt, r=3, serie=2,design = 'crd')
plot_design.factorial_crd(outdesign,ncols=3,nrows=6, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE, factor_name = "B")
```

## poster_theme
This function adds changes to a ggplot, which are good for poster presentations.

```{r, echo = TRUE, results='asis'}
set.seed(129866478)
trt<-c(3,2) # factorial 3x2
outdesign <- design.ab(trt, r=3, serie=2,design = 'crd')
plot_design.factorial_crd(outdesign,ncols=3,nrows=6, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE, factor_name = "B") + theme_poster()
```


## plotting continuous outcomes
Plotting design maps like a heat map has some advantages. First, it shows some generals eyeballing trends in the data. Second, missing data are shown as a plot, therefore a technician can go to the experiment and retake samples, if needed. NA are plotted in gray. 

## pres_theme
This function adds changes to a ggplot, which are good for presentations in slideshows.

```{r, echo = TRUE, results='asis'}
set.seed(12986)
trt<-c(3,2) # factorial 3x2
outdesign <- design.ab(trt, r=3, serie=2,design = 'crd')

plot_design.factorial_crd(outdesign,ncols=3,nrows=6, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE, factor_name = "B") + theme_pres()
```

## re-scaling colors
The user can use a scale for factors. Sometimes useful if factors are in a gradient manner, for example nitrogen fertilizer application or amount of irrigation in Liter.


```{r, echo = TRUE, results='asis'}
trt<-c(3,2) # factorial 3x2
outdesign <- design.ab(trt, r=3, serie=2,design = 'crd')
plot_design.factorial_crd(outdesign,ncols=3,nrows=6, width = 1, height = 2.5 , reverse_x = FALSE,reverse_y = TRUE, factor_name = "B") + theme_pres() + scale_fill_viridis_d()
```

## Plotting of exogenous variables
In experiment analysis, a visualization of the results prior to analysis is an important step in assuring a high quality. identifying plots with missing data enables a re-sampling of the missing plots, so less numeric data imputation will be necessary.

The following example shows, how to plot an exogenous variable.
The variable is here the factor_name. We assume that we want to plot yield of a crop experiment, and use in addition a scaling parameter to recolor the fill. The code is the following:

```{r}
set.seed(23488833)
trt <-c(3,2)
outdesign<-design.ab(trt, serie=2, design="lsd",seed = 454555)
length_table <- dim(outdesign$book)[1] # length of the table

outdesign$book$yield <- sample(c(5:12,c(NA,NA,NA)), size = length_table, replace = TRUE)
plot_design.factorial_lsd(outdesign,factor_name = "yield") + scale_fill_viridis_c()

```

The missing yields are easily to be seen.

## Plotting of exogenous variables from foreign tables
Sometimes, experiment data are in data frames, that must be merged with the outdesign object.
Therefore, a second data frame will be merged with the outdesign and the merged data frame will be stored in the outdesign$book.

```{r}
set.seed(23488833)
trt <-c(3,2)
outdesign<-design.ab(trt, serie=2, design="lsd",seed = 454555)
length_table <- dim(outdesign$book)[1] # length of the table

yield <- sample(c(5:20,c(NA,NA,NA)), size = length_table, replace = TRUE)
df <- cbind(plots=outdesign$book$plots,yield)
head(df,10)
outdesign$book <- merge(outdesign$book,df, by.x = "plots", by.y = "plots")
plot_design.factorial_lsd(outdesign,factor_name = "yield") + scale_fill_viridis_c()

```





## use your own design tables
It is also possible to use own generated experiments designs. However this comes with some caveats. As the functions are quite strict in parameter naming, the column names should match with those from agricolae. The design type and the applied design, sometimes referred as design type, needs to be set in a list in advance.

The following example provides a mock setup for an experiment.
The second chunk of code shows how the list is set up and how the parameters should be named for a randomized complete block design in a factorial way. It is possible to plot measured outcomes from plots, as ggplot supports plotting continuous values.

```{r}

set.seed(1298664)
plots <- as.factor(1:(8*6))
block <- as.factor((rep(1:6,each=8)))
A <- as.vector(replicate(8,sample(rep(1:2,times=3),6,replace=FALSE)))
outcome <- runif(48,20,100)
experiment <- cbind(plots,block,A,outcome)
experiment <- as.data.frame(experiment)
head(experiment)

experiment_design <- list()
experiment_design$parameters$design<- "factorial"
experiment_design$parameters$applied <-  "rcbd"

experiment_design$book <- experiment
head(experiment_design)
plot_design.factorial_rcbd(experiment_design,factor_name = "A")
plot_design.factorial_rcbd(experiment_design,factor_name = "outcome")

```

## plot split plot designs

Split plots designs are designs that divide a main plot into two or more smaller, equal sized subplots. This leads to the definition of the dimensions. Te dimensions are meant for a single main plot, the super set of all belonging subplots. The three functions of this family are returning two plots. One for the main factor, a second one for the subplots. The dimension for the subplots are those from main plot, divided by the number of subplots, in width. To ensure only one plot is produced at the time, the user is required to choose a subplot design or an main plot design. To have both plots, the user has currently to run the function twice.

```{r}
set.seed(1298664)
t1<-c('a','b','c','d','e',"f","g","h")
t2<-c("u",'v','w','x','y',"z")
outdesign2 <- design.split(trt1=t1, trt2=t2, r=r,serie = 2,
                           seed = 0, kinds = 'Super-Duper',
                           randomization=TRUE,first=TRUE,design = 'lsd')

plot_split_lsd(outdesign2,factor_name_1 = "t1",factor_name_2 = "t2",width = 2,height = 2, subplots = FALSE,labels = "plots")


plot_split_lsd(outdesign2,width = 2,height = 2, subplots = TRUE, labels = "splots", factor_name_1 = "t1", factor_name_2 = "t2")

```


```{r}
set.seed(1298664)
t1<-c('a','b','c','d','e','f','g')
t2<-c('v','w','x','y','z')
r <- 4
outdesign2 <- design.split(trt1=t1, trt2=t2, r=r,
serie = 2, seed = 0, kinds = 'Super-Duper',
randomization=TRUE,first=TRUE,design = 'crd')
plot_split_crd(outdesign2,ncols = 5,nrows=6, subplots = FALSE,
               factor_name_1 = "t1",factor_name_2 = "t2")
plot_split_crd(outdesign2,ncols = 5,nrows=6, subplots = TRUE, labels="splots",factor_name_1 = "t1",factor_name_2 = "t2")

```


```{r}
set.seed(1298664)
T1<-c('a','b','c','d','e',"f","g")
T2<-c("we",'v','w','x','y','z',"d")
r = 3
outdesign2 <- design.split(trt1=T1, trt2=T2, r=r,serie = 2,
 seed = 0, kinds = 'Super-Duper',randomization=TRUE,
 first=TRUE,design = 'rcbd')

plot_split_rcbd(outdesign2,width = 5,height = 5,subplots = FALSE, 
                factor_name_1 = "T1",factor_name_2 = "T2")

plot_split_rcbd(outdesign2,width = 5,height = 5,labels = "splots",
                factor_name_1 = "T1",factor_name_2 = "T2")


```

## Referring to other packages

If the user can provide a table containing all necessary information about the coordinates of each plot, the user can choose the 'desplot' package.

There are two functions namely 'ggdesplot' and 'ggdesplot' from the 'desplot' package, that has some other features. However, the user cannot swap the x and y coordinates that easy as here and there is no way to adjust for height and length of a plot (preferable in meter). The user needs to provide x and y coordinates to be able to use 'desplot' and 'ggdesplot'.

## Planed features for future versions

- match size of subplot to those to mainplots for all split plot designs
- create own function for user specific experiment designs with same API
- add field designs for replicated experiments made in 'FielDHub'


```{r message=FALSE, warning=FALSE}

trt<-c(3,2) # factorial 3x2
outdesign <- design.ab(trt, r=3, serie=2,design = 'crd')
plt <- plot_design.factorial_crd(outdesign,ncols=3,nrows=6, width = 5, height = 7.5 , reverse_x = FALSE,reverse_y = TRUE, factor_name = "B") + theme_pres() + scale_fill_viridis_d()

spat_df <- make_polygons(plt,east = 3454206.89, 
                         north = 5939183.21 ,
                         projection_output = '+init=EPSG:4326')

plot(spat_df["fill"],col=spat_df$fill)

# this part does not work well in a vignette
library(leaflet)

spat_df <- sf:::as_Spatial(spat_df)

spat_df <- sp::elide(spat_df,rotate = -90)
                   
 leaflet(spat_df) %>% addPolygons(
   fillColor = spat_df$fill,
   opacity=1,
   color="black",
   fillOpacity = 1) %>% addProviderTiles(provider = "OpenStreetMap.DE")
```


# Output of central properties of a design of experiment

The user can choose some outputs to have deeper insights of the experiment.

1. The net plot is more or less the actually used plot, without the borders on  a plot level.

2. The gross plot is the sum of the net plot and the border of a plot.

3. The sum of all gross plots with the respective space in between is named the field.

4. The experiment shows some properties that are unrelated to the plots on a field.

5. The user also can choose the combined output listed from the elements by typing part = 'all'.

The DOE_obj function uses the ggplot object and extract relevant field information. This is required, as the summary cannot use the ggplot object. The user also my use a different unit than meter (default).

The user may extract the actual values, as they are separated from the descriptors using normal data.frame related command and operations.

```{r}
varieties<-c('perricholi','yungay','maria bonita','tomasa')
outdesign <-design.youden(varieties,r=2,serie=2,seed=23)
p <- plot_youden(outdesign, labels = 'varieties', width=4, height=3)
stats <- DOE_obj(p)
r <- to_table(stats,part = "net_plot", digits = 2)
r
r <- to_table(stats,part = "gross_plot", digits = 2)
r
r <- to_table(stats,part = "field", digits = 2)
r
r <- to_table(stats,part = "experiment", digits = 2)
r
r <- to_table(stats,part = "all", digits = 2)
r
```

# full command of a plot

Sometime it it useful to use same plotting function with a known behaviour for plotting purposes. the user may choose the origin (0|0) being in the lower left corner of the net plot. The field design needs to be a data.frame, and not a list like usually used in 'agricolae'.
```{r}

varieties<-c('perricholi','yungay','maria bonita','tomasa')
outdesign <-design.youden(varieties,r=2,serie=2,seed=23)
design <- outdesign$book
design

p <- full_control_positions(design,"col","row","varieties","plots",
                       width=3,height=4.5,
                       space_width=1,space_height=1,
                       shift_x=-0.5*3,shift_y=-0.5*4.5)
p

p <- full_control_positions(design,"col","row","varieties","plots",
                       width=3,height=4.5,
                       space_width=0.93,space_height=0.945,
                       start_origin = TRUE)
                       p
```

# Scholar related functions

In science it is common to cite used R-packages as the authors put in a lot of effort, time and money to program and maintaining them. As the citations are the actual currency, the citation function should be run at the bottom off an R script.

```{r}
citation("agricolaeplotr")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{theme_poster}
\alias{theme_poster}
\title{ggplot2 theme for poster presentation}
\usage{
theme_poster()
}
\value{
ggplot2 theme
}
\description{
This theme is designed to increase font size to ensure readability on poster presentations
}
\examples{
library(agricolaeplotr)
library(agricolae)
T1<-c('a','b','c','d','e','f','g')
T2<-c('v','w','x','y','z')
r <- 4
outdesign2 <- design.split(trt1=T1, trt2=T2, r=r,
serie = 2, seed = 0, kinds = 'Super-Duper',
randomization=FALSE,first=TRUE,design = 'crd')
plot_split_crd(outdesign2,ncols = 6,nrows=5)+
theme_poster()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{protective_layers}
\alias{protective_layers}
\title{Create Protective Layers for Design of Experiments (DOEs)}
\usage{
protective_layers(design, borders = c(0, 3, 5, 10))
}
\arguments{
\item{design}{An \code{sf} object containing the polygons of the experiment.
The coordinate reference system (crs) of the data needs to be in
metric distance, not degrees.}

\item{borders}{A numeric vector specifying the distances (in meters) for which
protective layers should be created. The layers will be created
with decreasing distances, starting from the largest.}
}
\value{
An \code{sf} object representing the protective layers around the
        experiment polygons.
}
\description{
This function generates protective layers around the polygons of an experiment.
These layers can be used to plot boundaries, for example, to protect agricultural
on-farm experiments from accidental harvesting.
}
\examples{
library(agricolaeplotr)
library(sf)
library(ggplot2)
example("make_polygons")
polygo <- make_polygons(plt, north = 13454206.89, east = 7939183.21)
polygo <- st_transform(polygo, 25832)
pl <- protective_layers(polygo)
# plot experiment shape
ggplot(pl) + geom_sf(fill=c("black","orange","blue","red"))+ theme_minimal()
# write them to kml for Google Maps
# st_write(pl, "boundaries2.kml", append = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{test_name_in_column}
\alias{test_name_in_column}
\title{Test if input column names}
\usage{
test_name_in_column(x, design)
}
\arguments{
\item{x}{string input}

\item{design}{design from \code{agricolae} package}
}
\value{
error
}
\description{
Test if input is in column names of a table
}
\examples{
library(agricolaeplotr)
library(agricolae)
trt<-c(2,4)
k=6
outdesign<-design.ab(trt, r=k, serie=3,design='rcbd')
test_name_in_column('B',outdesign)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{citations}
\alias{citations}
\title{Citation}
\usage{
citations(includeURL = TRUE, bibtex = TRUE)
}
\arguments{
\item{includeURL}{boolean, Should the URL be returned?}

\item{bibtex}{boolean, Should the citations be returned as bibtex?}
}
\value{
printed output to console
}
\description{
Generates citations of all loaded packages
}
\examples{
library(ggplot2)
library(agricolaeplotr)
library(agricolae)
library(raster)
citations()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{theme_gi}
\alias{theme_gi}
\title{theme_gi}
\usage{
theme_gi()
}
\value{
a 'ggplot' graph with a modified theme
}
\description{
Creates a theme for 'ggplot' based graphics to ensure
to meet formal requirements for conferences of the
Gesellschaft fuer Informatik
}
\examples{
# example borrowed from ggplot2
library(ggplot2)
df <- data.frame(
gp = factor(rep(letters[1:3], each = 10)),
y = rnorm(30))

p <- ggplot() +
geom_point(data = df, aes(gp, y))
p <- p + theme_gi();p
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{plot_strip}
\alias{plot_strip}
\title{Plot Strip Design}
\usage{
plot_strip(
  design,
  x = "col",
  y = "row",
  factor_name_1 = "T1",
  factor_name_2 = "T2",
  labels = "plots",
  width = 1,
  height = 1,
  space_width = 0.95,
  space_height = 0.85,
  reverse_y = FALSE,
  reverse_x = FALSE
)
}
\arguments{
\item{design}{outdesign from \code{agricolae} package}

\item{x}{Describes the x coordinates of a experiment design}

\item{y}{Describes the y coordinates of a experiment design}

\item{factor_name_1}{Which factor should be used for plotting, needs to be a column in outdesign$book}

\item{factor_name_2}{Which factor should be used for plotting, needs to be a column in outdesign$book}

\item{labels}{Describes the column from that the plots are taken to display them}

\item{width}{numeric value, describes the width of a plot in an experiment}

\item{height}{numeric value, describes the height of a plot in an experiment}

\item{space_width}{numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width}

\item{space_height}{numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height}

\item{reverse_y}{boolean, should the plots of the experiment be changed in reverse order in Row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE}

\item{reverse_x}{boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE}
}
\value{
\code{ggplot} graphic that can be modified, if wished
}
\description{
Plot a design of an experiment with an Strip Plot design from \code{agricolae} design.strip
}
\examples{
library(agricolaeplotr)
library(agricolae)
T1<-c('a','b','c','d')
T2<-c('v','w','x','y','z')
r = 3
outdesign <- design.strip(trt1=T1, trt2=T2, r=r,serie = 2,
 seed = 0, kinds = 'Super-Duper',randomization=TRUE)
plot_strip(outdesign,factor_name_1 = "T1",factor_name_2="T2")
plot_strip(outdesign,factor_name_1 = "T1",factor_name_2="T2",reverse_x = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{plot_dau}
\alias{plot_dau}
\title{Plot Design of Augmented Blocks (dau)}
\usage{
plot_dau(
  design,
  y = "block",
  factor_name = "trt",
  labels = "plots",
  width = 1,
  height = 1,
  space_width = 0.95,
  space_height = 0.85,
  reverse_y = FALSE,
  reverse_x = FALSE
)
}
\arguments{
\item{design}{outdesign from \code{agricolae} package}

\item{y}{Describes the y coordinates of a experiment design}

\item{factor_name}{Which factor should be used for plotting, needs to be a column in outdesign$book}

\item{labels}{Describes the column from that the plots are taken to display them}

\item{width}{numeric value, describes the width of a plot in an experiment}

\item{height}{numeric value, describes the height of a plot in an experiment}

\item{space_width}{numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width}

\item{space_height}{numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height}

\item{reverse_y}{boolean, should the plots of the experiment be changed in reverse order in Row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE}

\item{reverse_x}{boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE}
}
\value{
\code{ggplot} graphic that can be modified, if wished
}
\description{
Plot a design of an experiment with an augmented block design from \code{agricolae} design.dau
}
\examples{
library(agricolaeplotr)
library(agricolae)
T1<-c('A','B','C','D','E','F')
T2<-letters[19:26]
outdesign <-design.dau(T1,T2, r=5,serie=2)
plot_dau(outdesign)
plot_dau(outdesign,reverse_y = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{summary}
\alias{summary}
\title{summary of a field Layout}
\usage{
summary(object, unit = "m", part = "net_plot", ...)
}
\arguments{
\item{object}{an object, created by DOE_obj with a FieldLayout class}

\item{unit}{a string that corresponds to measure unit (default is m)}

\item{part}{which part of the summary are you interested?
Choose one of the following: "net_plot","gross_plot","field","experiment","all"}

\item{...}{further arguments passed to or from other methods}
}
\description{
print a summary of a FieldLayout object
}
\examples{
library(agricolaeplotr)
library(agricolae)
varieties<-c('perricholi','yungay','maria bonita','tomasa')
outdesign <-design.youden(varieties,r=2,serie=2,seed=23)
p <- plot_youden(outdesign, labels = 'varieties')
stats <- DOE_obj(p)
# print plot summary for net plot (plots without space)
summary(stats, part = "net_plot")
# print plot summary for gross plot (plots with space)
summary(stats, part = "gross_plot")
# print plot summary for entire field
summary(stats, part = "field")
# print plot summary for design summary
summary(stats, part = "experiment")
# print plot summary for all information shown above in one output
summary(stats, part = "all")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{full_control_positions}
\alias{full_control_positions}
\title{full_control_positions}
\usage{
full_control_positions(
  design,
  x = "col",
  y = "row",
  factor_name = "trt",
  labels = "plots",
  width = 1,
  height = 1,
  space_width = 0.95,
  space_height = 0.85,
  reverse_y = FALSE,
  reverse_x = FALSE,
  way_x = 0,
  way_y = 0,
  shift_columns = 0,
  shift_rows = 0,
  shift_x = 0,
  shift_y = 0,
  dist_x = 1,
  dist_y = 1,
  n_shift_columns = 0,
  n_shift_rows = 0,
  start_origin = FALSE
)
}
\arguments{
\item{design}{data.frame containing the row and columns of an experiment}

\item{x}{Describes the x coordinates of a experiment design}

\item{y}{Describes the y coordinates of a experiment design}

\item{factor_name}{string Which factor should be used for plotting, needs to be a column in outdesign$book}

\item{labels}{string Describes the column from that the plots are taken to display them}

\item{width}{numeric value, describes the width of a plot in an experiment}

\item{height}{numeric value, describes the height of a plot in an experiment}

\item{space_width}{numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width}

\item{space_height}{numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height}

\item{reverse_y}{boolean, should the plots of the experiment be changed in reverse order in Row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE}

\item{reverse_x}{boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE}

\item{way_x}{numeric vector indicates the shift of the nth-plot in x-axis.}

\item{way_y}{numeric vector indicates the shift of the nth-plot in y-axis.}

\item{shift_columns}{numeric indicates the shift of the given plots of a specific row by n units in x-axis.}

\item{shift_rows}{numeric indicates the shift of the given plots of a specific column by n units in  y-axis.}

\item{shift_x}{numeric indicates the shift in units in x-axis.}

\item{shift_y}{numeric indicates the shift in units for the y-axis.}

\item{dist_x}{numeric indicates the shift in plots in x-axis.}

\item{dist_y}{numeric indicates the shift in plots for the y-axis.}

\item{n_shift_columns}{numeric vector indicating the number of plots of shift_columns. negative number indicate shift to left, otherwise right}

\item{n_shift_rows}{numeric vector indicating the number of plots of shift_rows. negative number indicate shift to left, otherwise right}

\item{start_origin}{boolean. Should the design start at the origin (0|0)?}
}
\value{
\code{ggplot} graphic that can be modified, if wished
}
\description{
This function provides full control about the plotting. The user also may shift the coordinates as liked.
}
\examples{
library(agricolaeplotr)
library(agricolae)
library(ggplot2)
varieties<-c('perricholi','yungay','maria bonita','tomasa')
outdesign <-design.youden(varieties,r=2,serie=2,seed=23)
design <- outdesign$book
design
p <- full_control_positions(design,"col","row","varieties","plots",
                       width=3,height=4.5,
                       space_width=0.5,space_height=0.5,
                       shift_x=(-0.5*3) + (-0.5*3*0.5),shift_y=-0.5*4.5 + (-0.5*4.5*0.5))
p
p <- full_control_positions(design,"col","row","varieties","plots",
                       width=3,height=4.5,
                       space_width=0.13,space_height=0.445,
                       shift_x=(-0.5*3) + (-0.5*3*(1-0.13)),shift_y=-0.5*4.5 + (-0.5*4.5*(1-0.445)))
                       p

varieties<-LETTERS[1:12]
outdesign <-design.youden(varieties,r=12,serie=2,seed=23)
design <- outdesign$book
p <- full_control_positions(design,"col","row","varieties","plots",
                       width=3,height=4.5,
                       space_width=1,space_height=1,
                       shift_x=-0.5*3,shift_y=-0.5*4.5)
p

p <- full_control_positions(design,"col","row","varieties","plots",
                       width=3,height=4.5,
                       space_width=0.93,space_height=0.945,
                       start_origin = TRUE)
                       p

p <- full_control_positions(design,"col","row","varieties","plots",
width=3,height=4.5,
space_width=0.93,space_height=0.945,way_x = c(2,6,8,10,12),way_y=c(3,8),dist_x=2,dist_y=4,
start_origin = TRUE, reverse_y = FALSE,  reverse_x = FALSE);p

p <- full_control_positions(design,"col","row","varieties","plots",
                                    width=3,height=4.5,
                                    space_width=0.93,space_height=0.945,
                                    way_x = c(2,4,6,8,10,12),way_y=c(3,8),
                                    start_origin = FALSE, reverse_y = FALSE,
                                    reverse_x = FALSE);p
p <- full_control_positions(design,"col","row","varieties","plots",
                                    width=3,height=4.5,shift_columns=c(4,8),
                                    shift_rows=c(3,5,9),
                                    n_shift_columns=c(1,5),
                                    n_shift_rows=c(1,-2,6),
                                    space_width=0.93,space_height=0.945,
                                    way_x = c(2,4,6,8,10,12),way_y=c(3,8),
                                    start_origin = TRUE, reverse_y = FALSE,
                                    reverse_x = FALSE);p
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{plot_longest_diagonal}
\alias{plot_longest_diagonal}
\title{Plot the longest diagonal of a field}
\usage{
plot_longest_diagonal(
  field,
  n = 8,
  type = "random",
  n_segments = 2,
  distance_field_boundary = 3,
  width_diagonal_path = 2
)
}
\arguments{
\item{field}{An object of class sf representing the field.}

\item{n}{Integer, the number of sample points along the longest diagonal.}

\item{type}{Type of sampling. Default is "random".}

\item{n_segments}{Numeric, the number of segments to divide the longest diagonal (default is 2).}

\item{distance_field_boundary}{Numeric, the distance to buffer the field for creating the boundary (default is 3.0).}

\item{width_diagonal_path}{Numeric, the width to buffer the diagonal path (default is 2.0).}
}
\value{
\itemize{
\item p: A ggplot object showing the field, the buffered field, the buffered line, and the sample points.
\item buffered_line: A sf object representing the buffered line.
\item my_line: A sf object representing the longest diagonal of the field.
\item sample_points: A sf object representing the sampled points.
\item length: A numeric value, representing the length of the longest line.
}
}
\description{
This function takes a field and plots the longest diagonal of the field. The field is divided into segments and points are sampled from these segments.
}
\examples{
library(sf)
my_sf <- st_read(system.file("shape/gfn_schlaege.shp", package="agricolaeplotr"))
st_crs(my_sf) <- 25832
field <- my_sf[my_sf$SCHLAG_NR == 170,]
plot_longest_diagonal(field)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{plot_split_crd}
\alias{plot_split_crd}
\title{Plot Split Plot Designs (crd)}
\usage{
plot_split_crd(
  design,
  nrows,
  ncols,
  factor_name_1 = "T1",
  factor_name_2 = "T2",
  labels = "plots",
  subplots = TRUE,
  width = 1,
  height = 1,
  space_width = 0.95,
  space_height = 0.85,
  reverse_y = FALSE,
  reverse_x = FALSE
)
}
\arguments{
\item{design}{outdesign from \code{agricolae} package}

\item{nrows}{Number of rows for the design}

\item{ncols}{Number of columns for the design}

\item{factor_name_1}{string Which factor should be used for plotting, needs to be a column in outdesign$book}

\item{factor_name_2}{string Which factor should be used for plotting, needs to be a column in outdesign$book}

\item{labels}{string Describes the column from that the plots are taken to display them}

\item{subplots}{should the plot function return the subplots (default) or main plots?}

\item{width}{numeric value, describes the width of a plot in an experiment}

\item{height}{numeric value, describes the height of a plot in an experiment}

\item{space_width}{numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width}

\item{space_height}{numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height}

\item{reverse_y}{boolean, should the plots of the experiment be changed in reverse order in Row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE}

\item{reverse_x}{boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE}
}
\value{
\code{ggplot} graphic that can be modified, if wished
}
\description{
Plot a design of a split plot experiment with a complete randomized design (crd) from design.split
}
\examples{
library(agricolaeplotr)
library(agricolae)
T1<-c('a','b','c','d','e','f','g')
T2<-c('v','w','x','y','zzz')
r <- 4
outdesign2 <- design.split(trt1=T1, trt2=T2, r=r,
serie = 2, seed = 0, kinds = 'Super-Duper',
randomization=TRUE,first=TRUE,design = 'crd')
plot_split_crd(outdesign2,ncols = 6,nrows=5)

outdesign2 <- design.split(trt1=T1, trt2=T2, r=r,
serie = 2, seed = 0, kinds = 'Super-Duper',
randomization=FALSE,first=TRUE,design = 'crd')
plot_split_crd(outdesign2,ncols = 6,nrows=5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{to_table}
\alias{to_table}
\title{to_table}
\usage{
to_table(object, part = "net_plot", unit = "m", digits = 3, ...)
}
\arguments{
\item{object}{an object, created by DOE_obj with a FieldLayout class}

\item{part}{which part of the summary are you interested?
Choose one of the following: "net_plot","gross_plot","field","experiment"}

\item{unit}{a string that corresponds to measure unit (default is m)}

\item{digits}{integer indicating the number of decimal places (round)
or significant digits (signif) to be used. Negative values are allowed}

\item{...}{further arguments passed to or from other methods}
}
\value{
dataframe with corresponding information about the experiment
}
\description{
Write field experiment information to a dataframe.
}
\examples{
library(agricolaeplotr)
library(agricolae)
varieties<-c('perricholi','yungay','maria bonita','tomasa')
outdesign <-design.youden(varieties,r=2,serie=2,seed=23)
p <- plot_youden(outdesign, labels = 'varieties', width=4, height=3)
stats <- DOE_obj(p)
r <- to_table(stats,part = "net_plot", digits = 2)
r
r <- to_table(stats,part = "gross_plot", digits = 2)
r
r <- to_table(stats,part = "field", digits = 2)
r
r <- to_table(stats,part = "experiment", digits = 2)
r
r <- to_table(stats,part = "all", digits = 2)
r
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{test_string}
\alias{test_string}
\title{Test if input is a string}
\usage{
test_string(x)
}
\arguments{
\item{x}{input to be tested}
}
\value{
error
}
\description{
Test if input is a string
}
\examples{
library(agricolaeplotr)
test_string('smallstring')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{plot_fieldhub}
\alias{plot_fieldhub}
\title{Plot FielDHub Design}
\usage{
plot_fieldhub(
  design,
  x = "COLUMN",
  y = "ROW",
  labels = "PLOT",
  factor_name = "TREATMENT",
  width = 1,
  height = 1,
  space_width = 0.95,
  space_height = 0.85,
  reverse_y = FALSE,
  reverse_x = FALSE,
  shift_x = 0,
  shift_y = 0
)
}
\arguments{
\item{design}{outdesign from \code{FielDHub} package with on of the following IDs: c(9,13,14,15,16)}

\item{x}{Describes the x coordinates of a experiment design}

\item{y}{Describes the y coordinates of a experiment design}

\item{labels}{string Describes the column from that the plots are taken to display them}

\item{factor_name}{string Which factor should be used for plotting, needs to be a column in outdesign$book}

\item{width}{numeric value, describes the width of a plot in an experiment}

\item{height}{numeric value, describes the height of a plot in an experiment}

\item{space_width}{numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width}

\item{space_height}{numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height}

\item{reverse_y}{boolean, should the plots of the experiment be changed in reverse order in Row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE}

\item{reverse_x}{boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE}

\item{shift_x}{numeric indicates the shift in units in x-axis.}

\item{shift_y}{numeric indicates the shift in units for the y-axis.}
}
\value{
\code{ggplot} graphic that can be modified, if wished
}
\description{
Plots designs from \code{FielDHub} package
}
\examples{
\dontrun{
library(agricolaeplotr)
library(FielDHub)
H <- paste("H", 1:4, sep = "")
V <- paste("V", 1:5, sep = "")

strip1 <- FielDHub::strip_plot(Hplots = H,
                              Vplots = V,
                              b = 1,
                              l = 1,
                              plotNumber = 101,
                              planter = "serpentine",
                              locationNames = "A",
                              seed = 333)


strip1$fieldBook$ROW <- as.numeric(ordered(strip1$fieldBook$VSTRIP,
                       levels = unique(strip1$fieldBook$VSTRIP)))
strip1$fieldBook$COLUMN <- as.numeric(ordered(strip1$fieldBook$HSTRIP,
                       levels = unique(strip1$fieldBook$HSTRIP)))

plot_fieldhub(strip1,
              x = "ROW",
              y = "COLUMN",
              labels = "HSTRIP",
              factor_name = "HSTRIP",
              width = 12,
              height = 10,
              reverse_y = FALSE,
              reverse_x = FALSE)

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{DOE_obj}
\alias{DOE_obj}
\title{Measures of a Field Design}
\usage{
DOE_obj(p)
}
\arguments{
\item{p}{\code{ggplot} object containing the data of the plot}
}
\value{
a list with several useful information about the experiment and the field
}
\description{
Returns a list with several useful information about the experiment
}
\examples{
library(agricolae)
library(agricolaeplotr)
trt = c(2,3,4,5,6)
outdesign1 <-design.crd(trt,r=5,serie=2,2543,'Mersenne-Twister')
p <- plot_design_crd(outdesign1,
             ncols = 7,
             nrows = 4,
             width = 10,
             height = 10,
             reverse_y = TRUE)
stats <- DOE_obj(p)
stats
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{make_polygons}
\alias{make_polygons}
\title{make_polygons}
\usage{
make_polygons(
  ggplot_object,
  north = 3454206.89,
  east = 5939183.21,
  projection_input = "+init=epsg:31467",
  projection_output = "+init=epsg:4326"
)
}
\arguments{
\item{ggplot_object}{saved ggplot object, containing the
coordinates of the rectangles of a 'ggplot' object of the first two layers}

\item{north}{float added to the rows
to have a northing ordinate}

\item{east}{float added to the rows
to have a easting ordinate}

\item{projection_input}{string defines
in which EPSG projection the ggplot object should be converted to a raster object?
a projection with a metric unit is highly recommended}

\item{projection_output}{string defines
in which EPSG projection the SpatialPolygonDataFrame should be exported.}
}
\value{
a SpatialPolygonDataFrame object
}
\description{
This function coerces all rectangles
from a 'ggplot' object to 'SpatialPolygonDataFrame'.
}
\examples{
library(agricolaeplotr)
library(agricolae)
trt = c(2,3,4)
outdesign1 <-design.crd(trt,r=5,serie=2,2543,'Mersenne-Twister')
plt <- plot_design_crd(outdesign1,ncols = 13,nrows = 3)
spat_df <- make_polygons(plt)
spat_df
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{serpentine}
\alias{serpentine}
\title{Serpentine}
\usage{
serpentine(n, times, m = 1)
}
\arguments{
\item{n}{integer value indicating the upper cap of a numeric sequence}

\item{times}{integer number of replications}

\item{m}{integer value indicating the lower
cap of a numeric sequence}
}
\value{
vector containing the serpentine sequence
}
\description{
This function produces a serpentine array of integers beginning by one
}
\examples{
serpentine(n=20,times = 15)
serpentine(n=20,times = 15,m=4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{test_input_ncols}
\alias{test_input_ncols}
\title{checks matrix column input}
\usage{
test_input_ncols(x)
}
\arguments{
\item{x}{input to be tested}
}
\value{
error
}
\description{
checks if input is suitable for matrix column indication
}
\examples{
library(agricolaeplotr)
test_input_ncols(9)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{plot_design_crd}
\alias{plot_design_crd}
\title{Plot Complete Randomized Design}
\usage{
plot_design_crd(
  design,
  ncols,
  nrows,
  y = "row",
  factor_name = "trt",
  labels = "plots",
  width = 1,
  height = 1,
  space_width = 0.95,
  space_height = 0.85,
  reverse_y = FALSE,
  reverse_x = FALSE
)
}
\arguments{
\item{design}{outdesign from \code{agricolae} package}

\item{ncols}{integer value, choose the number of columns to which the experiment should be plotted}

\item{nrows}{integer value, choose the number of rows to which the experiment should be plotted}

\item{y}{Describes the y coordinates of a experiment design, default is row}

\item{factor_name}{Which factor should be used for plotting, needs to be a column in outdesign$book}

\item{labels}{Describes the column from that the plots are taken to display them}

\item{width}{numeric value, describes the width of a plot in an experiment}

\item{height}{numeric value, describes the height of a plot in an experiment}

\item{space_width}{numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of width}

\item{space_height}{numeric value, describes the share of the space of the plots. 0=only space, 1=no space between plots in term of height}

\item{reverse_y}{boolean, should the plots of the experiment be changed in reverse order in Row direction? use reverse_y=TRUE to have same sketch as in agricolae. default:reverse_y=FALSE}

\item{reverse_x}{boolean, should the plots of the experiment be changed in reverse order in column direction? default:reverse_x=FALSE}
}
\value{
\code{ggplot} graphic that can be modified, if wished
}
\description{
Plot a design of a factorial experiment with randomized complete block design from \code{agricolae} design.ab
}
\examples{
library(agricolaeplotr)
library(agricolae)
trt = c(2,3,4,5,6)
outdesign1 <-design.crd(trt,r=5,serie=2,2543,'Mersenne-Twister')
plot_design_crd(outdesign1,ncols = 13,nrows = 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hello.R
\name{sample_locations}
\alias{sample_locations}
\title{Sample Locations}
\usage{
sample_locations(design, n, plot = TRUE, ...)
}
\arguments{
\item{design}{Your experiment design of plot layouts.}

\item{n}{Number of samples per plot (integer).}

\item{plot}{Logical, indicating whether to visualize the sample locations as a ggplot2-based map.}

\item{...}{further options for `st_sample` and `make_polygons`}
}
\value{
An `sf` object containing the sample locations within each plot.
}
\description{
Returns locations to sample for each plot.
}
\details{
This function takes an experiment design (plot layout) and returns random sample
locations within each plot. The function uses the `sf` package to generate
spatial polygons for the plots and then samples points within each polygon.
Optionally, it can also display the sample locations as a ggplot2-based map.
}
\examples{
library(agricolaeplotr)
library(agricolae)
library(ggplot2)
trt <- c('A', 'B', 'C', 'D')
k <- 3
outdesign <- design.bib(trt, k, serie = 2, seed = 41, kinds = 'Super-Duper')
plot_bib(outdesign)
p <- plot_bib(outdesign)
sample_locations(p, 3, TRUE, projection_output = 25832)

}
